(herald "Unified model (UM)" (bound 20) (limit 8000)
  (algebra diffie-hellman))

(comment "CPSA 3.6.11")
(comment "All input read from tst/eadh_um.scm")
(comment "Step count limited to 8000")
(comment "Strand count bounded at 20")

(defprotocol eadh-um diffie-hellman
  (defrole init
    (vars (l e rndx) (ll ee expt) (self peer name) (n data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l ll)) (exp (gen) (mul e ee)))))
      (recv n))
    (uniq-orig n)
    (uniq-gen e)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
    (neq (ee (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) ll))))
  (defrole resp
    (vars (l e rndx) (ll ee expt) (self peer name) (n data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l ll)) (exp (gen) (mul e ee)))))
      (send n))
    (uniq-gen e)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
    (neq (ee (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) ll))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))))

(defskeleton eadh-um
  (vars (n data) (A B name) (lA eA lB rndx) (ll ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l lA) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l lB))
  (absent (eA (exp (gen) lA)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (B lB) (A lA)) ("privdh-of" (lB B) (lA A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) lA lB)
  (uniq-gen eA lB)
  (uniq-orig n)
  (comment "Explicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul lA ll)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) lB) B (privk B)))))
  (label 0)
  (unrealized (0 0) (0 1) (0 5))
  (origs (n (0 4)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ll ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test (displaced 2 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul ll l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 1)
  (parent 0)
  (unrealized (0 1))
  (origs (n (0 4)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA lB rndx) (ll ee expt) (l rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l lB))
  (defstrand ltx-gen 1 (self A) (l l))
  (precedes ((2 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (A l) (B lB)) ("privdh-of" (l A) (lB B)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) lB l)
  (uniq-gen eA lB l)
  (uniq-orig n)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul ll l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) lB) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A)))))
  (label 2)
  (parent 0)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 0) (0 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test (displaced 2 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) B (privk B)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 3)
  (parent 1)
  (unrealized (0 5))
  (origs (n (0 4)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation encryption-test (displaced 3 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) B (privk B)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A)))))
  (label 4)
  (parent 2)
  (unrealized (0 5))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 1)) ((2 2) (0 3))
    ((2 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 6) n (0 5)
    (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 5)
  (parent 3)
  (unrealized (0 3) (2 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (2 0)) ((1 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))) n (0 5)
    (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))))
  (label 6)
  (parent 3)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1)) ((2 0) (0 0))
    ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 6) n (0 5)
    (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 7)
  (parent 4)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((3 1) (0 5)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))) n (0 5)
    (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))))
  (label 8)
  (parent 4)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee ee))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0)) ((2 2) (0 3))
    ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 ltx-gen 1) (exp (gen) l) (2 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 9)
  (parent 5)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (3 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 10)
  (parent 5)
  (unrealized (0 3) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((2 1) (0 5)) ((3 1) (2 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))))
  (label 11)
  (parent 6)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1)) ((2 0) (0 0))
    ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 12)
  (parent 7)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0)) ((2 0) (0 0))
    ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 13)
  (parent 7)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (4 0)) ((2 0) (0 0))
    ((2 0) (4 0)) ((3 2) (0 3)) ((3 5) (0 5)) ((4 1) (3 1)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 14)
  (parent 7)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (4 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((3 1) (0 5))
    ((4 1) (3 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))))
  (label 15)
  (parent 8)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 16)
  (parent 9)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee eA))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk self) l)
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 3 2 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 17)
  (parent 9)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l) (A self) (B self) (ll l) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 2) (0 3)) ((3 2) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 18)
  (parent 9)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n)))
  (label 19)
  (parent 9)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 0) (0 3)) ((3 0) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 20)
  (parent 9)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (ee expt) (l rndx)
    (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 21)
  (parent 9)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee w expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 22)
  (parent 10)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx) (w expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 23)
  (parent 11)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 24)
  (parent 12)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee eA))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 25)
  (parent 12)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (lA l) (eA eA) (lB l-0) (A A) (B B) (ll l-0) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 2) (0 3)) ((4 2) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 26)
  (parent 12)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 27)
  (parent 12)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 28)
  (parent 12)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 0) (0 3)) ((4 0) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 29)
  (parent 12)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0)) ((3 5) (0 5))
    ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 30)
  (parent 12)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 31)
  (parent 13)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee eA))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 32)
  (parent 13)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (lA l-0) (eA eA) (lB l) (A A) (B B) (ll l) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 2) (0 3)) ((4 2) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 33)
  (parent 13)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 34)
  (parent 13)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 35)
  (parent 13)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 0) (0 3)) ((4 0) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 36)
  (parent 13)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0)) ((3 5) (0 5))
    ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 37)
  (parent 13)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 38)
  (parent 16)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 39)
  (parent 16)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 40)
  (parent 18)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 41)
  (parent 18)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 42)
  (parent 18)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) (mul eA l)) e))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA l)) e))
    (exp (gen) (mul eA (rec e) l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l)) e))
      (send (cat (exp (gen) (mul eA l)) e))))
  (label 43)
  (parent 19)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) (mul (rec e) l)) eA))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l)) eA))
    (exp (gen) (mul eA (rec e) l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l)) eA))
      (send (cat (exp (gen) (mul (rec e) l)) eA))))
  (label 44)
  (parent 19)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-0))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0))))
  (label 45)
  (parent 20)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 46)
  (parent 20)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 47)
  (parent 20)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 48)
  (parent 21)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 49)
  (parent 21)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 50)
  (parent 21)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 51)
  (parent 21)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w l) (one)) ((mul w l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (send n))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 52)
  (parent 21)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 53)
  (parent 21)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l eA e rndx) (ee expt) (l-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-0)))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-0)
  (uniq-gen eA e l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 54)
  (parent 22)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l e rndx) (ee expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e e-0) (ll l-0)
    (ee (mul e ee (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-0))))
  (fn-of ("principal-of" (B l-0) (self l))
    ("privdh-of" (l-0 B) (l self)))
  (neq (ee (one)) ((mul e ee (rec e-0)) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e ee (rec e-0))))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))))
  (label 55)
  (parent 22)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 peer-0 name) (l eA e rndx)
    (ee expt) (l-0 rndx) (ll expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-1) (e eA) (ll l-1)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-1 l-1)) l))
  (deflistener (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-1))) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-1)
  (precur (4 0))
  (uniq-gen eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-1)) l))
      (send (cat (exp (gen) (mul l-1 l-1)) l)))
    ((recv (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 56)
  (parent 22)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e rndx) (ee expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-0)))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 57)
  (parent 22)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 58)
  (parent 23)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B name) (ee expt) (l e rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e e) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
  (deflistener (cat (exp (gen) e) (mul l l (rec e))))
  (precedes ((0 2) (4 0)) ((1 0) (0 0)) ((2 1) (0 5)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) ee))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
    ((recv (cat (exp (gen) e) (mul l l (rec e))))
      (send (cat (exp (gen) e) (mul l l (rec e))))))
  (label 59)
  (parent 23)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (eA rndx) (ee expt) (l rndx)
    (ll expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) e) (mul l-0 l-0 (rec e))))
  (defstrand resp 3 (self self) (peer peer) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq (ee (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) e) (mul l-0 l-0 (rec e))))
      (send (cat (exp (gen) e) (mul l-0 l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e))))
  (label 60)
  (parent 23)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 61)
  (parent 23)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 62)
  (parent 24)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 63)
  (parent 24)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 64)
  (parent 26)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 65)
  (parent 26)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 66)
  (parent 26)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 67)
  (parent 27)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 68)
  (parent 27)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 69)
  (parent 28)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 70)
  (parent 28)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 71)
  (parent 29)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 72)
  (parent 29)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 73)
  (parent 29)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 74)
  (parent 30)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 75)
  (parent 30)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 76)
  (parent 30)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 77)
  (parent 30)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 78)
  (parent 30)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 79)
  (parent 30)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 80)
  (parent 30)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 81)
  (parent 31)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 82)
  (parent 31)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 83)
  (parent 33)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 84)
  (parent 33)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 85)
  (parent 33)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 86)
  (parent 34)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 87)
  (parent 34)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 88)
  (parent 35)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 89)
  (parent 35)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 90)
  (parent 36)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 91)
  (parent 36)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 92)
  (parent 36)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 93)
  (parent 37)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 94)
  (parent 37)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 95)
  (parent 37)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 96)
  (parent 37)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 97)
  (parent 37)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 98)
  (parent 37)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 99)
  (parent 37)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 3 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 100)
  (parent 45)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 101)
  (parent 48)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 102)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 103)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (gen) (mul (rec eA) e l)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul (rec eA) e l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l)))
      (send (cat (gen) (mul (rec eA) e l)))))
  (label 104)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 105)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 106)
  (parent 48)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 107)
  (parent 49)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 108)
  (parent 49)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 109)
  (parent 49)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul e (rec e-0) l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))))
  (label 110)
  (parent 49)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 111)
  (parent 49)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 112)
  (parent 49)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA w)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 113)
  (parent 50)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l) (A self) (B self) (ll l)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-1)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen e e-0 l-0 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 114)
  (parent 51)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 115)
  (parent 51)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA e l-0 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 4 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 116)
  (parent 51)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)) ((5 2) (2 3)) ((5 2) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 117)
  (parent 51)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 118)
  (parent 51)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)) ((5 0) (2 3)) ((5 0) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 119)
  (parent 51)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 120)
  (parent 51)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 121)
  (parent 52)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 122)
  (parent 52)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 123)
  (parent 52)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul (rec eA) e l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e)))
      (send (cat (exp (gen) l) (mul (rec eA) e)))))
  (label 124)
  (parent 52)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 125)
  (parent 52)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w l) (one)) ((mul w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (send n))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))))
  (label 126)
  (parent 52)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 127)
  (parent 53)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 128)
  (parent 53)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)) ((5 2) (2 3)) ((5 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 129)
  (parent 53)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 130)
  (parent 53)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 131)
  (parent 53)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)) ((5 0) (2 3)) ((5 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 132)
  (parent 53)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 133)
  (parent 53)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 134)
  (parent 71)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 135)
  (parent 74)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 136)
  (parent 74)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 137)
  (parent 74)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 138)
  (parent 74)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 139)
  (parent 74)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 140)
  (parent 74)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 141)
  (parent 74)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 142)
  (parent 75)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 143)
  (parent 75)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 144)
  (parent 75)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 145)
  (parent 75)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 146)
  (parent 75)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 147)
  (parent 75)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 148)
  (parent 75)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 149)
  (parent 76)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l-0) (A A) (B B) (ll l-0)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 150)
  (parent 77)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 151)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e l-1 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 152)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 153)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 154)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 155)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 156)
  (parent 77)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 157)
  (parent 77)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 158)
  (parent 78)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 159)
  (parent 78)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 160)
  (parent 78)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 161)
  (parent 78)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 162)
  (parent 78)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 163)
  (parent 78)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 164)
  (parent 78)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 165)
  (parent 79)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 166)
  (parent 79)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 167)
  (parent 79)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 168)
  (parent 79)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 169)
  (parent 79)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 170)
  (parent 79)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 171)
  (parent 79)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 172)
  (parent 80)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 173)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 174)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 175)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 176)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 177)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 178)
  (parent 80)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 179)
  (parent 80)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 180)
  (parent 90)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 181)
  (parent 93)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 182)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 183)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 184)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 185)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 186)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 187)
  (parent 93)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 188)
  (parent 94)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 189)
  (parent 94)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 190)
  (parent 94)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 191)
  (parent 94)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 192)
  (parent 94)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 193)
  (parent 94)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 194)
  (parent 94)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 195)
  (parent 95)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l-0) (eA eA) (lB l) (A A) (B B) (ll l)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 196)
  (parent 96)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 197)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e l-1 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 198)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 199)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 200)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 201)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 202)
  (parent 96)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 203)
  (parent 96)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 204)
  (parent 97)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 205)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 206)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 207)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 208)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 209)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 210)
  (parent 97)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 211)
  (parent 98)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 212)
  (parent 98)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 213)
  (parent 98)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 214)
  (parent 98)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 215)
  (parent 98)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 216)
  (parent 98)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 217)
  (parent 98)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-1)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 218)
  (parent 99)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 219)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 220)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 221)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 222)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 223)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 224)
  (parent 99)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 225)
  (parent 99)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 226)
  (parent 100)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 227)
  (parent 100)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 228)
  (parent 106)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 229)
  (parent 106)
  (unrealized (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 230)
  (parent 106)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 231)
  (parent 106)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (gen) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l)))
      (send (cat (gen) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 232)
  (parent 106)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 233)
  (parent 106)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 234)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 235)
  (parent 112)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 236)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 237)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul e (rec e-0) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 238)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 239)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 240)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-1)
    (ll l-0) (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 241)
  (parent 120)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 242)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 243)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 244)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 245)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 246)
  (parent 120)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l) w))
      (l-0 l) (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 247)
  (parent 126)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 248)
  (parent 126)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 249)
  (parent 126)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 250)
  (parent 126)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 251)
  (parent 126)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 252)
  (parent 126)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 253)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 254)
  (parent 133)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 255)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 256)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 257)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 258)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 259)
  (parent 133)
  (unrealized (0 3) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 260)
  (parent 134)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 261)
  (parent 134)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 262)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 263)
  (parent 141)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 264)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 265)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 266)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 267)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 268)
  (parent 141)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 269)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 270)
  (parent 148)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 271)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 272)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 273)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 274)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 275)
  (parent 148)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 276)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 277)
  (parent 157)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 278)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 279)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 280)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 281)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 282)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 283)
  (parent 157)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 284)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 285)
  (parent 164)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 286)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 287)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 288)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 289)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 290)
  (parent 164)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 291)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 292)
  (parent 171)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 293)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 294)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 295)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 296)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 297)
  (parent 171)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 298)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 299)
  (parent 179)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 300)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 301)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 302)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 303)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 304)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 305)
  (parent 179)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 306)
  (parent 180)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 307)
  (parent 180)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 308)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 309)
  (parent 187)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 310)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 311)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 312)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 313)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 314)
  (parent 187)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 315)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 316)
  (parent 194)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 317)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 318)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 319)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 320)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 321)
  (parent 194)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 322)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 323)
  (parent 203)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 324)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 325)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 326)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 327)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 328)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 329)
  (parent 203)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 330)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 331)
  (parent 210)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 332)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 333)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 334)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 335)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 336)
  (parent 210)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 337)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 338)
  (parent 217)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 339)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 340)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 341)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 342)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 343)
  (parent 217)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 344)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-1)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 345)
  (parent 225)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 346)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 347)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 348)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 349)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 350)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 351)
  (parent 225)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 352)
  (parent 228)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l e eA)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 353)
  (parent 228)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 354)
  (parent 228)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 355)
  (parent 229)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 356)
  (parent 229)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 357)
  (parent 229)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 358)
  (parent 230)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 359)
  (parent 230)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 360)
  (parent 230)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 361)
  (parent 231)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 362)
  (parent 231)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 363)
  (parent 231)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 364)
  (parent 231)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 365)
  (parent 232)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (gen) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l))) (send (cat (gen) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 366)
  (parent 232)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))))
  (label 367)
  (parent 232)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 368)
  (parent 232)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 369)
  (parent 233)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 370)
  (parent 233)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 371)
  (parent 233)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 372)
  (parent 233)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 373)
  (parent 233)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 374)
  (parent 234)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 375)
  (parent 234)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 376)
  (parent 234)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 377)
  (parent 234)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 378)
  (parent 235)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 379)
  (parent 235)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 380)
  (parent 235)
  (unrealized (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 381)
  (parent 235)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 382)
  (parent 236)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e e (rec e-0) l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 383)
  (parent 236)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 384)
  (parent 236)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 385)
  (parent 236)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 386)
  (parent 237)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 387)
  (parent 237)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 388)
  (parent 237)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-1) (ll l)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 389)
  (parent 237)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 390)
  (parent 237)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l)))
      (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))))
  (label 391)
  (parent 238)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 392)
  (parent 238)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e-0)) (one)) ((mul (rec e) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l))
    (mul (rec e) (rec e) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e-0))))
      (send (cat (exp (gen) l) (mul w (rec e-0))))))
  (label 393)
  (parent 238)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w l)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l e-0 e-0) (one)) ((mul e w l e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l)))
      (send (cat (exp (gen) e-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w e-0 e-0)))
      (send (cat (exp (gen) l) (mul w e-0 e-0)))))
  (label 394)
  (parent 238)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 395)
  (parent 238)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 396)
  (parent 239)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 397)
  (parent 239)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 398)
  (parent 239)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 399)
  (parent 239)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 400)
  (parent 239)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 401)
  (parent 239)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 402)
  (parent 240)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-1)))
      (send (cat (gen) (mul eA (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 403)
  (parent 240)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 404)
  (parent 240)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 405)
  (parent 240)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 406)
  (parent 240)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (l-0 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee e-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e-1 (exp (gen) l)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 407)
  (parent 241)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (l-0 e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee (mul e-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) e-1 l-1)))
  (deflistener (cat (exp (gen) e-1) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e-1 (exp (gen) l)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul e (rec e-0) e-1 l-1) (one)) ((mul e-1 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-1)) (send (cat (exp (gen) e-1) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 408)
  (parent 241)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll l-0) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 409)
  (parent 241)
  (unrealized (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 410)
  (parent 241)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (l-0 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 411)
  (parent 242)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (l-0 eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul eA e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 412)
  (parent 242)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 413)
  (parent 242)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 414)
  (parent 242)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 415)
  (parent 242)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 416)
  (parent 243)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e eA e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e eA (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e eA (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 417)
  (parent 243)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 418)
  (parent 243)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA rndx) (w expt)
    (l-0 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 419)
  (parent 243)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 420)
  (parent 244)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 421)
  (parent 244)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (e e-0 rndx) (w expt)
    (e-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 422)
  (parent 244)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 423)
  (parent 244)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 424)
  (parent 244)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 425)
  (parent 244)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-0))) (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))))
  (label 426)
  (parent 245)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (eA e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 427)
  (parent 245)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-0))
    (mul e (rec e-0) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 428)
  (parent 245)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-0))
    (mul (rec eA) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 429)
  (parent 245)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 e-0) (one)) ((mul (rec eA) e w l-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0)))))
  (label 430)
  (parent 245)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 431)
  (parent 245)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 432)
  (parent 246)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 433)
  (parent 246)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 434)
  (parent 246)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 435)
  (parent 246)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 436)
  (parent 246)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 437)
  (parent 246)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 438)
  (parent 246)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-0 l) (w (mul eA (rec e) l))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul eA (rec e) l)))
      (send (cat (gen) (mul eA (rec e) l)))))
  (label 439)
  (parent 247)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l l-0)))
      (send (cat (gen) (mul eA (rec e) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 440)
  (parent 247)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e (rec l) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 441)
  (parent 247)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 442)
  (parent 247)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l))) (send (cat (gen) (mul w l)))))
  (label 443)
  (parent 247)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul e (rec e-0) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l)) (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l)))
      (send (cat (exp (gen) e) (mul (rec e-0) l)))))
  (label 444)
  (parent 248)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 445)
  (parent 248)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 446)
  (parent 248)
  (unrealized (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 l) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l)))
      (send (cat (exp (gen) e-0) (mul w l)))))
  (label 447)
  (parent 248)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA l (rec e)) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l (rec e) (rec e))) (l-0 l)
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 448)
  (parent 249)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA l (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 449)
  (parent 249)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 450)
  (parent 249)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e l) (one)) ((mul (rec eA) w e e l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 451)
  (parent 249)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 452)
  (parent 249)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-1 l-0)
      (w (mul eA (rec e) l-0 (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 453)
  (parent 250)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (eA e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 454)
  (parent 250)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e l-0 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul e (rec l-0) w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w e-0))
    (mul e (rec l-0) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 455)
  (parent 250)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA l-0 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-0) w e))
    (mul (rec eA) (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 456)
  (parent 250)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (l-0 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 l-0) (one)) ((mul (rec eA) e w e-0 l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 l-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 l-0))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 457)
  (parent 250)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) e (rec l-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 458)
  (parent 250)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul eA (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul eA (rec e))))
      (send (cat (exp (gen) l) (mul eA (rec e))))))
  (label 459)
  (parent 251)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l eA (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 460)
  (parent 251)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 461)
  (parent 251)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))))
  (label 462)
  (parent 251)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 463)
  (parent 252)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 464)
  (parent 252)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 465)
  (parent 252)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 466)
  (parent 252)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 467)
  (parent 252)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 468)
  (parent 252)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 469)
  (parent 252)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 470)
  (parent 253)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-0)))
      (send (cat (gen) (mul eA (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 471)
  (parent 253)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 472)
  (parent 253)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 473)
  (parent 253)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 474)
  (parent 253)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 475)
  (parent 253)
  (unrealized (0 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-1 l-0) (w (mul (rec e-0) l-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 476)
  (parent 254)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 477)
  (parent 254)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self l) (self-0 l-0))
    ("privdh-of" (l-1 self-1) (l self) (l-0 self-0)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 478)
  (parent 254)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 479)
  (parent 254)
  (unrealized (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 480)
  (parent 254)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul eA l-0 (rec e) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 481)
  (parent 255)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 482)
  (parent 255)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self l) (self-0 l-0))
    ("privdh-of" (l-1 self-1) (l self) (l-0 self-0)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 483)
  (parent 255)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 484)
  (parent 255)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 485)
  (parent 255)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 486)
  (parent 255)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 487)
  (parent 256)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 488)
  (parent 256)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 rndx) (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 489)
  (parent 256)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 490)
  (parent 256)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 491)
  (parent 256)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 492)
  (parent 256)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 493)
  (parent 256)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 494)
  (parent 257)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 495)
  (parent 257)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 496)
  (parent 257)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 497)
  (parent 257)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 498)
  (parent 257)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 499)
  (parent 257)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 500)
  (parent 257)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 501)
  (parent 258)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 502)
  (parent 258)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 503)
  (parent 258)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 504)
  (parent 258)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 505)
  (parent 258)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 506)
  (parent 259)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 507)
  (parent 259)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 508)
  (parent 259)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 509)
  (parent 259)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 510)
  (parent 259)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 511)
  (parent 259)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))))
  (label 512)
  (parent 259)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 513)
  (parent 259)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 514)
  (parent 262)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 515)
  (parent 262)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 516)
  (parent 262)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 517)
  (parent 263)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 518)
  (parent 263)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 519)
  (parent 263)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 520)
  (parent 264)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 521)
  (parent 264)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 522)
  (parent 264)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 523)
  (parent 265)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 524)
  (parent 265)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 525)
  (parent 265)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 526)
  (parent 265)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 527)
  (parent 266)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 528)
  (parent 266)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 529)
  (parent 266)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 530)
  (parent 266)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 531)
  (parent 267)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 532)
  (parent 267)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 533)
  (parent 267)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 534)
  (parent 267)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 535)
  (parent 268)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 536)
  (parent 268)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 537)
  (parent 268)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 538)
  (parent 268)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 539)
  (parent 268)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 540)
  (parent 269)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 541)
  (parent 269)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 542)
  (parent 269)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 543)
  (parent 269)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 544)
  (parent 270)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 545)
  (parent 270)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 546)
  (parent 270)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 547)
  (parent 270)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 548)
  (parent 271)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 549)
  (parent 271)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 550)
  (parent 271)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 551)
  (parent 271)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 552)
  (parent 272)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 553)
  (parent 272)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 554)
  (parent 272)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 555)
  (parent 272)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 556)
  (parent 272)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 557)
  (parent 273)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 558)
  (parent 273)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 559)
  (parent 273)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 560)
  (parent 273)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 561)
  (parent 273)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 562)
  (parent 274)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 563)
  (parent 274)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 564)
  (parent 274)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 565)
  (parent 274)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 566)
  (parent 274)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 567)
  (parent 275)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 568)
  (parent 275)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 569)
  (parent 275)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 570)
  (parent 275)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 571)
  (parent 275)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 572)
  (parent 275)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 573)
  (parent 276)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 574)
  (parent 276)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 575)
  (parent 276)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 576)
  (parent 276)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 577)
  (parent 276)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 578)
  (parent 277)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 579)
  (parent 277)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-1) (ll l-1)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 580)
  (parent 277)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 581)
  (parent 277)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 582)
  (parent 278)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 583)
  (parent 278)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 584)
  (parent 278)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 585)
  (parent 278)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 586)
  (parent 278)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 587)
  (parent 279)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 588)
  (parent 279)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 589)
  (parent 279)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 590)
  (parent 279)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 591)
  (parent 280)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 592)
  (parent 280)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 593)
  (parent 280)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 594)
  (parent 280)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 595)
  (parent 280)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 596)
  (parent 280)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 597)
  (parent 281)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 598)
  (parent 281)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 599)
  (parent 281)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 600)
  (parent 281)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 601)
  (parent 281)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 602)
  (parent 281)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 603)
  (parent 282)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 604)
  (parent 282)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 605)
  (parent 282)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 606)
  (parent 282)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 607)
  (parent 282)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 608)
  (parent 282)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 609)
  (parent 283)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 610)
  (parent 283)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 611)
  (parent 283)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 612)
  (parent 283)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 613)
  (parent 283)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 614)
  (parent 283)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 615)
  (parent 283)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 616)
  (parent 284)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 617)
  (parent 284)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 618)
  (parent 284)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 619)
  (parent 284)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 620)
  (parent 284)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 621)
  (parent 285)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 622)
  (parent 285)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 623)
  (parent 285)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 624)
  (parent 285)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 625)
  (parent 286)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 626)
  (parent 286)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 627)
  (parent 286)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 628)
  (parent 286)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 629)
  (parent 286)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 630)
  (parent 287)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 631)
  (parent 287)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 632)
  (parent 287)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 633)
  (parent 287)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 634)
  (parent 287)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 635)
  (parent 287)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 636)
  (parent 288)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 637)
  (parent 288)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 638)
  (parent 288)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 639)
  (parent 288)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 640)
  (parent 289)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 641)
  (parent 289)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 642)
  (parent 289)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 643)
  (parent 289)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 644)
  (parent 289)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 645)
  (parent 289)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 646)
  (parent 290)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 647)
  (parent 290)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 648)
  (parent 290)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 649)
  (parent 290)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 650)
  (parent 290)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 651)
  (parent 290)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 652)
  (parent 290)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 653)
  (parent 291)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 654)
  (parent 291)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 655)
  (parent 291)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 656)
  (parent 291)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 657)
  (parent 291)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 658)
  (parent 292)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 659)
  (parent 292)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 660)
  (parent 292)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 661)
  (parent 292)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 662)
  (parent 293)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 663)
  (parent 293)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 664)
  (parent 293)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 665)
  (parent 293)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 666)
  (parent 293)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 667)
  (parent 294)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 668)
  (parent 294)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 669)
  (parent 294)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 670)
  (parent 294)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 671)
  (parent 294)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 672)
  (parent 294)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 673)
  (parent 295)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 674)
  (parent 295)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 675)
  (parent 295)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 676)
  (parent 295)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 677)
  (parent 295)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 678)
  (parent 295)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 679)
  (parent 296)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 680)
  (parent 296)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 681)
  (parent 296)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 682)
  (parent 296)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 683)
  (parent 297)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 684)
  (parent 297)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 685)
  (parent 297)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 686)
  (parent 297)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 687)
  (parent 297)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 688)
  (parent 297)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 689)
  (parent 297)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 690)
  (parent 298)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 691)
  (parent 298)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 692)
  (parent 298)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 693)
  (parent 298)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 694)
  (parent 298)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 695)
  (parent 298)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-2 l-1) (w (mul (rec e-0) l-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 696)
  (parent 299)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 697)
  (parent 299)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 698)
  (parent 299)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 699)
  (parent 299)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 700)
  (parent 299)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 701)
  (parent 300)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 702)
  (parent 300)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 703)
  (parent 300)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 704)
  (parent 300)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 705)
  (parent 300)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 706)
  (parent 300)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 707)
  (parent 301)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 708)
  (parent 301)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 709)
  (parent 301)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 710)
  (parent 301)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 711)
  (parent 301)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 712)
  (parent 301)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 713)
  (parent 301)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 714)
  (parent 302)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 715)
  (parent 302)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 716)
  (parent 302)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 717)
  (parent 302)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 718)
  (parent 302)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 719)
  (parent 302)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 720)
  (parent 302)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 721)
  (parent 303)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 722)
  (parent 303)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 723)
  (parent 303)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 724)
  (parent 303)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 725)
  (parent 303)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 726)
  (parent 303)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 727)
  (parent 303)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 728)
  (parent 304)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 729)
  (parent 304)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 730)
  (parent 304)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 731)
  (parent 304)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 732)
  (parent 304)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 733)
  (parent 305)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 734)
  (parent 305)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 735)
  (parent 305)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 736)
  (parent 305)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 737)
  (parent 305)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 738)
  (parent 305)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 739)
  (parent 305)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 740)
  (parent 305)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 741)
  (parent 308)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 742)
  (parent 308)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 743)
  (parent 308)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 744)
  (parent 309)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 745)
  (parent 309)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 746)
  (parent 309)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 747)
  (parent 310)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 748)
  (parent 310)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 749)
  (parent 310)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 750)
  (parent 311)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 751)
  (parent 311)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 752)
  (parent 311)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 753)
  (parent 311)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 754)
  (parent 312)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 755)
  (parent 312)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 756)
  (parent 312)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 757)
  (parent 312)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 758)
  (parent 313)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 759)
  (parent 313)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 760)
  (parent 313)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 761)
  (parent 313)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 762)
  (parent 314)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 763)
  (parent 314)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 764)
  (parent 314)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 765)
  (parent 314)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 766)
  (parent 314)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 767)
  (parent 315)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 768)
  (parent 315)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 769)
  (parent 315)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 770)
  (parent 315)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 771)
  (parent 316)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 772)
  (parent 316)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 773)
  (parent 316)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 774)
  (parent 316)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l) (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 775)
  (parent 317)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 776)
  (parent 317)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 777)
  (parent 317)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 778)
  (parent 317)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 779)
  (parent 318)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 780)
  (parent 318)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 781)
  (parent 318)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 782)
  (parent 318)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 783)
  (parent 318)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 784)
  (parent 319)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 785)
  (parent 319)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 786)
  (parent 319)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 787)
  (parent 319)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 788)
  (parent 319)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 789)
  (parent 320)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 790)
  (parent 320)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 791)
  (parent 320)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 792)
  (parent 320)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 793)
  (parent 320)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 794)
  (parent 321)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 795)
  (parent 321)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 796)
  (parent 321)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 797)
  (parent 321)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 798)
  (parent 321)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 799)
  (parent 321)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 800)
  (parent 322)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 801)
  (parent 322)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 802)
  (parent 322)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 803)
  (parent 322)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 804)
  (parent 322)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 805)
  (parent 323)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 806)
  (parent 323)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 807)
  (parent 323)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 808)
  (parent 323)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 809)
  (parent 324)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (B l-0) (self l) (A l-1))
    ("privdh-of" (l-2 self-0) (l-0 B) (l self) (l-1 A)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 810)
  (parent 324)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 811)
  (parent 324)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 812)
  (parent 324)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 813)
  (parent 324)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 814)
  (parent 325)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 815)
  (parent 325)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 816)
  (parent 325)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 817)
  (parent 325)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 818)
  (parent 326)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 819)
  (parent 326)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 820)
  (parent 326)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 821)
  (parent 326)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 822)
  (parent 326)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 823)
  (parent 326)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 824)
  (parent 327)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 825)
  (parent 327)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 826)
  (parent 327)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 827)
  (parent 327)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 828)
  (parent 327)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 829)
  (parent 327)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 830)
  (parent 328)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 831)
  (parent 328)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 832)
  (parent 328)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 833)
  (parent 328)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 834)
  (parent 328)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 835)
  (parent 328)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 836)
  (parent 329)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 837)
  (parent 329)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 838)
  (parent 329)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 839)
  (parent 329)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 840)
  (parent 329)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 841)
  (parent 329)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 842)
  (parent 329)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 843)
  (parent 330)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 844)
  (parent 330)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 845)
  (parent 330)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 846)
  (parent 330)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 847)
  (parent 330)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 848)
  (parent 331)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 849)
  (parent 331)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 850)
  (parent 331)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 851)
  (parent 331)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 852)
  (parent 332)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 853)
  (parent 332)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 854)
  (parent 332)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 855)
  (parent 332)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 856)
  (parent 332)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 857)
  (parent 333)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 858)
  (parent 333)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 859)
  (parent 333)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 860)
  (parent 333)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 861)
  (parent 333)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 862)
  (parent 333)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 863)
  (parent 334)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 864)
  (parent 334)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 865)
  (parent 334)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 866)
  (parent 334)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 867)
  (parent 335)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 868)
  (parent 335)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 869)
  (parent 335)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 870)
  (parent 335)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 871)
  (parent 335)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 872)
  (parent 335)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 873)
  (parent 336)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 874)
  (parent 336)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 875)
  (parent 336)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 876)
  (parent 336)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 877)
  (parent 336)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 878)
  (parent 336)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 879)
  (parent 336)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 880)
  (parent 337)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 881)
  (parent 337)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 882)
  (parent 337)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 883)
  (parent 337)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 884)
  (parent 337)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 885)
  (parent 338)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 886)
  (parent 338)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 887)
  (parent 338)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 888)
  (parent 338)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 889)
  (parent 339)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 890)
  (parent 339)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 891)
  (parent 339)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 892)
  (parent 339)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 893)
  (parent 339)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 894)
  (parent 340)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 895)
  (parent 340)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 896)
  (parent 340)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 897)
  (parent 340)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 898)
  (parent 340)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 899)
  (parent 340)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 900)
  (parent 341)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 901)
  (parent 341)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 902)
  (parent 341)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 903)
  (parent 341)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 904)
  (parent 341)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 905)
  (parent 341)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 906)
  (parent 342)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 907)
  (parent 342)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 908)
  (parent 342)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 909)
  (parent 342)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 910)
  (parent 343)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 911)
  (parent 343)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 912)
  (parent 343)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 913)
  (parent 343)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 914)
  (parent 343)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 915)
  (parent 343)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 916)
  (parent 343)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 917)
  (parent 344)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 918)
  (parent 344)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 919)
  (parent 344)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 920)
  (parent 344)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 921)
  (parent 344)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 922)
  (parent 344)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-2 l-1) (w (mul (rec e-0) l-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 923)
  (parent 345)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 924)
  (parent 345)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 925)
  (parent 345)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 926)
  (parent 345)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 927)
  (parent 345)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 928)
  (parent 346)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 929)
  (parent 346)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (B l) (self l-1) (A l-0))
    ("privdh-of" (l-2 self-0) (l B) (l-1 self) (l-0 A)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 930)
  (parent 346)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 931)
  (parent 346)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 932)
  (parent 346)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 933)
  (parent 346)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 934)
  (parent 347)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 935)
  (parent 347)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 936)
  (parent 347)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 937)
  (parent 347)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 938)
  (parent 347)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 939)
  (parent 347)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 940)
  (parent 347)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 941)
  (parent 348)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 942)
  (parent 348)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 943)
  (parent 348)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 944)
  (parent 348)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 945)
  (parent 348)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 946)
  (parent 348)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 947)
  (parent 348)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 948)
  (parent 349)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 949)
  (parent 349)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 950)
  (parent 349)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 951)
  (parent 349)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 952)
  (parent 349)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 953)
  (parent 349)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 954)
  (parent 349)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 955)
  (parent 350)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 956)
  (parent 350)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 957)
  (parent 350)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 958)
  (parent 350)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 959)
  (parent 350)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 960)
  (parent 351)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 961)
  (parent 351)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 962)
  (parent 351)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 963)
  (parent 351)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 964)
  (parent 351)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 965)
  (parent 351)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 966)
  (parent 351)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 967)
  (parent 351)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l e eA)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 968)
  (parent 353)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 969)
  (parent 354)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 970)
  (parent 356)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 971)
  (parent 359)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 972)
  (parent 360)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 973)
  (parent 362)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 974)
  (parent 363)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 975)
  (parent 364)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (gen) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l))) (send (cat (gen) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 976)
  (parent 366)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))) ((recv e) (send e)))
  (label 977)
  (parent 367)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 978)
  (parent 371)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 979)
  (parent 372)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 980)
  (parent 373)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 981)
  (parent 376)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 982)
  (parent 377)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 983)
  (parent 380)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 984)
  (parent 381)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 985)
  (parent 384)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 986)
  (parent 385)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 987)
  (parent 388)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-1) (ll l)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 988)
  (parent 389)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 989)
  (parent 390)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e-0)) (one)) ((mul (rec e) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e-0))))
      (send (cat (exp (gen) l) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 990)
  (parent 393)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w l)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l e-0 e-0) (one)) ((mul e w l e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l)))
      (send (cat (exp (gen) e-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w e-0 e-0)))
      (send (cat (exp (gen) l) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 991)
  (parent 394)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 992)
  (parent 399)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 993)
  (parent 400)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 994)
  (parent 401)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 995)
  (parent 404)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 996)
  (parent 405)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 997)
  (parent 406)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll l-0) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 998)
  (parent 409)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 999)
  (parent 410)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1000)
  (parent 413)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1001)
  (parent 414)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1002)
  (parent 415)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1003)
  (parent 418)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA rndx) (w expt)
    (l-0 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1004)
  (parent 419)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (e e-0 rndx) (w expt)
    (e-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 1005)
  (parent 422)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1006)
  (parent 423)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1007)
  (parent 424)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 1008)
  (parent 425)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec e-0) w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1009)
  (parent 428)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1010)
  (parent 429)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 e-0) (one)) ((mul (rec eA) e w l-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1011)
  (parent 430)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 1012)
  (parent 435)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 1013)
  (parent 436)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 1014)
  (parent 437)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 1015)
  (parent 438)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1016)
  (parent 441)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1017)
  (parent 442)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1018)
  (parent 446)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1019)
  (parent 450)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1020)
  (parent 452)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e l-0 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e (rec l-0) w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1021)
  (parent 455)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA l-0 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1022)
  (parent 456)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) e (rec l-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1023)
  (parent 458)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 1024)
  (parent 461)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))) ((recv e) (send e)))
  (label 1025)
  (parent 462)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1026)
  (parent 466)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1027)
  (parent 467)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 1028)
  (parent 469)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1029)
  (parent 473)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1030)
  (parent 474)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1031)
  (parent 475)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1032)
  (parent 475)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1033)
  (parent 479)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) e))
  (deflistener (cat (exp (gen) e-0) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 e) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e-0 l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) e)) (send (cat (exp (gen) l-0) e)))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1034)
  (parent 480)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1035)
  (parent 480)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1036)
  (parent 484)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0) (one)) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1037)
  (parent 485)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1038)
  (parent 485)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1039)
  (parent 486)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1040)
  (parent 490)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1041)
  (parent 491)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e-0 l-1) (one)) ((mul (rec eA) e e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1042)
  (parent 492)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (7 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1)))
  (label 1043)
  (parent 492)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1044)
  (parent 493)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1045)
  (parent 497)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1046)
  (parent 498)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l l-0) (one)) ((mul (rec eA) e l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1047)
  (parent 499)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1048)
  (parent 499)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1049)
  (parent 504)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1050)
  (parent 505)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 1051)
  (parent 510)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 1052)
  (parent 511)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1053)
  (parent 512)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (7 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 1054)
  (parent 512)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 1055)
  (parent 513)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1056)
  (parent 515)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1057)
  (parent 516)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1058)
  (parent 518)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1059)
  (parent 521)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1060)
  (parent 522)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1061)
  (parent 524)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1062)
  (parent 525)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1063)
  (parent 526)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1064)
  (parent 528)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1065)
  (parent 529)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1066)
  (parent 532)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1067)
  (parent 533)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1068)
  (parent 537)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1069)
  (parent 538)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1070)
  (parent 539)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 1071)
  (parent 542)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 1072)
  (parent 543)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1073)
  (parent 546)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1074)
  (parent 547)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1075)
  (parent 550)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1076)
  (parent 551)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1077)
  (parent 554)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1078)
  (parent 555)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 1079)
  (parent 556)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1080)
  (parent 559)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1081)
  (parent 560)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1082)
  (parent 564)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1083)
  (parent 565)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 1084)
  (parent 570)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 1085)
  (parent 571)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1086)
  (parent 572)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1087)
  (parent 575)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 1088)
  (parent 576)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 1089)
  (parent 577)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-1) (ll l-1)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 1090)
  (parent 580)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 1091)
  (parent 581)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1092)
  (parent 584)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1093)
  (parent 585)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1094)
  (parent 586)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1095)
  (parent 589)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1096)
  (parent 590)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 1097)
  (parent 593)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1098)
  (parent 594)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1099)
  (parent 595)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 1100)
  (parent 596)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1101)
  (parent 599)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1102)
  (parent 600)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1103)
  (parent 601)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1104)
  (parent 605)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1105)
  (parent 606)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1106)
  (parent 607)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1107)
  (parent 612)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 1108)
  (parent 613)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 1109)
  (parent 614)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1110)
  (parent 615)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1111)
  (parent 618)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1112)
  (parent 619)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1113)
  (parent 623)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1114)
  (parent 627)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1115)
  (parent 629)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1116)
  (parent 632)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1117)
  (parent 633)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1118)
  (parent 635)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1119)
  (parent 638)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1120)
  (parent 639)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1121)
  (parent 642)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1122)
  (parent 643)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1123)
  (parent 649)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1124)
  (parent 650)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1125)
  (parent 652)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1126)
  (parent 655)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1127)
  (parent 656)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1128)
  (parent 660)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1129)
  (parent 664)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1130)
  (parent 666)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1131)
  (parent 669)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1132)
  (parent 670)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1133)
  (parent 672)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1134)
  (parent 675)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1135)
  (parent 676)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1136)
  (parent 681)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1137)
  (parent 682)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1138)
  (parent 686)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1139)
  (parent 687)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1140)
  (parent 689)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1141)
  (parent 693)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1142)
  (parent 694)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1143)
  (parent 695)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1144)
  (parent 695)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1145)
  (parent 699)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) e))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 e) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e-0 l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) e)) (send (cat (exp (gen) l-1) e)))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1146)
  (parent 700)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1147)
  (parent 700)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1148)
  (parent 704)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1) (one)) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1149)
  (parent 705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1150)
  (parent 705)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1151)
  (parent 706)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1152)
  (parent 710)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1153)
  (parent 711)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul e-0 l-2) (one)) ((mul (rec eA) e e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1154)
  (parent 712)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2)))
  (label 1155)
  (parent 712)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1156)
  (parent 713)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1157)
  (parent 717)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1158)
  (parent 718)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1159)
  (parent 719)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1160)
  (parent 719)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1161)
  (parent 724)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1162)
  (parent 725)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1163)
  (parent 726)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1164)
  (parent 726)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1165)
  (parent 731)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1166)
  (parent 732)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1167)
  (parent 737)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1168)
  (parent 738)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1169)
  (parent 739)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1170)
  (parent 739)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1171)
  (parent 740)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1172)
  (parent 742)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1173)
  (parent 743)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1174)
  (parent 745)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1175)
  (parent 748)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1176)
  (parent 749)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1177)
  (parent 751)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1178)
  (parent 752)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1179)
  (parent 753)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1180)
  (parent 755)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1181)
  (parent 756)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1182)
  (parent 759)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1183)
  (parent 760)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1184)
  (parent 764)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1185)
  (parent 765)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1186)
  (parent 766)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 1187)
  (parent 769)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 1188)
  (parent 770)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1189)
  (parent 773)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1190)
  (parent 774)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1191)
  (parent 777)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1192)
  (parent 778)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1193)
  (parent 781)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1194)
  (parent 782)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 1195)
  (parent 783)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1196)
  (parent 786)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1197)
  (parent 787)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1198)
  (parent 791)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1199)
  (parent 792)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 1200)
  (parent 797)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 1201)
  (parent 798)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1202)
  (parent 799)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1203)
  (parent 802)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 1204)
  (parent 803)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 1205)
  (parent 804)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 1206)
  (parent 807)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 1207)
  (parent 808)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1208)
  (parent 811)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1209)
  (parent 812)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1210)
  (parent 813)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1211)
  (parent 816)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1212)
  (parent 817)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 1213)
  (parent 820)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1214)
  (parent 821)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 1215)
  (parent 822)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 1216)
  (parent 823)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1217)
  (parent 826)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1218)
  (parent 827)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1219)
  (parent 828)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1220)
  (parent 832)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1221)
  (parent 833)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1222)
  (parent 834)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1223)
  (parent 839)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 1224)
  (parent 840)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 1225)
  (parent 841)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1226)
  (parent 842)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1227)
  (parent 845)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1228)
  (parent 846)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1229)
  (parent 850)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1230)
  (parent 854)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1231)
  (parent 856)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1232)
  (parent 859)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1233)
  (parent 860)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1234)
  (parent 862)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1235)
  (parent 865)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1236)
  (parent 866)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1237)
  (parent 869)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1238)
  (parent 870)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1239)
  (parent 876)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1240)
  (parent 877)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1241)
  (parent 879)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1242)
  (parent 882)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1243)
  (parent 883)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1244)
  (parent 887)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1245)
  (parent 891)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1246)
  (parent 893)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1247)
  (parent 896)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1248)
  (parent 897)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1249)
  (parent 899)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1250)
  (parent 902)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1251)
  (parent 903)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1252)
  (parent 908)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 1253)
  (parent 909)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1254)
  (parent 913)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1255)
  (parent 914)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1256)
  (parent 916)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1257)
  (parent 920)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 1258)
  (parent 921)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1259)
  (parent 922)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1260)
  (parent 922)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1261)
  (parent 926)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) e))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 e) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e-0 l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) e)) (send (cat (exp (gen) l-1) e)))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1262)
  (parent 927)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1263)
  (parent 927)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1264)
  (parent 931)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul e l-1) (one)) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1265)
  (parent 932)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1266)
  (parent 932)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1267)
  (parent 933)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1268)
  (parent 937)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1269)
  (parent 938)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul e-0 l-2) (one)) ((mul (rec eA) e e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1270)
  (parent 939)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2)))
  (label 1271)
  (parent 939)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1272)
  (parent 940)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1273)
  (parent 944)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1274)
  (parent 945)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1275)
  (parent 946)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1276)
  (parent 946)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1277)
  (parent 951)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1278)
  (parent 952)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1279)
  (parent 953)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1280)
  (parent 953)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1281)
  (parent 958)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 1282)
  (parent 959)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1283)
  (parent 964)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1284)
  (parent 965)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1285)
  (parent 966)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1286)
  (parent 966)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1287)
  (parent 967)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1288)
  (parent 980)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1289)
  (parent 994)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1290)
  (parent 1015)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1291)
  (parent 1028)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1292)
  (parent 1032)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1293)
  (parent 1035)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1294)
  (parent 1038)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1295)
  (parent 1043)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1296)
  (parent 1048)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1297)
  (parent 1054)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1298)
  (parent 1055)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1299)
  (parent 1070)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1300)
  (parent 1086)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1301)
  (parent 1110)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1302)
  (parent 1125)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1303)
  (parent 1140)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1304)
  (parent 1144)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1305)
  (parent 1147)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1306)
  (parent 1150)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1307)
  (parent 1155)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1308)
  (parent 1160)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1309)
  (parent 1164)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1310)
  (parent 1170)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1311)
  (parent 1171)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1312)
  (parent 1186)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1313)
  (parent 1202)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1314)
  (parent 1226)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1315)
  (parent 1241)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1316)
  (parent 1256)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1317)
  (parent 1260)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1318)
  (parent 1263)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1319)
  (parent 1266)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1320)
  (parent 1271)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1321)
  (parent 1276)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1322)
  (parent 1280)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1323)
  (parent 1286)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1324)
  (parent 1287)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1325)
  (parent 1288)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1326)
  (parent 1288)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1327)
  (parent 1288)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1328)
  (parent 1289)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1329)
  (parent 1289)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1330)
  (parent 1289)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1331)
  (parent 1289)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1332)
  (parent 1290)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 1333)
  (parent 1290)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1334)
  (parent 1290)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1335)
  (parent 1290)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1336)
  (parent 1290)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 1337)
  (parent 1291)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1338)
  (parent 1291)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1339)
  (parent 1291)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1340)
  (parent 1291)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 1341)
  (parent 1291)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec e))))
      (send (cat (gen) (mul l-0 eA (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1342)
  (parent 1292)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1343)
  (parent 1292)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1344)
  (parent 1292)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1345)
  (parent 1292)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1346)
  (parent 1293)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1347)
  (parent 1293)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1348)
  (parent 1293)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1349)
  (parent 1293)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1350)
  (parent 1294)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1351)
  (parent 1294)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1352)
  (parent 1294)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1353)
  (parent 1294)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1354)
  (parent 1295)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 1355)
  (parent 1295)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1356)
  (parent 1295)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1357)
  (parent 1295)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1358)
  (parent 1295)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 1359)
  (parent 1296)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1360)
  (parent 1296)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1361)
  (parent 1296)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1362)
  (parent 1296)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 1363)
  (parent 1296)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 1364)
  (parent 1297)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1365)
  (parent 1297)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 1366)
  (parent 1297)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1367)
  (parent 1297)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1368)
  (parent 1297)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 1369)
  (parent 1297)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 1370)
  (parent 1298)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1371)
  (parent 1298)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 1372)
  (parent 1298)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1373)
  (parent 1298)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1374)
  (parent 1298)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 1375)
  (parent 1298)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1376)
  (parent 1299)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1377)
  (parent 1299)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1378)
  (parent 1299)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1379)
  (parent 1300)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1380)
  (parent 1300)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1381)
  (parent 1300)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1382)
  (parent 1300)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1383)
  (parent 1301)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1384)
  (parent 1301)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1385)
  (parent 1301)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1386)
  (parent 1301)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1387)
  (parent 1301)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1388)
  (parent 1302)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1389)
  (parent 1302)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1390)
  (parent 1302)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1391)
  (parent 1302)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1392)
  (parent 1302)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1393)
  (parent 1303)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1394)
  (parent 1303)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1395)
  (parent 1303)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1396)
  (parent 1303)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1397)
  (parent 1303)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec e))))
      (send (cat (gen) (mul l-1 eA (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1398)
  (parent 1304)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1399)
  (parent 1304)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1400)
  (parent 1304)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1401)
  (parent 1304)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1402)
  (parent 1305)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1403)
  (parent 1305)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1404)
  (parent 1305)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1405)
  (parent 1305)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1406)
  (parent 1306)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1407)
  (parent 1306)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1408)
  (parent 1306)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1409)
  (parent 1306)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1410)
  (parent 1307)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1411)
  (parent 1307)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1412)
  (parent 1307)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1413)
  (parent 1307)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1414)
  (parent 1307)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1415)
  (parent 1308)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1416)
  (parent 1308)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1417)
  (parent 1308)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1418)
  (parent 1308)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1419)
  (parent 1308)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1420)
  (parent 1309)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1421)
  (parent 1309)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1422)
  (parent 1309)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1423)
  (parent 1309)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1424)
  (parent 1309)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1425)
  (parent 1310)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1426)
  (parent 1310)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1427)
  (parent 1310)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1428)
  (parent 1310)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1429)
  (parent 1310)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1430)
  (parent 1310)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1431)
  (parent 1311)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1432)
  (parent 1311)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1433)
  (parent 1311)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1434)
  (parent 1311)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1435)
  (parent 1311)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1436)
  (parent 1311)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1437)
  (parent 1312)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1438)
  (parent 1312)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1439)
  (parent 1312)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1440)
  (parent 1313)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1441)
  (parent 1313)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1442)
  (parent 1313)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1443)
  (parent 1313)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1444)
  (parent 1314)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1445)
  (parent 1314)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1446)
  (parent 1314)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1447)
  (parent 1314)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1448)
  (parent 1314)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1449)
  (parent 1315)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1450)
  (parent 1315)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1451)
  (parent 1315)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1452)
  (parent 1315)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1453)
  (parent 1315)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1454)
  (parent 1316)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1455)
  (parent 1316)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1456)
  (parent 1316)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1457)
  (parent 1316)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1458)
  (parent 1316)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec e))))
      (send (cat (gen) (mul l-1 eA (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1459)
  (parent 1317)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1460)
  (parent 1317)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1461)
  (parent 1317)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1462)
  (parent 1317)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1463)
  (parent 1318)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1464)
  (parent 1318)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1465)
  (parent 1318)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1466)
  (parent 1318)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1467)
  (parent 1319)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l) (A l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l B) (l-0 A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1468)
  (parent 1319)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1469)
  (parent 1319)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1470)
  (parent 1319)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1471)
  (parent 1320)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1472)
  (parent 1320)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1473)
  (parent 1320)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1474)
  (parent 1320)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1475)
  (parent 1320)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1476)
  (parent 1321)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1477)
  (parent 1321)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1478)
  (parent 1321)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1479)
  (parent 1321)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1480)
  (parent 1321)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1481)
  (parent 1322)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1482)
  (parent 1322)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1483)
  (parent 1322)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1484)
  (parent 1322)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1485)
  (parent 1322)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1486)
  (parent 1323)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1487)
  (parent 1323)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1488)
  (parent 1323)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1489)
  (parent 1323)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1490)
  (parent 1323)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1491)
  (parent 1323)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1492)
  (parent 1324)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1493)
  (parent 1324)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1494)
  (parent 1324)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1495)
  (parent 1324)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1496)
  (parent 1324)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1497)
  (parent 1324)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 1498)
  (parent 1326)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1499)
  (parent 1327)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 1500)
  (parent 1330)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) e-0 e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 1501)
  (parent 1331)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1502)
  (parent 1334)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 1503)
  (parent 1335)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 1504)
  (parent 1336)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 1505)
  (parent 1339)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1506)
  (parent 1340)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 1507)
  (parent 1344)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1508)
  (parent 1345)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-0 (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 1509)
  (parent 1346)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-0 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 1510)
  (parent 1348)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 1511)
  (parent 1352)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1512)
  (parent 1353)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1513)
  (parent 1356)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 1514)
  (parent 1357)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (8 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 1515)
  (parent 1358)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 1516)
  (parent 1361)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 1517)
  (parent 1362)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1518)
  (parent 1367)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 1519)
  (parent 1368)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (8 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1520)
  (parent 1369)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1521)
  (parent 1373)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 1522)
  (parent 1374)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1523)
  (parent 1375)
  (seen 1584)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1524)
  (parent 1377)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1525)
  (parent 1378)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1526)
  (parent 1381)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0 e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1527)
  (parent 1382)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1528)
  (parent 1385)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1529)
  (parent 1386)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1530)
  (parent 1387)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1531)
  (parent 1390)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1532)
  (parent 1391)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1533)
  (parent 1395)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1534)
  (parent 1396)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1535)
  (parent 1400)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1536)
  (parent 1401)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1537)
  (parent 1402)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1538)
  (parent 1404)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1539)
  (parent 1408)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1540)
  (parent 1409)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1541)
  (parent 1412)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1542)
  (parent 1413)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1543)
  (parent 1414)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1544)
  (parent 1417)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1545)
  (parent 1418)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1546)
  (parent 1422)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1547)
  (parent 1423)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1548)
  (parent 1428)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1549)
  (parent 1429)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1550)
  (parent 1430)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1551)
  (parent 1434)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1552)
  (parent 1435)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1553)
  (parent 1436)
  (seen 1585)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1554)
  (parent 1438)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1555)
  (parent 1439)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1556)
  (parent 1442)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0 e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1557)
  (parent 1443)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1558)
  (parent 1446)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1559)
  (parent 1447)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1560)
  (parent 1448)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1561)
  (parent 1451)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1562)
  (parent 1452)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1563)
  (parent 1456)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1564)
  (parent 1457)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1565)
  (parent 1461)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1566)
  (parent 1462)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1567)
  (parent 1463)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 1568)
  (parent 1465)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1569)
  (parent 1469)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1570)
  (parent 1470)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1571)
  (parent 1473)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1572)
  (parent 1474)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 1573)
  (parent 1475)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1574)
  (parent 1478)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1575)
  (parent 1479)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1576)
  (parent 1483)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1577)
  (parent 1484)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1578)
  (parent 1489)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1579)
  (parent 1490)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1580)
  (parent 1491)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1581)
  (parent 1495)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1582)
  (parent 1496)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1583)
  (parent 1497)
  (seen 1586)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1584)
  (parent 1520)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1585)
  (parent 1550)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1586)
  (parent 1580)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1587)
  (parent 1584)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 1588)
  (parent 1584)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1589)
  (parent 1584)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 1590)
  (parent 1584)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1591)
  (parent 1585)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1592)
  (parent 1585)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1593)
  (parent 1585)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1594)
  (parent 1585)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1595)
  (parent 1586)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1596)
  (parent 1586)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1597)
  (parent 1586)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1598)
  (parent 1586)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1599)
  (parent 1589)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 1600)
  (parent 1590)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1601)
  (parent 1593)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (10 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec l-2) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1602)
  (parent 1594)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1603)
  (parent 1597)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (10 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec l-2) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1604)
  (parent 1598)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol eadh-um diffie-hellman
  (defrole init
    (vars (l e rndx) (ll ee expt) (self peer name) (n data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l ll)) (exp (gen) (mul e ee)))))
      (recv n))
    (uniq-orig n)
    (uniq-gen e)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
    (neq (ee (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) ll))))
  (defrole resp
    (vars (l e rndx) (ll ee expt) (self peer name) (n data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l ll)) (exp (gen) (mul e ee)))))
      (send n))
    (uniq-gen e)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
    (neq (ee (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) ll))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))))

(defskeleton eadh-um
  (vars (n data) (A B name) (lA eA lB rndx) (ll ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l lA) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l lB))
  (absent (eA (exp (gen) lA)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (B lB) (A lA)) ("privdh-of" (lB B) (lA A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) lB)
  (uniq-gen eA lB)
  (uniq-orig n)
  (comment "Explicit authentication, not assuming own LTK safe")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul lA ll)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) lB) B (privk B)))))
  (label 1605)
  (unrealized (0 0) (0 1))
  (origs (n (0 4)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ll ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test (displaced 2 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul ll l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 1606)
  (parent 1605)
  (unrealized (0 1))
  (origs (n (0 4)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA lB rndx) (ll ee expt) (l rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll ll)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l lB))
  (defstrand ltx-gen 1 (self A) (l l))
  (precedes ((2 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) ll)))
  (fn-of ("principal-of" (A l) (B lB)) ("privdh-of" (l A) (lB B)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) lB)
  (uniq-gen eA lB l)
  (uniq-orig n)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) ll) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul ll l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) lB) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A)))))
  (label 1607)
  (parent 1605)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 0) (0 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test (displaced 2 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) B (privk B)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 1608)
  (parent 1606)
  (unrealized (0 5))
  (origs (n (0 4)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation encryption-test (displaced 3 1 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) B (privk B)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A)))))
  (label 1609)
  (parent 1607)
  (unrealized (0 5))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 1)) ((2 2) (0 3))
    ((2 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 6) n (0 5)
    (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 1610)
  (parent 1608)
  (unrealized (0 3) (2 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (2 0)) ((1 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))) n (0 5)
    (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))))
  (label 1611)
  (parent 1608)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1)) ((2 0) (0 0))
    ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 6) n (0 5)
    (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 1612)
  (parent 1609)
  (unrealized (0 3) (3 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((3 1) (0 5)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))) n (0 5)
    (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))))
  (label 1613)
  (parent 1609)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee ee))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0)) ((2 2) (0 3))
    ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 ltx-gen 1) (exp (gen) l) (2 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 1614)
  (parent 1610)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (3 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 1615)
  (parent 1610)
  (unrealized (0 3) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((2 1) (0 5)) ((3 1) (2 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))))
  (label 1616)
  (parent 1611)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1)) ((2 0) (0 0))
    ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 1617)
  (parent 1612)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0)) ((2 0) (0 0))
    ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n)))
  (label 1618)
  (parent 1612)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (4 0)) ((2 0) (0 0))
    ((2 0) (4 0)) ((3 2) (0 3)) ((3 5) (0 5)) ((4 1) (3 1)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1619)
  (parent 1612)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (4 0)) ((2 0) (0 0))
    ((2 0) (4 0)) ((3 2) (0 3)) ((3 5) (0 5)) ((4 1) (3 1)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1620)
  (parent 1612)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (precedes ((0 2) (4 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((3 1) (0 5))
    ((4 1) (3 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))))
  (label 1621)
  (parent 1613)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 1622)
  (parent 1614)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee eA))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk self) l)
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 3 2 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 1623)
  (parent 1614)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l) (A self) (B self) (ll l) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 2) (0 3)) ((3 2) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1624)
  (parent 1614)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n)))
  (label 1625)
  (parent 1614)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 0) (0 3)) ((3 0) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1626)
  (parent 1614)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (ee expt) (l rndx)
    (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 1627)
  (parent 1614)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l l-0 eA e rndx) (ee w expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l l (rec l-0))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l (rec l-0))) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1628)
  (parent 1615)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx) (w expt))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1629)
  (parent 1616)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 1630)
  (parent 1617)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee eA))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 1631)
  (parent 1617)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (lA l) (eA eA) (lB l-0) (A A) (B B) (ll l-0) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 2) (0 3)) ((4 2) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1632)
  (parent 1617)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 1633)
  (parent 1617)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 1634)
  (parent 1617)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 0) (0 3)) ((4 0) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1635)
  (parent 1617)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0)) ((3 5) (0 5))
    ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 1636)
  (parent 1617)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n)))
  (label 1637)
  (parent 1618)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee eA))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n)))
  (label 1638)
  (parent 1618)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (lA l-0) (eA eA) (lB l) (A A) (B B) (ll l) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 2) (0 3)) ((4 2) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1639)
  (parent 1618)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 1640)
  (parent 1618)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)))
  (label 1641)
  (parent 1618)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5))
    ((4 0) (0 3)) ((4 0) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1642)
  (parent 1618)
  (unrealized (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (ee expt) (l l-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee ee))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0)) ((3 5) (0 5))
    ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 1643)
  (parent 1618)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B self peer name) (l l-0 l-1 eA e rndx) (ee expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee ee))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (5 0)) ((2 0) (0 0))
    ((2 0) (5 0)) ((3 2) (0 3)) ((3 5) (0 5)) ((4 1) (3 1))
    ((5 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1644)
  (parent 1619)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (ee expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 5 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1645)
  (parent 1620)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 0) (0 1)) ((1 0) (5 0)) ((2 0) (0 0))
    ((2 0) (5 0)) ((3 1) (0 5)) ((4 1) (3 0)) ((5 1) (4 0)))
  (absent (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (ee (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1646)
  (parent 1621)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 1647)
  (parent 1622)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 1648)
  (parent 1622)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 1649)
  (parent 1624)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 1650)
  (parent 1624)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 2) (0 3))
    ((3 2) (4 0)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 1651)
  (parent 1624)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) (mul eA l)) e))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA l)) e))
    (exp (gen) (mul eA (rec e) l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l)) e))
      (send (cat (exp (gen) (mul eA l)) e))))
  (label 1652)
  (parent 1625)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) (mul (rec e) l)) eA))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l)) eA))
    (exp (gen) (mul eA (rec e) l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l)) eA))
      (send (cat (exp (gen) (mul (rec e) l)) eA))))
  (label 1653)
  (parent 1625)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-0))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0))))
  (label 1654)
  (parent 1626)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 1655)
  (parent 1626)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (4 0)) ((2 5) (0 5)) ((3 0) (0 3))
    ((3 0) (4 0)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 1656)
  (parent 1626)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1657)
  (parent 1627)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1658)
  (parent 1627)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1659)
  (parent 1627)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1660)
  (parent 1627)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w l) (one)) ((mul w l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (send n))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 1661)
  (parent 1627)
  (unrealized (2 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1662)
  (parent 1627)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l eA e rndx) (ee expt) (l-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-0)))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-0)
  (uniq-gen eA e l-0)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 1663)
  (parent 1628)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (l e rndx) (ee expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e e-0) (ll l-0)
    (ee (mul e ee (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-0))))
  (fn-of ("principal-of" (B l-0) (self l))
    ("privdh-of" (l-0 B) (l self)))
  (neq (ee (one)) ((mul e ee (rec e-0)) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e ee (rec e-0))))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-0 l-0 (rec e-0))))))
  (label 1664)
  (parent 1628)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 peer-0 name) (l eA e rndx)
    (ee expt) (l-0 rndx) (ll expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-1) (e eA) (ll l-1)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-1 l-1)) l))
  (deflistener (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-1))) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-1)
  (precur (4 0))
  (uniq-gen eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-1)) l))
      (send (cat (exp (gen) (mul l-1 l-1)) l)))
    ((recv (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1665)
  (parent 1628)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e rndx) (ee expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-0)) (ee ee))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-0)))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-0)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1666)
  (parent 1628)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B name) (eA rndx) (ee expt) (l rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 1667)
  (parent 1629)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B name) (ee expt) (l e rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e e) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
  (deflistener (cat (exp (gen) e) (mul l l (rec e))))
  (precedes ((0 2) (4 0)) ((1 0) (0 0)) ((2 1) (0 5)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l)))
  (fn-of ("principal-of" (B l)) ("privdh-of" (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) ee))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul ee e)))))
    ((recv (cat (exp (gen) e) (mul l l (rec e))))
      (send (cat (exp (gen) e) (mul l l (rec e))))))
  (label 1668)
  (parent 1629)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer name) (eA rndx) (ee expt) (l rndx)
    (ll expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l-0) (e eA) (ll l-0)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) e) (mul l-0 l-0 (rec e))))
  (defstrand resp 3 (self self) (peer peer) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq (ee (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) e) (mul l-0 l-0 (rec e))))
      (send (cat (exp (gen) e) (mul l-0 l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1669)
  (parent 1629)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self B) (peer B) (l l) (e eA) (ll l) (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 0) (0 0)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1670)
  (parent 1629)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 1671)
  (parent 1630)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 1672)
  (parent 1630)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 1673)
  (parent 1632)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 1674)
  (parent 1632)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 1675)
  (parent 1632)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 1676)
  (parent 1633)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 1677)
  (parent 1633)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-0))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0))))
  (label 1678)
  (parent 1634)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 1679)
  (parent 1634)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 1680)
  (parent 1634)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 1681)
  (parent 1635)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 1682)
  (parent 1635)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 1683)
  (parent 1635)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1684)
  (parent 1636)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1685)
  (parent 1636)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1686)
  (parent 1636)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1687)
  (parent 1636)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1688)
  (parent 1636)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1689)
  (parent 1636)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1690)
  (parent 1636)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 1691)
  (parent 1637)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 1692)
  (parent 1637)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 1693)
  (parent 1639)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 1694)
  (parent 1639)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 2) (0 3)) ((4 2) (5 0)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 1695)
  (parent 1639)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 1696)
  (parent 1640)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 1697)
  (parent 1640)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-0))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0))))
  (label 1698)
  (parent 1641)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA l-0)) e))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) e))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA l-0)) e))
      (send (cat (exp (gen) (mul eA l-0)) e))))
  (label 1699)
  (parent 1641)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec e) l-0)) eA))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-0)) eA))
    (exp (gen) (mul eA (rec e) l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec e) l-0)) eA))
      (send (cat (exp (gen) (mul (rec e) l-0)) eA))))
  (label 1700)
  (parent 1641)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 1701)
  (parent 1642)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 1702)
  (parent 1642)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (5 0))
    ((3 5) (0 5)) ((4 0) (0 3)) ((4 0) (5 0)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 1703)
  (parent 1642)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1704)
  (parent 1643)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1705)
  (parent 1643)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1706)
  (parent 1643)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1707)
  (parent 1643)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1708)
  (parent 1643)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1709)
  (parent 1643)
  (unrealized (3 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1710)
  (parent 1643)
  (unrealized (3 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (ee expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0)) ((2 2) (0 3))
    ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1711)
  (parent 1644)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1712)
  (parent 1645)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee eA))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1713)
  (parent 1645)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)) ((5 2) (0 3)) ((5 2) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1714)
  (parent 1645)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1715)
  (parent 1645)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1716)
  (parent 1645)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)) ((5 0) (0 3)) ((5 0) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 1717)
  (parent 1645)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (ee expt)
    (l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 0) (0 0))
    ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 1718)
  (parent 1645)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((1 0) (4 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-0 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1719)
  (parent 1646)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 3 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1720)
  (parent 1654)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 1721)
  (parent 1657)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 1722)
  (parent 1657)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1723)
  (parent 1657)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (gen) (mul (rec eA) e l)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul (rec eA) e l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l)))
      (send (cat (gen) (mul (rec eA) e l)))))
  (label 1724)
  (parent 1657)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1725)
  (parent 1657)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 1726)
  (parent 1657)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 1727)
  (parent 1658)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 1728)
  (parent 1658)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1729)
  (parent 1658)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul e (rec e-0) l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l)))))
  (label 1730)
  (parent 1658)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1731)
  (parent 1658)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 1732)
  (parent 1658)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA w)))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 5) (0 5)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk self) l)
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 1733)
  (parent 1659)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l) (A self) (B self) (ll l)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-1)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen e e-0 l-0 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1734)
  (parent 1660)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 1735)
  (parent 1660)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA e l-0 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 4 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1736)
  (parent 1660)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)) ((5 2) (2 3)) ((5 2) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1737)
  (parent 1660)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1738)
  (parent 1660)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)) ((5 0) (2 3)) ((5 0) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 1739)
  (parent 1660)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 1740)
  (parent 1660)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 1741)
  (parent 1661)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 1742)
  (parent 1661)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 2) (2 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1743)
  (parent 1661)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (l (one)) ((mul (rec eA) e l) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e)))
      (send (cat (exp (gen) l) (mul (rec eA) e)))))
  (label 1744)
  (parent 1661)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1745)
  (parent 1661)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) w l) (one)) ((mul w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l)))))
      (send n))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l (rec w-0))) w-0))))
  (label 1746)
  (parent 1661)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 1747)
  (parent 1662)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 1748)
  (parent 1662)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)) ((5 2) (2 3)) ((5 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1749)
  (parent 1662)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 1750)
  (parent 1662)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1751)
  (parent 1662)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (0 3))
    ((4 0) (2 3)) ((4 0) (3 0)) ((5 0) (2 3)) ((5 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk self) l)
  (precur (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 1752)
  (parent 1662)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 1753)
  (parent 1662)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 1))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1754)
  (parent 1678)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1755)
  (parent 1681)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 1756)
  (parent 1684)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 1757)
  (parent 1684)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1758)
  (parent 1684)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 1759)
  (parent 1684)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 1760)
  (parent 1684)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1761)
  (parent 1684)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 1762)
  (parent 1684)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 1763)
  (parent 1685)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 1764)
  (parent 1685)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1765)
  (parent 1685)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 1766)
  (parent 1685)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 1767)
  (parent 1685)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1768)
  (parent 1685)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 1769)
  (parent 1685)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 1770)
  (parent 1686)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l) (eA eA) (lB l-0) (A A) (B B) (ll l-0)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1771)
  (parent 1687)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 1772)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e l-1 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1773)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1774)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1775)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1776)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1777)
  (parent 1687)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 1778)
  (parent 1687)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 1779)
  (parent 1688)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 1780)
  (parent 1688)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1781)
  (parent 1688)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 1782)
  (parent 1688)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 1783)
  (parent 1688)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1784)
  (parent 1688)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 1785)
  (parent 1688)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 1786)
  (parent 1689)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 1787)
  (parent 1689)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1788)
  (parent 1689)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 1789)
  (parent 1689)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 1790)
  (parent 1689)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1791)
  (parent 1689)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 1792)
  (parent 1689)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 1793)
  (parent 1690)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 1794)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1795)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1796)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1797)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1798)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1799)
  (parent 1690)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 1800)
  (parent 1690)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (3 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 1)) ((4 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1801)
  (parent 1698)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1802)
  (parent 1701)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 1803)
  (parent 1704)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 1804)
  (parent 1704)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1805)
  (parent 1704)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 1806)
  (parent 1704)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-0)))
      (send (cat (gen) (mul (rec eA) e l-0)))))
  (label 1807)
  (parent 1704)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1808)
  (parent 1704)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 1809)
  (parent 1704)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 1810)
  (parent 1705)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 1811)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1812)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 1813)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul e (rec e-0) l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-0)))))
  (label 1814)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1815)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 1816)
  (parent 1705)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA w)))
  (precedes ((0 2) (3 3)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 5) (0 5)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n)))
  (label 1817)
  (parent 1706)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l-0) (eA eA) (lB l) (A A) (B B) (ll l)
        (ee (mul w e)))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1818)
  (parent 1707)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 1819)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e l-1 rndx)
    (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1820)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1821)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1822)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1823)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1824)
  (parent 1707)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 1825)
  (parent 1707)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 1826)
  (parent 1708)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 1827)
  (parent 1708)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1828)
  (parent 1708)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 1829)
  (parent 1708)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 1830)
  (parent 1708)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1831)
  (parent 1708)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 1832)
  (parent 1708)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l))))))
  (label 1833)
  (parent 1709)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 1834)
  (parent 1709)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee e-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (3 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1835)
  (parent 1709)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) l-0)))))
  (label 1836)
  (parent 1709)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e)))))
  (label 1837)
  (parent 1709)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1838)
  (parent 1709)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-0) (one)) ((mul w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-0)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-0 (rec w-0))) w-0))))
  (label 1839)
  (parent 1709)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l) (mul e (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-1)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l))))
      (send (cat (exp (gen) l) (mul e (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 1840)
  (parent 1710)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee e))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 1841)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee e-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 2) (3 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1842)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1843)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1844)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1845)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (3 3)) ((0 2) (4 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (3 3)) ((5 0) (4 0))
    ((6 0) (3 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1846)
  (parent 1710)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (3 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 1847)
  (parent 1710)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1848)
  (parent 1711)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee eA))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) eA))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1849)
  (parent 1711)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)) ((6 2) (0 3)) ((6 2) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1850)
  (parent 1711)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1851)
  (parent 1711)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1852)
  (parent 1711)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)) ((6 0) (0 3)) ((6 0) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 1853)
  (parent 1711)
  (unrealized (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (ee expt)
    (l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e ee)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee ee))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1)) ((4 1) (3 0))
    ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (ee (one)) ((mul (rec eA) e ee) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (precur (6 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
    (exp (gen) (mul (rec eA) e ee)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e ee)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) ee))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e ee)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e ee (rec w))) w))))
  (label 1854)
  (parent 1711)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 1855)
  (parent 1712)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 1856)
  (parent 1712)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((1 0) (2 0)) ((2 1) (0 3)) ((3 0) (0 0))
    ((3 1) (2 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1857)
  (parent 1713)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 2) (0 3)) ((5 2) (6 0))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 1858)
  (parent 1714)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 2) (0 3)) ((5 2) (6 0))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 1859)
  (parent 1714)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 2) (0 3)) ((5 2) (6 0))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 1860)
  (parent 1714)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 1861)
  (parent 1715)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 1862)
  (parent 1715)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 1863)
  (parent 1716)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 1864)
  (parent 1716)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 1865)
  (parent 1716)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 0) (0 3)) ((5 0) (6 0))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-2))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2))))
  (label 1866)
  (parent 1717)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA l-2)) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 0) (0 3)) ((5 0) (6 0))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-2)) e))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA l-2)) e))
      (send (cat (exp (gen) (mul eA l-2)) e))))
  (label 1867)
  (parent 1717)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec e) l-2)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 0) (0 3)) ((5 0) (6 0))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-2)) eA))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul (rec e) l-2)) eA))
      (send (cat (exp (gen) (mul (rec e) l-2)) eA))))
  (label 1868)
  (parent 1717)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1869)
  (parent 1718)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1870)
  (parent 1718)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA w)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1871)
  (parent 1718)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1872)
  (parent 1718)
  (unrealized (2 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1873)
  (parent 1718)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1874)
  (parent 1718)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-2) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 1875)
  (parent 1718)
  (unrealized (2 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (2 0)) ((1 0) (0 1)) ((1 0) (3 0)) ((2 1) (0 5))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 1) (3 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1876)
  (parent 1719)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 1877)
  (parent 1720)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 3)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 1878)
  (parent 1720)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1879)
  (parent 1726)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1880)
  (parent 1726)
  (unrealized (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1881)
  (parent 1726)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1882)
  (parent 1726)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (gen) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l)))
      (send (cat (gen) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 1883)
  (parent 1726)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1884)
  (parent 1726)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1885)
  (parent 1732)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1886)
  (parent 1732)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1887)
  (parent 1732)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1888)
  (parent 1732)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul e (rec e-0) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 1889)
  (parent 1732)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1890)
  (parent 1732)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1891)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-1)
    (ll l-0) (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 1892)
  (parent 1740)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1893)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1894)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1895)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1896)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 1897)
  (parent 1740)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l) w))
      (l-0 l) (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1898)
  (parent 1746)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 resp 3) (exp (gen) e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1899)
  (parent 1746)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (displaced 5 2 resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1900)
  (parent 1746)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1901)
  (parent 1746)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 1902)
  (parent 1746)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 1903)
  (parent 1746)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1904)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1905)
  (parent 1753)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1906)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1907)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1908)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1909)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 1910)
  (parent 1753)
  (unrealized (0 3) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 1))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 1911)
  (parent 1754)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 1))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 1912)
  (parent 1754)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 1913)
  (parent 1755)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 1))
    ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 1914)
  (parent 1755)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1915)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1916)
  (parent 1762)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1917)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1918)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1919)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1920)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1921)
  (parent 1762)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1922)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1923)
  (parent 1769)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1924)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1925)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1926)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1927)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1928)
  (parent 1769)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1929)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 1930)
  (parent 1778)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1931)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1932)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1933)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1934)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1935)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1936)
  (parent 1778)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1937)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1938)
  (parent 1785)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1939)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1940)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1941)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1942)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1943)
  (parent 1785)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1944)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1945)
  (parent 1792)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1946)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1947)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1948)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1949)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1950)
  (parent 1792)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1951)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1952)
  (parent 1800)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1953)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1954)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1955)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1956)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1957)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1958)
  (parent 1800)
  (unrealized (0 3) (4 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 1959)
  (parent 1801)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (2 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 3))
    ((4 0) (0 0)) ((4 0) (2 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-0))
      (send (cat (exp (gen) (mul eA (rec e))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 1960)
  (parent 1801)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 1961)
  (parent 1802)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (0 3)) ((3 2) (6 0))
    ((3 5) (0 5)) ((4 1) (3 3)) ((5 0) (0 3)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 1962)
  (parent 1802)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1963)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1964)
  (parent 1809)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1965)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1966)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1967)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (gen) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-0)))
      (send (cat (gen) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1968)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1969)
  (parent 1809)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1970)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1971)
  (parent 1816)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1972)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 1973)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1974)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul e (rec e-0) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1975)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1976)
  (parent 1816)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1977)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 l-1 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-1) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e e-0 l-0 l-1 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 1978)
  (parent 1825)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1979)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1980)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 1981)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1982)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1983)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1984)
  (parent 1825)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1985)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1986)
  (parent 1832)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1987)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1988)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1989)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1990)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1991)
  (parent 1832)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1992)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e w (rec l))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w (rec l))))
      (send (cat (exp (gen) l) (mul e w (rec l)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 1993)
  (parent 1839)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 3 resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 1994)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 1995)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1996)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1997)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1998)
  (parent 1839)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx) (w expt))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1999)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l-1)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2000)
  (parent 1847)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 3 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2001)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2002)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2003)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2004)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2005)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2006)
  (parent 1847)
  (unrealized (0 3) (4 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (rec e)) e-0))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) e-0))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (rec e)) e-0))
      (send (cat (exp (gen) (rec e)) e-0))))
  (label 2007)
  (parent 1848)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul e-0 e-0)) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e-0 e-0)) e))
    (exp (gen) (mul (rec e) e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul e-0 e-0)) e))
      (send (cat (exp (gen) (mul e-0 e-0)) e))))
  (label 2008)
  (parent 1848)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((1 0) (3 0)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2009)
  (parent 1849)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e))) e-0))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 2) (0 3))
    ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) e-0))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e))) e-0))
      (send (cat (exp (gen) (mul eA (rec e))) e-0))))
  (label 2010)
  (parent 1850)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA e-0)) e))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 2) (0 3))
    ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA e-0)) e))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA e-0)) e))
      (send (cat (exp (gen) (mul eA e-0)) e))))
  (label 2011)
  (parent 1850)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul (rec e) e-0)) eA))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 2) (0 3))
    ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) e-0)) eA))
    (exp (gen) (mul eA (rec e) e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul (rec e) e-0)) eA))
      (send (cat (exp (gen) (mul (rec e) e-0)) eA))))
  (label 2012)
  (parent 1850)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 2013)
  (parent 1851)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 2014)
  (parent 1851)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-1))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1))))
  (label 2015)
  (parent 1852)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) e))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA l-1)) e))
      (send (cat (exp (gen) (mul eA l-1)) e))))
  (label 2016)
  (parent 1852)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec e) l-1)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-1)) eA))
    (exp (gen) (mul eA (rec e) l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul (rec e) l-1)) eA))
      (send (cat (exp (gen) (mul (rec e) l-1)) eA))))
  (label 2017)
  (parent 1852)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 0) (0 3))
    ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e))) l-2))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2))))
  (label 2018)
  (parent 1853)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA l-2)) e))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 0) (0 3))
    ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-2)) e))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA l-2)) e))
      (send (cat (exp (gen) (mul eA l-2)) e))))
  (label 2019)
  (parent 1853)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec e) l-2)) eA))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 0) (0 3))
    ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec e) l-2)) eA))
    (exp (gen) (mul eA (rec e) l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul (rec e) l-2)) eA))
      (send (cat (exp (gen) (mul (rec e) l-2)) eA))))
  (label 2020)
  (parent 1853)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (6 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (ee (mul eA (rec e) w)) (w-0 w)) (gen)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2021)
  (parent 1854)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2022)
  (parent 1854)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA w)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2023)
  (parent 1854)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2024)
  (parent 1854)
  (unrealized (2 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2025)
  (parent 1854)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2026)
  (parent 1854)
  (unrealized (2 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-2) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2027)
  (parent 1854)
  (unrealized (2 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 3)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2028)
  (parent 1857)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)) ((6 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 2029)
  (parent 1863)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)) ((6 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 2030)
  (parent 1863)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)) ((6 0) (0 3))
    ((6 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2031)
  (parent 1866)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 2032)
  (parent 1869)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 2033)
  (parent 1869)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2034)
  (parent 1869)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1)))))
  (label 2035)
  (parent 1869)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1)))))
  (label 2036)
  (parent 1869)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e l-2)))
      (send (cat (gen) (mul (rec eA) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2037)
  (parent 1869)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 2038)
  (parent 1869)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 2039)
  (parent 1870)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 2040)
  (parent 1870)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2041)
  (parent 1870)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))))
  (label 2042)
  (parent 1870)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))))
  (label 2043)
  (parent 1870)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (l-2 (one)) ((mul e (rec e-0) l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2044)
  (parent 1870)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 2045)
  (parent 1870)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 0) (0 1)) ((1 0) (2 0)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 1) (2 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2046)
  (parent 1871)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e e-0 l-1 l-2 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-1)
    (ll l-2) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee e-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-2))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (5 0))
  (uniq-gen e e-0 l-1 l-2 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2047)
  (parent 1872)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2048)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA e l-2 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2049)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-1))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)) ((7 2) (2 3)) ((7 2) (5 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2050)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2051)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2052)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-3))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)) ((7 0) (2 3)) ((7 0) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (l-3 (one)) ((mul (rec eA) e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-3))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2053)
  (parent 1872)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 2054)
  (parent 1872)
  (unrealized (7 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1))))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e (rec l-1))))))
  (label 2055)
  (parent 1873)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 2056)
  (parent 1873)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2057)
  (parent 1873)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e)))))
  (label 2058)
  (parent 1873)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))))
  (label 2059)
  (parent 1873)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2060)
  (parent 1873)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 2061)
  (parent 1873)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 2062)
  (parent 1874)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 6 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 2063)
  (parent 1874)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 2) (2 3))
    ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2064)
  (parent 1874)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))))
  (label 2065)
  (parent 1874)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e)))))
  (label 2066)
  (parent 1874)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2067)
  (parent 1874)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 2068)
  (parent 1874)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l e l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (5 0))
  (uniq-gen e l-0 l-1 l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))))
  (label 2069)
  (parent 1875)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (eA l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee e))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) (mul l-0 l-1 (rec l-2))))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1)))))
  (label 2070)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)) ((7 2) (2 3)) ((7 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2071)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2072)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2073)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2074)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-3))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 0) (2 3))
    ((6 0) (5 0)) ((7 0) (2 3)) ((7 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (l-3 (one)) ((mul (rec eA) e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-3))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2075)
  (parent 1875)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-2) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))))
  (label 2076)
  (parent 1875)
  (unrealized (7 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((1 0) (2 0)) ((2 1) (0 5)) ((3 0) (0 0))
    ((3 1) (2 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2077)
  (parent 1876)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2078)
  (parent 1879)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l e eA)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2079)
  (parent 1879)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2080)
  (parent 1879)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2081)
  (parent 1880)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2082)
  (parent 1880)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 2083)
  (parent 1880)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2084)
  (parent 1881)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2085)
  (parent 1881)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2086)
  (parent 1881)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2087)
  (parent 1882)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2088)
  (parent 1882)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2089)
  (parent 1882)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2090)
  (parent 1882)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2091)
  (parent 1883)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (gen) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l))) (send (cat (gen) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 2092)
  (parent 1883)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))))
  (label 2093)
  (parent 1883)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 2094)
  (parent 1883)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2095)
  (parent 1884)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2096)
  (parent 1884)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2097)
  (parent 1884)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2098)
  (parent 1884)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2099)
  (parent 1884)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 2100)
  (parent 1885)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2101)
  (parent 1885)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2102)
  (parent 1885)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 2103)
  (parent 1885)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 2104)
  (parent 1886)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2105)
  (parent 1886)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2106)
  (parent 1886)
  (unrealized (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 2107)
  (parent 1886)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 2108)
  (parent 1887)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e e (rec e-0) l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2109)
  (parent 1887)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 2110)
  (parent 1887)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2111)
  (parent 1887)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2112)
  (parent 1888)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2113)
  (parent 1888)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2114)
  (parent 1888)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-1) (ll l)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2115)
  (parent 1888)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2116)
  (parent 1888)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l)))
      (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l))))))
  (label 2117)
  (parent 1889)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2118)
  (parent 1889)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e-0)) (one)) ((mul (rec e) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l))
    (mul (rec e) (rec e) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e-0))))
      (send (cat (exp (gen) l) (mul w (rec e-0))))))
  (label 2119)
  (parent 1889)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul e w l)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l e-0 e-0) (one)) ((mul e w l e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l)))
      (send (cat (exp (gen) e-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w e-0 e-0)))
      (send (cat (exp (gen) l) (mul w e-0 e-0)))))
  (label 2120)
  (parent 1889)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 2121)
  (parent 1889)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2122)
  (parent 1890)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2123)
  (parent 1890)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2124)
  (parent 1890)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2125)
  (parent 1890)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2126)
  (parent 1890)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2127)
  (parent 1890)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 2128)
  (parent 1891)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-1)))
      (send (cat (gen) (mul eA (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2129)
  (parent 1891)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2130)
  (parent 1891)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2131)
  (parent 1891)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 2132)
  (parent 1891)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (l-0 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee e-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e-1 (exp (gen) l)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 2133)
  (parent 1892)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (l-0 e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee (mul e-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) e-1 l-1)))
  (deflistener (cat (exp (gen) e-1) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e-1 (exp (gen) l)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul e (rec e-0) e-1 l-1) (one)) ((mul e-1 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-1)) (send (cat (exp (gen) e-1) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2134)
  (parent 1892)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll l-0) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 2135)
  (parent 1892)
  (unrealized (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 2136)
  (parent 1892)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (l-0 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 2137)
  (parent 1893)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (l-0 eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul eA e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2138)
  (parent 1893)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2139)
  (parent 1893)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 2140)
  (parent 1893)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2141)
  (parent 1893)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 2142)
  (parent 1894)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e eA e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e eA (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e eA (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2143)
  (parent 1894)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2144)
  (parent 1894)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA rndx) (w expt)
    (l-0 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2145)
  (parent 1894)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2146)
  (parent 1895)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 2147)
  (parent 1895)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (e e-0 rndx) (w expt)
    (e-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2148)
  (parent 1895)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2149)
  (parent 1895)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2150)
  (parent 1895)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2151)
  (parent 1895)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-0))) (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0))))))
  (label 2152)
  (parent 1896)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (eA e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2153)
  (parent 1896)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-0))
    (mul e (rec e-0) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2154)
  (parent 1896)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-0))
    (mul (rec eA) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 2155)
  (parent 1896)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 e-0) (one)) ((mul (rec eA) e w l-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0)))))
  (label 2156)
  (parent 1896)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2157)
  (parent 1896)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2158)
  (parent 1897)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2159)
  (parent 1897)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 2160)
  (parent 1897)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2161)
  (parent 1897)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2162)
  (parent 1897)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2163)
  (parent 1897)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2164)
  (parent 1897)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-0 l) (w (mul eA (rec e) l))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul eA (rec e) l)))
      (send (cat (gen) (mul eA (rec e) l)))))
  (label 2165)
  (parent 1898)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l l-0)))
      (send (cat (gen) (mul eA (rec e) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2166)
  (parent 1898)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e (rec l) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2167)
  (parent 1898)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2168)
  (parent 1898)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA e rndx) (w expt) (l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l))) (send (cat (gen) (mul w l)))))
  (label 2169)
  (parent 1898)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul e (rec e-0) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l)) (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l)))
      (send (cat (exp (gen) e) (mul (rec e-0) l)))))
  (label 2170)
  (parent 1899)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2171)
  (parent 1899)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2172)
  (parent 1899)
  (unrealized (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (e-0 l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 l) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l)))
      (send (cat (exp (gen) e-0) (mul w l)))))
  (label 2173)
  (parent 1899)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul eA l (rec e)) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l (rec e) (rec e))) (l-0 l)
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 2174)
  (parent 1900)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA l (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2175)
  (parent 1900)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2176)
  (parent 1900)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (e l rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (l (mul (rec eA) w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e l) (one)) ((mul (rec eA) w e e l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (uniq-orig n)
  (operation nonce-test (added-absence l (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 2177)
  (parent 1900)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2178)
  (parent 1900)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-1 l-0)
      (w (mul eA (rec e) l-0 (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2179)
  (parent 1901)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l rndx)
    (ll expt) (eA e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2180)
  (parent 1901)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e l-0 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul e (rec l-0) w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w e-0))
    (mul e (rec l-0) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2181)
  (parent 1901)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA l-0 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-0) w e))
    (mul (rec eA) (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2182)
  (parent 1901)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w e-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (l-0 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 l-0) (one)) ((mul (rec eA) e w e-0 l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 l-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 l-0))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2183)
  (parent 1901)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul (rec eA) e (rec l-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2184)
  (parent 1901)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul eA (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul eA (rec e))))
      (send (cat (exp (gen) l) (mul eA (rec e))))))
  (label 2185)
  (parent 1902)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l eA (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2186)
  (parent 1902)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 2187)
  (parent 1902)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))))
  (label 2188)
  (parent 1902)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2189)
  (parent 1903)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2190)
  (parent 1903)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2191)
  (parent 1903)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2192)
  (parent 1903)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2193)
  (parent 1903)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0)))))
  (label 2194)
  (parent 1903)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 2195)
  (parent 1903)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 2196)
  (parent 1904)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-0)))
      (send (cat (gen) (mul eA (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2197)
  (parent 1904)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2198)
  (parent 1904)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2199)
  (parent 1904)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2200)
  (parent 1904)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 2201)
  (parent 1904)
  (unrealized (0 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-1 l-0) (w (mul (rec e-0) l-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 2202)
  (parent 1905)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e (rec e-0) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2203)
  (parent 1905)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self l) (self-0 l-0))
    ("privdh-of" (l-1 self-1) (l self) (l-0 self-0)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2204)
  (parent 1905)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2205)
  (parent 1905)
  (unrealized (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 2206)
  (parent 1905)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul eA l-0 (rec e) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 2207)
  (parent 1906)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2208)
  (parent 1906)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self l) (self-0 l-0))
    ("privdh-of" (l-1 self-1) (l self) (l-0 self-0)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2209)
  (parent 1906)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2210)
  (parent 1906)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 2211)
  (parent 1906)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2212)
  (parent 1906)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2213)
  (parent 1907)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2214)
  (parent 1907)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 rndx) (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 2215)
  (parent 1907)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2216)
  (parent 1907)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2217)
  (parent 1907)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2218)
  (parent 1907)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2219)
  (parent 1907)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 2220)
  (parent 1908)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2221)
  (parent 1908)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self l-0) (self-0 l))
    ("privdh-of" (l-1 self-1) (l-0 self) (l self-0)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2222)
  (parent 1908)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2223)
  (parent 1908)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2224)
  (parent 1908)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 2225)
  (parent 1908)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2226)
  (parent 1908)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 2227)
  (parent 1909)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2228)
  (parent 1909)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2229)
  (parent 1909)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2230)
  (parent 1909)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2231)
  (parent 1909)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2232)
  (parent 1910)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2233)
  (parent 1910)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2234)
  (parent 1910)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 2235)
  (parent 1910)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2236)
  (parent 1910)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2237)
  (parent 1910)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))))
  (label 2238)
  (parent 1910)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2239)
  (parent 1910)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2240)
  (parent 1915)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2241)
  (parent 1915)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2242)
  (parent 1915)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2243)
  (parent 1915)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2244)
  (parent 1916)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2245)
  (parent 1916)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2246)
  (parent 1916)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 2247)
  (parent 1916)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2248)
  (parent 1917)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2249)
  (parent 1917)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2250)
  (parent 1917)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2251)
  (parent 1917)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2252)
  (parent 1918)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2253)
  (parent 1918)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2254)
  (parent 1918)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2255)
  (parent 1918)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2256)
  (parent 1918)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2257)
  (parent 1919)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2258)
  (parent 1919)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2259)
  (parent 1919)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2260)
  (parent 1919)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2261)
  (parent 1919)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2262)
  (parent 1920)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2263)
  (parent 1920)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2264)
  (parent 1920)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2265)
  (parent 1920)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2266)
  (parent 1920)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2267)
  (parent 1921)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2268)
  (parent 1921)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2269)
  (parent 1921)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2270)
  (parent 1921)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2271)
  (parent 1921)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2272)
  (parent 1921)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 2273)
  (parent 1922)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2274)
  (parent 1922)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2275)
  (parent 1922)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2276)
  (parent 1922)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 2277)
  (parent 1922)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 2278)
  (parent 1923)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2279)
  (parent 1923)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2280)
  (parent 1923)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2281)
  (parent 1923)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 2282)
  (parent 1923)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 2283)
  (parent 1924)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e e (rec e-0) l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2284)
  (parent 1924)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2285)
  (parent 1924)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 2286)
  (parent 1924)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2287)
  (parent 1924)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2288)
  (parent 1925)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l)) (e (exp (gen) l-1))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2289)
  (parent 1925)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2290)
  (parent 1925)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2291)
  (parent 1925)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2292)
  (parent 1925)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2293)
  (parent 1925)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 2294)
  (parent 1926)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2295)
  (parent 1926)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2296)
  (parent 1926)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 2297)
  (parent 1926)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 2298)
  (parent 1926)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2299)
  (parent 1926)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 2300)
  (parent 1927)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2301)
  (parent 1927)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2302)
  (parent 1927)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 2303)
  (parent 1927)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 2304)
  (parent 1927)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2305)
  (parent 1927)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2306)
  (parent 1928)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l)) (e (exp (gen) l-1))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2307)
  (parent 1928)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2308)
  (parent 1928)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2309)
  (parent 1928)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2310)
  (parent 1928)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2311)
  (parent 1928)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2312)
  (parent 1928)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 2313)
  (parent 1929)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-1)))
      (send (cat (gen) (mul eA (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2314)
  (parent 1929)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2315)
  (parent 1929)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2316)
  (parent 1929)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2317)
  (parent 1929)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 2318)
  (parent 1929)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 2319)
  (parent 1930)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul e-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) e-1 l-1)))
  (deflistener (cat (exp (gen) e-1) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) e-1 l-1) (one)) ((mul e-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-1)) (send (cat (exp (gen) e-1) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2320)
  (parent 1930)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2321)
  (parent 1930)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-1) (ll l-1)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 2322)
  (parent 1930)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 2323)
  (parent 1930)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 2324)
  (parent 1931)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l-0) (ee (mul eA e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2325)
  (parent 1931)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2326)
  (parent 1931)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2327)
  (parent 1931)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 2328)
  (parent 1931)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2329)
  (parent 1931)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 2330)
  (parent 1932)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e eA e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul e eA (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2331)
  (parent 1932)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2332)
  (parent 1932)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2333)
  (parent 1932)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2334)
  (parent 1932)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2335)
  (parent 1933)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 0) (2 0))
    ((7 1) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-2))
    (e (exp (gen) l)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2336)
  (parent 1933)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2337)
  (parent 1933)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2338)
  (parent 1933)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2339)
  (parent 1933)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2340)
  (parent 1933)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2341)
  (parent 1933)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 2342)
  (parent 1934)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e)
    (ll l-0) (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2343)
  (parent 1934)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2344)
  (parent 1934)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2345)
  (parent 1934)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 2346)
  (parent 1934)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 2347)
  (parent 1934)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2348)
  (parent 1934)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 2349)
  (parent 1935)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2350)
  (parent 1935)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2351)
  (parent 1935)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2352)
  (parent 1935)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 2353)
  (parent 1935)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 2354)
  (parent 1935)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2355)
  (parent 1935)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2356)
  (parent 1936)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 0) (2 0))
    ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-2))
    (e (exp (gen) l)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2357)
  (parent 1936)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2358)
  (parent 1936)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2359)
  (parent 1936)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2360)
  (parent 1936)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2361)
  (parent 1936)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2362)
  (parent 1936)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2363)
  (parent 1936)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 2364)
  (parent 1937)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l l-0)))
      (send (cat (gen) (mul eA (rec e) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2365)
  (parent 1937)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2366)
  (parent 1937)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2367)
  (parent 1937)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2368)
  (parent 1937)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 2369)
  (parent 1937)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 2370)
  (parent 1938)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2371)
  (parent 1938)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2372)
  (parent 1938)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2373)
  (parent 1938)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 2374)
  (parent 1938)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 2375)
  (parent 1939)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA l (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2376)
  (parent 1939)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2377)
  (parent 1939)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2378)
  (parent 1939)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 2379)
  (parent 1939)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2380)
  (parent 1939)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2381)
  (parent 1940)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e)
    (ll l-0) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2382)
  (parent 1940)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2383)
  (parent 1940)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2384)
  (parent 1940)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2385)
  (parent 1940)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2386)
  (parent 1940)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2387)
  (parent 1940)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 2388)
  (parent 1941)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l eA (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2389)
  (parent 1941)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2390)
  (parent 1941)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2391)
  (parent 1941)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2392)
  (parent 1941)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 2393)
  (parent 1942)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2394)
  (parent 1942)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2395)
  (parent 1942)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2396)
  (parent 1942)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2397)
  (parent 1942)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 2398)
  (parent 1942)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2399)
  (parent 1942)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2400)
  (parent 1943)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2401)
  (parent 1943)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2402)
  (parent 1943)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2403)
  (parent 1943)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2404)
  (parent 1943)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2405)
  (parent 1943)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 2406)
  (parent 1943)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2407)
  (parent 1943)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 2408)
  (parent 1944)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-0)))
      (send (cat (gen) (mul eA (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2409)
  (parent 1944)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2410)
  (parent 1944)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2411)
  (parent 1944)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2412)
  (parent 1944)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 2413)
  (parent 1944)
  (unrealized (0 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 2414)
  (parent 1945)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2415)
  (parent 1945)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2416)
  (parent 1945)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2417)
  (parent 1945)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 2418)
  (parent 1945)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 2419)
  (parent 1946)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2420)
  (parent 1946)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2421)
  (parent 1946)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2422)
  (parent 1946)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 2423)
  (parent 1946)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2424)
  (parent 1946)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2425)
  (parent 1947)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2426)
  (parent 1947)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2427)
  (parent 1947)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2428)
  (parent 1947)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2429)
  (parent 1947)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2430)
  (parent 1947)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2431)
  (parent 1947)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 2432)
  (parent 1948)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2433)
  (parent 1948)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2434)
  (parent 1948)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2435)
  (parent 1948)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2436)
  (parent 1948)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 2437)
  (parent 1948)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2438)
  (parent 1948)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 2439)
  (parent 1949)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2440)
  (parent 1949)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2441)
  (parent 1949)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2442)
  (parent 1949)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2443)
  (parent 1949)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2444)
  (parent 1950)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2445)
  (parent 1950)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2446)
  (parent 1950)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2447)
  (parent 1950)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2448)
  (parent 1950)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2449)
  (parent 1950)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 2450)
  (parent 1950)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2451)
  (parent 1950)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 2452)
  (parent 1951)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2453)
  (parent 1951)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2454)
  (parent 1951)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2455)
  (parent 1951)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2456)
  (parent 1951)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2457)
  (parent 1951)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 2458)
  (parent 1951)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-2 l-1) (w (mul (rec e-0) l-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 2459)
  (parent 1952)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2460)
  (parent 1952)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2461)
  (parent 1952)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2462)
  (parent 1952)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2463)
  (parent 1952)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 2464)
  (parent 1952)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 2465)
  (parent 1953)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2466)
  (parent 1953)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2467)
  (parent 1953)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2468)
  (parent 1953)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2469)
  (parent 1953)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 2470)
  (parent 1953)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2471)
  (parent 1953)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2472)
  (parent 1954)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 0) (2 0))
    ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-2))
    (e (exp (gen) l)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2473)
  (parent 1954)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2474)
  (parent 1954)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2475)
  (parent 1954)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2476)
  (parent 1954)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2477)
  (parent 1954)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2478)
  (parent 1954)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2479)
  (parent 1954)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 2480)
  (parent 1955)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e)
    (ll l-0) (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2481)
  (parent 1955)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2482)
  (parent 1955)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2483)
  (parent 1955)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2484)
  (parent 1955)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2485)
  (parent 1955)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 2486)
  (parent 1955)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2487)
  (parent 1955)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 2488)
  (parent 1956)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2489)
  (parent 1956)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2490)
  (parent 1956)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2491)
  (parent 1956)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2492)
  (parent 1956)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2493)
  (parent 1956)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 2494)
  (parent 1956)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2495)
  (parent 1956)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 2496)
  (parent 1957)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2497)
  (parent 1957)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2498)
  (parent 1957)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2499)
  (parent 1957)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2500)
  (parent 1957)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2501)
  (parent 1957)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2502)
  (parent 1958)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 0) (2 0))
    ((7 1) (3 0)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) l)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2503)
  (parent 1958)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2504)
  (parent 1958)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2505)
  (parent 1958)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2506)
  (parent 1958)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2507)
  (parent 1958)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2508)
  (parent 1958)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 2509)
  (parent 1958)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2510)
  (parent 1958)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2511)
  (parent 1963)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2512)
  (parent 1963)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2513)
  (parent 1963)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2514)
  (parent 1963)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2515)
  (parent 1964)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2516)
  (parent 1964)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2517)
  (parent 1964)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 2518)
  (parent 1964)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2519)
  (parent 1965)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2520)
  (parent 1965)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2521)
  (parent 1965)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2522)
  (parent 1965)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2523)
  (parent 1966)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2524)
  (parent 1966)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2525)
  (parent 1966)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2526)
  (parent 1966)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2527)
  (parent 1966)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2528)
  (parent 1967)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2529)
  (parent 1967)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2530)
  (parent 1967)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2531)
  (parent 1967)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2532)
  (parent 1967)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2533)
  (parent 1968)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2534)
  (parent 1968)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2535)
  (parent 1968)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2536)
  (parent 1968)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2537)
  (parent 1968)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2538)
  (parent 1969)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2539)
  (parent 1969)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2540)
  (parent 1969)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2541)
  (parent 1969)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2542)
  (parent 1969)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2543)
  (parent 1969)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 2544)
  (parent 1970)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2545)
  (parent 1970)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2546)
  (parent 1970)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2547)
  (parent 1970)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 2548)
  (parent 1970)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 2549)
  (parent 1971)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2550)
  (parent 1971)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2551)
  (parent 1971)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2552)
  (parent 1971)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 2553)
  (parent 1971)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l) (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 2554)
  (parent 1972)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e e (rec e-0) l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2555)
  (parent 1972)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2556)
  (parent 1972)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 2557)
  (parent 1972)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2558)
  (parent 1972)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2559)
  (parent 1973)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2560)
  (parent 1973)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2561)
  (parent 1973)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2562)
  (parent 1973)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2563)
  (parent 1973)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2564)
  (parent 1973)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 2565)
  (parent 1974)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec e) e-0 e-0 (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2566)
  (parent 1974)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2567)
  (parent 1974)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 2568)
  (parent 1974)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 2569)
  (parent 1974)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2570)
  (parent 1974)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0))))))
  (label 2571)
  (parent 1975)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-0)))
  (deflistener (cat (exp (gen) e-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-0) (one)) ((mul e-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2572)
  (parent 1975)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2573)
  (parent 1975)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0))))))
  (label 2574)
  (parent 1975)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0)))))
  (label 2575)
  (parent 1975)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-0 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2576)
  (parent 1975)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2577)
  (parent 1976)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2578)
  (parent 1976)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2579)
  (parent 1976)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2580)
  (parent 1976)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2581)
  (parent 1976)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2582)
  (parent 1976)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2583)
  (parent 1976)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 2584)
  (parent 1977)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-1)))
      (send (cat (gen) (mul eA (rec e) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2585)
  (parent 1977)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2586)
  (parent 1977)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2587)
  (parent 1977)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 2588)
  (parent 1977)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 2589)
  (parent 1977)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 2590)
  (parent 1978)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 e e-0 e-1 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul e-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) e-1 l-1)))
  (deflistener (cat (exp (gen) e-1) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) e-1 l-1) (one)) ((mul e-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e e-0 e-1 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-1)) (send (cat (exp (gen) e-1) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2591)
  (parent 1978)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (A l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 A) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2592)
  (parent 1978)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 2593)
  (parent 1978)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 2594)
  (parent 1978)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 2595)
  (parent 1979)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2596)
  (parent 1979)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (B l-0) (self l) (A l-1))
    ("privdh-of" (l-2 self-0) (l-0 B) (l self) (l-1 A)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2597)
  (parent 1979)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2598)
  (parent 1979)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 2599)
  (parent 1979)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 2600)
  (parent 1979)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e eA e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 2601)
  (parent 1980)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e eA e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul e eA (rec e-0) l-1)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2602)
  (parent 1980)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2603)
  (parent 1980)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 2604)
  (parent 1980)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2605)
  (parent 1980)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2606)
  (parent 1981)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 0) (2 1))
    ((7 1) (3 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-2)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2607)
  (parent 1981)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2608)
  (parent 1981)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2609)
  (parent 1981)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2610)
  (parent 1981)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 2611)
  (parent 1981)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2612)
  (parent 1981)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 2613)
  (parent 1982)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (eA e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) e-0 (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2614)
  (parent 1982)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2615)
  (parent 1982)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2616)
  (parent 1982)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 2617)
  (parent 1982)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 2618)
  (parent 1982)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2619)
  (parent 1982)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1))))))
  (label 2620)
  (parent 1983)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2621)
  (parent 1983)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2622)
  (parent 1983)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-1))
    (mul e (rec e-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2623)
  (parent 1983)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 2624)
  (parent 1983)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))))
  (label 2625)
  (parent 1983)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2626)
  (parent 1983)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2627)
  (parent 1984)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 0) (2 1))
    ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-2)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2628)
  (parent 1984)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2629)
  (parent 1984)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2630)
  (parent 1984)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2631)
  (parent 1984)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2632)
  (parent 1984)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2633)
  (parent 1984)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2634)
  (parent 1984)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 2635)
  (parent 1985)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l l-0)))
      (send (cat (gen) (mul eA (rec e) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2636)
  (parent 1985)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2637)
  (parent 1985)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2638)
  (parent 1985)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2639)
  (parent 1985)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 2640)
  (parent 1985)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 2641)
  (parent 1986)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e e-0 l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e e-0 l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2642)
  (parent 1986)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2643)
  (parent 1986)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2644)
  (parent 1986)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 2645)
  (parent 1986)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 2646)
  (parent 1987)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA l (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2647)
  (parent 1987)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2648)
  (parent 1987)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2649)
  (parent 1987)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 2650)
  (parent 1987)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2651)
  (parent 1987)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2652)
  (parent 1988)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l rndx) (ll expt)
    (eA e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-0 (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2653)
  (parent 1988)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2654)
  (parent 1988)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2655)
  (parent 1988)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2656)
  (parent 1988)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2657)
  (parent 1988)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2658)
  (parent 1988)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 2659)
  (parent 1989)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l eA (rec e) l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2660)
  (parent 1989)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2661)
  (parent 1989)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2662)
  (parent 1989)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2663)
  (parent 1989)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 2664)
  (parent 1990)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-0) (one)) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2665)
  (parent 1990)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2666)
  (parent 1990)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2667)
  (parent 1990)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2668)
  (parent 1990)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 2669)
  (parent 1990)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2670)
  (parent 1990)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2671)
  (parent 1991)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2672)
  (parent 1991)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2673)
  (parent 1991)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2674)
  (parent 1991)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2675)
  (parent 1991)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2676)
  (parent 1991)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 2677)
  (parent 1991)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2678)
  (parent 1991)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-0)))
      (send (cat (gen) (mul eA (rec e) l-0)))))
  (label 2679)
  (parent 1992)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-0)))
      (send (cat (gen) (mul eA (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2680)
  (parent 1992)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2681)
  (parent 1992)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2682)
  (parent 1992)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2683)
  (parent 1992)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 2684)
  (parent 1992)
  (unrealized (0 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-0)) (l-1 l-0)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0)))))
  (label 2685)
  (parent 1993)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2686)
  (parent 1993)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2687)
  (parent 1993)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2688)
  (parent 1993)
  (unrealized (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))))
  (label 2689)
  (parent 1993)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul eA l-0 (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 2690)
  (parent 1994)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2691)
  (parent 1994)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2692)
  (parent 1994)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2693)
  (parent 1994)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 2694)
  (parent 1994)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2695)
  (parent 1994)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-2 l-1)
      (w (mul eA (rec e) l-1 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2696)
  (parent 1995)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2697)
  (parent 1995)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2698)
  (parent 1995)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w e-0))
    (mul e (rec l-1) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2699)
  (parent 1995)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2700)
  (parent 1995)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2701)
  (parent 1995)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2702)
  (parent 1995)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l)))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul eA (rec e) l) (one)) (l (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l)
      (w (mul eA (rec e) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0))))))
  (label 2703)
  (parent 1996)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec e) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2704)
  (parent 1996)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2705)
  (parent 1996)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w l-0))
    (mul e (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2706)
  (parent 1996)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2707)
  (parent 1996)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w l))
    (mul (rec eA) e w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 2708)
  (parent 1996)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e (rec l) w))
    (mul (rec eA) e (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 2709)
  (parent 1996)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e))))
      (send (cat (exp (gen) l-0) (mul eA (rec e))))))
  (label 2710)
  (parent 1997)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-0)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2711)
  (parent 1997)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2712)
  (parent 1997)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 2713)
  (parent 1997)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))))
  (label 2714)
  (parent 1997)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2715)
  (parent 1998)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2716)
  (parent 1998)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2717)
  (parent 1998)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2718)
  (parent 1998)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2719)
  (parent 1998)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2720)
  (parent 1998)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 2721)
  (parent 1998)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 2722)
  (parent 1998)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 2723)
  (parent 1999)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2724)
  (parent 1999)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2725)
  (parent 1999)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2726)
  (parent 1999)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 2727)
  (parent 1999)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 2728)
  (parent 1999)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 2729)
  (parent 1999)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-2 l-1) (w (mul (rec e-0) l-1))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 2730)
  (parent 2000)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2731)
  (parent 2000)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2732)
  (parent 2000)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l) (self l-1) (B l-0))
    ("privdh-of" (l-2 self-0) (l A) (l-1 self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2733)
  (parent 2000)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 2734)
  (parent 2000)
  (unrealized (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 2735)
  (parent 2000)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 2736)
  (parent 2001)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (B l) (self l-0))
    ("privdh-of" (l-1 self-0) (l B) (l-0 self)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2737)
  (parent 2001)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2738)
  (parent 2001)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (B l) (self l-1) (A l-0))
    ("privdh-of" (l-2 self-0) (l B) (l-1 self) (l-0 A)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2739)
  (parent 2001)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 2740)
  (parent 2001)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 2741)
  (parent 2001)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 2742)
  (parent 2001)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2743)
  (parent 2002)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 0) (2 1))
    ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-2)) (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2744)
  (parent 2002)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2745)
  (parent 2002)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2746)
  (parent 2002)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2747)
  (parent 2002)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2748)
  (parent 2002)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2749)
  (parent 2002)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2750)
  (parent 2002)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 2751)
  (parent 2003)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (eA e l l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul eA (rec e) l l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2752)
  (parent 2003)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2753)
  (parent 2003)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self) (l A)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2754)
  (parent 2003)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2755)
  (parent 2003)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2756)
  (parent 2003)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 2757)
  (parent 2003)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2758)
  (parent 2003)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 2759)
  (parent 2004)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2760)
  (parent 2004)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2761)
  (parent 2004)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (A l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 A) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2762)
  (parent 2004)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2763)
  (parent 2004)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2764)
  (parent 2004)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 2765)
  (parent 2004)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 2766)
  (parent 2004)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 2767)
  (parent 2005)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 2768)
  (parent 2005)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 2769)
  (parent 2005)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2770)
  (parent 2005)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 2771)
  (parent 2005)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 2772)
  (parent 2005)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2773)
  (parent 2006)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 0) (2 1))
    ((7 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-2)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2774)
  (parent 2006)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2775)
  (parent 2006)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 2776)
  (parent 2006)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 2777)
  (parent 2006)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2778)
  (parent 2006)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2779)
  (parent 2006)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 2780)
  (parent 2006)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 2781)
  (parent 2006)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((1 0) (2 0)) ((2 1) (0 3)) ((3 0) (0 0))
    ((3 1) (2 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2782)
  (parent 2009)
  (seen 2028)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3))
    ((7 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 2783)
  (parent 2015)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-1))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3))
    ((7 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-1))
      (send (cat (exp (gen) (mul eA (rec e))) l-1)))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 2784)
  (parent 2015)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3))
    ((7 0) (0 3)) ((7 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 2785)
  (parent 2018)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) e))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) e)) (send (cat (gen) e))))
  (label 2786)
  (parent 2021)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 2787)
  (parent 2021)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e e-0)))
      (send (cat (gen) (mul (rec eA) e e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2788)
  (parent 2021)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1)))))
  (label 2789)
  (parent 2021)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e l-1)))
      (send (cat (gen) (mul (rec eA) e l-1)))))
  (label 2790)
  (parent 2021)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e l-2)))
      (send (cat (gen) (mul (rec eA) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2791)
  (parent 2021)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w (rec w-0))) w-0))))
  (label 2792)
  (parent 2021)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0))))))
  (label 2793)
  (parent 2022)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee e-0))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 2794)
  (parent 2022)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (e-1 (one)) ((mul e (rec e-0) e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2795)
  (parent 2022)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))))
  (label 2796)
  (parent 2022)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq (l-1 (one)) ((mul e (rec e-0) l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-1)))))
  (label 2797)
  (parent 2022)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (l-2 (one)) ((mul e (rec e-0) l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2798)
  (parent 2022)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) w e-0 e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec e) w e-0 e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec e) w e-0 e-0 (rec w-0))) w-0))))
  (label 2799)
  (parent 2022)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((1 0) (0 1)) ((1 0) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2800)
  (parent 2023)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e e-0 l-1 l-2 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-1)
    (ll l-2) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee e-1))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-2))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq (e-1 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0))
  (uniq-gen e e-0 l-1 l-2 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2801)
  (parent 2024)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq (e-0 (one)) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 2802)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA e l-2 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 7 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2803)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-1))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)) ((8 2) (2 3)) ((8 2) (6 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (e-1 (one)) ((mul (rec eA) e e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2804)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2805)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2806)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-3))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)) ((8 0) (2 3)) ((8 0) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (l-3 (one)) ((mul (rec eA) e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-3))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2807)
  (parent 2024)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx)
    (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w e-0) (one)) ((mul w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w e-0)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w e-0 (rec w-0))) w-0))))
  (label 2808)
  (parent 2024)
  (unrealized (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1))))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e (rec l-1))))))
  (label 2809)
  (parent 2025)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 2810)
  (parent 2025)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2811)
  (parent 2025)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e)))))
  (label 2812)
  (parent 2025)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))))
  (label 2813)
  (parent 2025)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2814)
  (parent 2025)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 2815)
  (parent 2025)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0))))))
  (label 2816)
  (parent 2026)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee e))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 2817)
  (parent 2026)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 2) (2 3)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2818)
  (parent 2026)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) l-1)))))
  (label 2819)
  (parent 2026)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-1))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e)))))
  (label 2820)
  (parent 2026)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2821)
  (parent 2026)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-1) (one)) ((mul w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-1)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-1 (rec w-0))) w-0))))
  (label 2822)
  (parent 2026)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l e l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0))
  (uniq-gen e l-0 l-1 l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))))
  (label 2823)
  (parent 2027)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (eA l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee e))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) (mul l-0 l-1 (rec l-2))))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq (e (one)) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1)))))
  (label 2824)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)) ((8 2) (2 3)) ((8 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (e-0 (one)) ((mul (rec eA) e e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2825)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2826)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 2827)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 7 ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2828)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-3))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 0) (2 3)) ((7 0) (6 0)) ((8 0) (2 3)) ((8 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (l-3 (one)) ((mul (rec eA) e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-3))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2829)
  (parent 2027)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) w l-2) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec e) w l-2)) (2 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec e) w l-2 (rec w-0))) w-0))))
  (label 2830)
  (parent 2027)
  (unrealized (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (eA (one)) (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2831)
  (parent 2028)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)) ((6 0) (0 3))
    ((6 1) (5 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 2832)
  (parent 2031)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (3 0))
    ((2 2) (0 3)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (2 3)) ((6 0) (0 3))
    ((6 1) (5 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 2833)
  (parent 2031)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2834)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2835)
  (parent 2038)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2836)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2837)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2838)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2839)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w l-2)))
      (send (cat (gen) (mul (rec eA) e w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2840)
  (parent 2038)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2841)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2842)
  (parent 2045)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2843)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 2844)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2845)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2846)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2) (one)) ((mul e (rec e-0) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2847)
  (parent 2045)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (3 0))
    ((3 1) (0 3)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2848)
  (parent 2046)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2849)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 l-2 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-1)
    (ll l-2) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-2))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e e-0 l-1 l-2 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 2850)
  (parent 2054)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2851)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2852)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 2853)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 2854)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 2855)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) w))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) w)) (send (cat (exp (gen) l-3) w)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2856)
  (parent 2054)
  (unrealized (0 3) (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2857)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w (rec l-1))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e w (rec l-1)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2858)
  (parent 2061)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2859)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2860)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2861)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2862)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2863)
  (parent 2061)
  (unrealized (0 3) (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (deflistener e)
  (precedes ((0 2) (2 3)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (6 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0)))))
    ((recv e) (send e)))
  (label 2864)
  (parent 2062)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2865)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 resp 3) (exp (gen) e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2866)
  (parent 2068)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 7 2 resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2867)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2868)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2869)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2870)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 2871)
  (parent 2068)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-2) w))
      (l-3 l-2) (w-1 w)) (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2872)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e l-0 rndx)
    (w expt) (l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e l-0 l-1 l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 2873)
  (parent 2076)
  (unrealized (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) (mul l-0 l-1 (rec l-2))))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 2874)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 2875)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 2876)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 2877)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 2878)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) w))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) w)) (send (cat (exp (gen) l-3) w)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 2879)
  (parent 2076)
  (unrealized (0 3) (5 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (ee expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee ee))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (ee (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) ee))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA ee)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 2880)
  (parent 2077)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l-0) (eA eA) (lB l) (A self) (B B) (ll l) (ee ee))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l e eA)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 2881)
  (parent 2079)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (uniq-gen l eA e)
  (uniq-orig n)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 2882)
  (parent 2080)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2883)
  (parent 2082)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 2884)
  (parent 2085)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 2885)
  (parent 2086)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 2886)
  (parent 2088)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 2887)
  (parent 2089)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 2888)
  (parent 2090)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (gen) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l))) (send (cat (gen) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 2889)
  (parent 2092)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))) ((recv e) (send e)))
  (label 2890)
  (parent 2093)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 2891)
  (parent 2097)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 2892)
  (parent 2098)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 2893)
  (parent 2099)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 2894)
  (parent 2102)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (l e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 2895)
  (parent 2103)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2896)
  (parent 2106)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 2897)
  (parent 2107)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 2898)
  (parent 2110)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2899)
  (parent 2111)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2900)
  (parent 2114)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-1) (ll l)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2901)
  (parent 2115)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 2902)
  (parent 2116)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e-0)) (one)) ((mul (rec e) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec e-0))))
      (send (cat (exp (gen) l) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2903)
  (parent 2119)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul e w l)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l e-0 e-0) (one)) ((mul e w l e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l)))
      (send (cat (exp (gen) e-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w e-0 e-0)))
      (send (cat (exp (gen) l) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 2904)
  (parent 2120)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 2905)
  (parent 2125)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 2906)
  (parent 2126)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 2907)
  (parent 2127)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 2908)
  (parent 2130)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 2909)
  (parent 2131)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 2910)
  (parent 2132)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll l-0) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 2911)
  (parent 2135)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e-0)
    (ll l-0) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-1 (exp (gen) l))
    (e-1 (exp (gen) ll)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 2912)
  (parent 2136)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e rndx) (w expt) (l-0 e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2913)
  (parent 2139)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 2914)
  (parent 2140)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) ll)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2915)
  (parent 2141)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l e rndx) (w expt)
    (l-0 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2916)
  (parent 2144)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l eA rndx) (w expt)
    (l-0 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2917)
  (parent 2145)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (e e-0 rndx) (w expt)
    (e-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 2918)
  (parent 2148)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2919)
  (parent 2149)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e rndx) (w expt)
    (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2920)
  (parent 2150)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 peer-1 name)
    (l l-0 rndx) (ll expt) (l-1 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-1) (l l-1) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 2921)
  (parent 2151)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec e-0) w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e e-0 l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-0)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2922)
  (parent 2154)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2923)
  (parent 2155)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 e-0) (one)) ((mul (rec eA) e w l-0 e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-0))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-0) (mul w e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 2924)
  (parent 2156)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 2925)
  (parent 2161)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 2926)
  (parent 2162)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 2927)
  (parent 2163)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 2928)
  (parent 2164)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e l rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 2929)
  (parent 2167)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA l rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 2930)
  (parent 2168)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2931)
  (parent 2172)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (w expt) (l e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 2932)
  (parent 2176)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 2933)
  (parent 2178)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (e l-0 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e (rec l-0) w e-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w e-0)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 2934)
  (parent 2181)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA l-0 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 2935)
  (parent 2182)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 name) (l rndx) (ll expt)
    (eA e l-0 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (6 0))
    ((6 1) (4 0)))
  (absent (e-0 (mul (rec eA) e (rec l-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 2936)
  (parent 2184)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (e rndx) (w expt) (l eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l eA)))
  (deflistener (cat (exp (gen) l) (mul e w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l eA) (one)) ((mul e w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e w)))
      (send (cat (exp (gen) l) (mul e w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 2937)
  (parent 2187)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer name) (eA rndx) (w expt) (l e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l)) ("privdh-of" (l self)))
  (neq ((mul w l (rec e)) (one)) ((mul (rec eA) w l) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec e))))
      (send (cat (exp (gen) l) (mul w (rec e))))) ((recv e) (send e)))
  (label 2938)
  (parent 2188)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 2939)
  (parent 2192)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 2940)
  (parent 2193)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (6 0))
    ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-0) (send l-0)))
  (label 2941)
  (parent 2195)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 2942)
  (parent 2199)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA l-0 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 2943)
  (parent 2200)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2944)
  (parent 2201)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 2945)
  (parent 2201)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e e) (ll l-0)
    (ee w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll l-0) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2946)
  (parent 2205)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) e))
  (deflistener (cat (exp (gen) e-0) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 e) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e-0 l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) e)) (send (cat (exp (gen) l-0) e)))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2947)
  (parent 2206)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 2948)
  (parent 2206)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l rndx) (w expt)
    (l-0 e eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 2949)
  (parent 2210)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0) (one)) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2950)
  (parent 2211)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 2951)
  (parent 2211)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 2952)
  (parent 2212)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 2953)
  (parent 2216)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 2954)
  (parent 2217)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e-0 l-1) (one)) ((mul (rec eA) e e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2955)
  (parent 2218)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (7 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1)))
  (label 2956)
  (parent 2218)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 2) (3 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 2957)
  (parent 2219)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul e w l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2958)
  (parent 2223)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (self-0 l))
    ("privdh-of" (l-0 self) (l self-0)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 2959)
  (parent 2224)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l l-0) (one)) ((mul (rec eA) e l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 2960)
  (parent 2225)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (6 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l) (self-0 l-0))
    ("privdh-of" (l self) (l-0 self-0)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 2961)
  (parent 2225)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2962)
  (parent 2230)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 2963)
  (parent 2231)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 2964)
  (parent 2236)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 2965)
  (parent 2237)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 2966)
  (parent 2238)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (7 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 2967)
  (parent 2238)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 0) (3 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv l-1) (send l-1)))
  (label 2968)
  (parent 2239)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 2969)
  (parent 2242)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 2970)
  (parent 2243)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2971)
  (parent 2246)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 2972)
  (parent 2250)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 2973)
  (parent 2251)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 2974)
  (parent 2254)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 2975)
  (parent 2255)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 2976)
  (parent 2256)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2977)
  (parent 2259)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 2978)
  (parent 2260)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 2979)
  (parent 2264)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 2980)
  (parent 2265)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 2981)
  (parent 2266)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 2982)
  (parent 2270)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 2983)
  (parent 2271)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 2984)
  (parent 2272)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 2985)
  (parent 2276)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 2986)
  (parent 2277)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2987)
  (parent 2281)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 2988)
  (parent 2282)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 2989)
  (parent 2286)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2990)
  (parent 2287)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2991)
  (parent 2291)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 2992)
  (parent 2292)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 2993)
  (parent 2293)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2994)
  (parent 2297)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 2995)
  (parent 2298)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 2996)
  (parent 2303)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 2997)
  (parent 2304)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 2998)
  (parent 2305)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 2999)
  (parent 2310)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 3000)
  (parent 2311)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3001)
  (parent 2312)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3002)
  (parent 2316)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 3003)
  (parent 2317)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 3004)
  (parent 2318)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-1) (ll l-1)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 3005)
  (parent 2322)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 3006)
  (parent 2323)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3007)
  (parent 2327)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3008)
  (parent 2328)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3009)
  (parent 2329)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3010)
  (parent 2333)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3011)
  (parent 2334)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 3012)
  (parent 2338)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3013)
  (parent 2339)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3014)
  (parent 2340)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 3015)
  (parent 2341)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3016)
  (parent 2345)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3017)
  (parent 2346)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3018)
  (parent 2347)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3019)
  (parent 2352)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3020)
  (parent 2353)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3021)
  (parent 2354)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 3022)
  (parent 2355)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 3023)
  (parent 2360)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 3024)
  (parent 2361)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 3025)
  (parent 2362)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3026)
  (parent 2363)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3027)
  (parent 2367)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3028)
  (parent 2368)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3029)
  (parent 2373)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3030)
  (parent 2378)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3031)
  (parent 2380)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3032)
  (parent 2384)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3033)
  (parent 2385)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3034)
  (parent 2387)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3035)
  (parent 2391)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3036)
  (parent 2392)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3037)
  (parent 2396)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3038)
  (parent 2397)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 3039)
  (parent 2399)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 3040)
  (parent 2404)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 3041)
  (parent 2405)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3042)
  (parent 2407)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3043)
  (parent 2411)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3044)
  (parent 2412)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3045)
  (parent 2413)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 3046)
  (parent 2413)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3047)
  (parent 2417)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) e))
  (deflistener (cat (exp (gen) e-0) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (4 0)))
  (absent (l-0 e) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e-0 l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) e)) (send (cat (exp (gen) l-0) e)))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3048)
  (parent 2418)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3049)
  (parent 2418)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3050)
  (parent 2422)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0) (one)) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3051)
  (parent 2423)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3052)
  (parent 2423)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3053)
  (parent 2424)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3054)
  (parent 2428)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3055)
  (parent 2429)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e-0 l-1) (one)) ((mul (rec eA) e e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3056)
  (parent 2430)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1)))
  (label 3057)
  (parent 2430)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3058)
  (parent 2431)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3059)
  (parent 2435)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3060)
  (parent 2436)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e l)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l l-0) (one)) ((mul (rec eA) e l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3061)
  (parent 2437)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3062)
  (parent 2437)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3063)
  (parent 2442)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3064)
  (parent 2443)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 3065)
  (parent 2448)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 3066)
  (parent 2449)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3067)
  (parent 2450)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3068)
  (parent 2450)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3069)
  (parent 2451)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3070)
  (parent 2456)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3071)
  (parent 2457)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3072)
  (parent 2458)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 3073)
  (parent 2458)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3074)
  (parent 2463)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) e))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 e) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e-0 l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) e)) (send (cat (exp (gen) l-1) e)))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3075)
  (parent 2464)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3076)
  (parent 2464)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3077)
  (parent 2469)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1) (one)) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3078)
  (parent 2470)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3079)
  (parent 2470)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3080)
  (parent 2471)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3081)
  (parent 2476)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3082)
  (parent 2477)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul e-0 l-2) (one)) ((mul (rec eA) e e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3083)
  (parent 2478)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2)))
  (label 3084)
  (parent 2478)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3085)
  (parent 2479)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3086)
  (parent 2484)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3087)
  (parent 2485)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3088)
  (parent 2486)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3089)
  (parent 2486)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3090)
  (parent 2492)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3091)
  (parent 2493)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3092)
  (parent 2494)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3093)
  (parent 2494)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 3094)
  (parent 2495)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3095)
  (parent 2500)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3096)
  (parent 2501)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 3097)
  (parent 2507)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 3098)
  (parent 2508)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3099)
  (parent 2509)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3100)
  (parent 2509)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3101)
  (parent 2510)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3102)
  (parent 2513)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3103)
  (parent 2514)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3104)
  (parent 2517)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3105)
  (parent 2521)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3106)
  (parent 2522)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3107)
  (parent 2525)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3108)
  (parent 2526)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3109)
  (parent 2527)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3110)
  (parent 2530)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3111)
  (parent 2531)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (gen) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-0))) (send (cat (gen) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3112)
  (parent 2535)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3113)
  (parent 2536)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 3114)
  (parent 2537)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 3115)
  (parent 2541)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 3116)
  (parent 2542)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3117)
  (parent 2543)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 3118)
  (parent 2547)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l l-0 e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 3119)
  (parent 2548)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3120)
  (parent 2552)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3121)
  (parent 2553)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3122)
  (parent 2557)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3123)
  (parent 2558)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-1) (ll l)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3124)
  (parent 2562)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-1) (ll l-0)
    (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3125)
  (parent 2563)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-1))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 3126)
  (parent 2564)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3127)
  (parent 2568)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3128)
  (parent 2569)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-0 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e-0)) (one)) ((mul (rec e) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3129)
  (parent 2574)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 e-0 e-0) (one)) ((mul e w l-0 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-0)))
      (send (cat (exp (gen) e-0) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3130)
  (parent 2575)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 3131)
  (parent 2576)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 3132)
  (parent 2581)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e-0) (send e-0)))
  (label 3133)
  (parent 2582)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3134)
  (parent 2583)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3135)
  (parent 2587)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 3136)
  (parent 2588)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 l-1 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 3137)
  (parent 2589)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 3138)
  (parent 2593)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l-1)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 3139)
  (parent 2594)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3140)
  (parent 2598)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA rndx) (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3141)
  (parent 2599)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l rndx) (ll expt)
    (eA e rndx) (w expt) (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) ll))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3142)
  (parent 2600)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 e rndx) (w expt)
    (l-1 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3143)
  (parent 2604)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3144)
  (parent 2605)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 3145)
  (parent 2609)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-1) (ll l-0)
    (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-1)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3146)
  (parent 2610)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3147)
  (parent 2611)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt) (eA e e-0 rndx)
    (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-0) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 3148)
  (parent 2612)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3149)
  (parent 2616)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3150)
  (parent 2617)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3151)
  (parent 2618)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3152)
  (parent 2623)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3153)
  (parent 2624)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 e-0) (one)) ((mul (rec eA) e w l-1 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3154)
  (parent 2625)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 3155)
  (parent 2626)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 3156)
  (parent 2631)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 3157)
  (parent 2632)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e-0) (send e-0)))
  (label 3158)
  (parent 2633)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3159)
  (parent 2634)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3160)
  (parent 2638)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3161)
  (parent 2639)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3162)
  (parent 2644)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3163)
  (parent 2649)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3164)
  (parent 2651)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3165)
  (parent 2655)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3166)
  (parent 2656)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3167)
  (parent 2658)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3168)
  (parent 2662)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3169)
  (parent 2663)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3170)
  (parent 2667)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3171)
  (parent 2668)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 3172)
  (parent 2670)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 3173)
  (parent 2675)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 3174)
  (parent 2676)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3175)
  (parent 2678)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e l-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3176)
  (parent 2682)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA l-0 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3177)
  (parent 2683)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (l-0 (one)) ((mul (rec eA) e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3178)
  (parent 2684)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 3179)
  (parent 2684)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e-0 (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (A l) (B l-0)) ("privdh-of" (l A) (l-0 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3180)
  (parent 2688)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) e))
  (deflistener (cat (exp (gen) e-0) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (4 0)))
  (absent (l-0 e) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e-0 l-0) (one)) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) e)) (send (cat (exp (gen) l-0) e)))
    ((recv (cat (exp (gen) e-0) l-0)) (send (cat (exp (gen) e-0) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3181)
  (parent 2689)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3182)
  (parent 2689)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (w expt) (l l-0 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3183)
  (parent 2693)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0) (one)) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3184)
  (parent 2694)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3185)
  (parent 2694)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3186)
  (parent 2695)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (e l-1 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-1) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w e-0)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3187)
  (parent 2699)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA l-1 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3188)
  (parent 2700)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e-0 l-1) (one)) ((mul (rec eA) e e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3189)
  (parent 2701)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1)))
  (label 3190)
  (parent 2701)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 name) (l l-0 rndx) (ll expt)
    (eA e l-1 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (7 0)) ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) e (rec l-1) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3191)
  (parent 2702)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul e (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3192)
  (parent 2706)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (A l)) ("privdh-of" (l-0 B) (l A)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3193)
  (parent 2707)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (0 0)) ((5 0) (2 1))
    ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e l)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l l-0) (one)) ((mul (rec eA) e l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3194)
  (parent 2708)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (6 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (A l-0)) ("privdh-of" (l B) (l-0 A)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 3195)
  (parent 2708)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3196)
  (parent 2713)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (B l)) ("privdh-of" (l-0 A) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec e))))) ((recv e) (send e)))
  (label 3197)
  (parent 2714)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 3198)
  (parent 2719)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 3199)
  (parent 2720)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 0) (0 0)) ((6 0) (2 1)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3200)
  (parent 2721)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3201)
  (parent 2721)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 3202)
  (parent 2722)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3203)
  (parent 2727)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3204)
  (parent 2728)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (l-1 (one)) ((mul (rec eA) e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3205)
  (parent 2729)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 3206)
  (parent 2729)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (A l-0) (self l) (B l-1))
    ("privdh-of" (l-0 A) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3207)
  (parent 2734)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) e))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 e) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e-0 l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) e)) (send (cat (exp (gen) l-1) e)))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3208)
  (parent 2735)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (A l) (self l-1) (B l-0))
    ("privdh-of" (l A) (l-1 self) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3209)
  (parent 2735)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w e eA)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3210)
  (parent 2740)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul e l-1) (one)) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3211)
  (parent 2741)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (B l) (self l-1) (A l-0))
    ("privdh-of" (l B) (l-1 self) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3212)
  (parent 2741)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l) (A l-1))
    ("privdh-of" (l-0 B) (l self) (l-1 A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3213)
  (parent 2742)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3214)
  (parent 2747)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3215)
  (parent 2748)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul e-0 l-2) (one)) ((mul (rec eA) e e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3216)
  (parent 2749)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2)))
  (label 3217)
  (parent 2749)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 peer-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 2) (4 0)) ((7 2) (8 0))
    ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3218)
  (parent 2750)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3219)
  (parent 2755)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self l-0) (A l))
    ("privdh-of" (l-1 B) (l-0 self) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3220)
  (parent 2756)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3221)
  (parent 2757)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (B l-0) (self l-1) (A l))
    ("privdh-of" (l-0 B) (l-1 self) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3222)
  (parent 2757)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3223)
  (parent 2763)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3224)
  (parent 2764)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-0 l-1) (one)) ((mul (rec eA) e l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3225)
  (parent 2765)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-0) (self l-1) (B l))
    ("privdh-of" (l-0 A) (l-1 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 3226)
  (parent 2765)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (7 0))
    ((3 2) (4 0)) ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (A l-1) (self l-0) (B l))
    ("privdh-of" (l-1 A) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 3227)
  (parent 2766)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3228)
  (parent 2771)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (7 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3229)
  (parent 2772)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 3230)
  (parent 2778)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 3231)
  (parent 2779)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3232)
  (parent 2780)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3233)
  (parent 2780)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 3234)
  (parent 2781)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) eA) e))
  (precedes ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (8 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3))
    ((7 0) (0 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) eA) e))
    (exp (gen) (mul eA (rec e))) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((recv (cat (exp (gen) eA) e)) (send (cat (exp (gen) eA) e))))
  (label 3235)
  (parent 2785)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec e))) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) (rec e)) eA))
  (precedes ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1)) ((1 0) (4 0))
    ((2 2) (0 3)) ((2 2) (8 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (2 3))
    ((7 0) (0 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener (cat (exp (gen) (rec e)) eA))
    (exp (gen) (mul eA (rec e))) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) (mul eA (rec e))) l-2))
      (send (cat (exp (gen) (mul eA (rec e))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((recv (cat (exp (gen) (rec e)) eA))
      (send (cat (exp (gen) (rec e)) eA))))
  (label 3236)
  (parent 2785)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e w)) (w-1 w))
    (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3237)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3238)
  (parent 2792)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 3239)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w e-0)))
      (send (cat (gen) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3240)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3241)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w l-1)))
      (send (cat (gen) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3242)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w l-2)))
      (send (cat (gen) (mul (rec eA) e w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3243)
  (parent 2792)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w-0 (mul e (rec e-0) (rec e-0) w)) (e-2 e-0)
      (w-1 w)) (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3244)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w (rec e-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e w (rec e-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3245)
  (parent 2799)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3246)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-1) (one)) ((mul e (rec e-0) w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w e-1))))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3247)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3248)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e (rec e-0) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1) (one)) ((mul e (rec e-0) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3249)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2) (one)) ((mul e (rec e-0) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3250)
  (parent 2799)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 0) (0 1)) ((1 0) (2 0)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 1) (2 0)) ((4 1) (0 3)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 3251)
  (parent 2800)
  (seen 2848)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w-0 (mul (rec eA) e (rec e-0) w))
      (e-2 e-0) (w-1 w)) (gen) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3252)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 l-2 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-1)
    (ll l-2) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) l-2))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-1) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e e-0 l-1 l-2 e-1)
  (uniq-orig n)
  (operation nonce-test (displaced 9 0 resp 3) (exp (gen) e-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w))))
  (label 3253)
  (parent 2808)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 9 2 resp 3) (exp (gen) e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3254)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 9 7 resp 3) (exp (gen) e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3255)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) w))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1) (one)) ((mul (rec eA) e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) w)) (send (cat (exp (gen) e-1) w)))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3256)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 1 ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 3257)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 3258)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) w))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) w)) (send (cat (exp (gen) l-3) w)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3259)
  (parent 2808)
  (unrealized (0 3) (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3260)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w (rec l-1))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e w (rec l-1)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3261)
  (parent 2815)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 3262)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3263)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3264)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3265)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3266)
  (parent 2815)
  (unrealized (0 3) (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee e-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0))))
  (deflistener e)
  (precedes ((0 2) (2 3)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (7 0)) ((2 5) (0 5)) ((3 1) (2 1))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq (e-0 (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e (rec l-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) e-0))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e (rec l-0)))))
    ((recv e) (send e)))
  (label 3267)
  (parent 2816)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx)
    (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3268)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 resp 3) (exp (gen) e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3269)
  (parent 2822)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (displaced 8 2 resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 3270)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3271)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3272)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 3273)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3274)
  (parent 2822)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w-0 (mul (rec eA) e (rec l-2) w))
      (l-3 l-2) (w-1 w)) (gen) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 3275)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e l-0 rndx)
    (w expt) (l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e l-0 l-1 l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (displaced 9 0 resp 3) (exp (gen) e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 3276)
  (parent 2830)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) (mul l-0 l-1 (rec l-2))))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e) (one)) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (displaced 9 2 resp 3) (exp (gen) e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 3277)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 3) (exp (gen) e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3278)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 1 ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 3279)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 3280)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 7 ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w))))
  (label 3281)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) w))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) w)) (send (cat (exp (gen) l-3) w)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3282)
  (parent 2830)
  (unrealized (0 3) (6 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (e (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization forgot ((neq (eA (one)))))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3283)
  (parent 2831)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (lA l-0) (eA eA) (lB l) (A self) (B B) (ll l) (ee e))))
  (origs (n (0 4))))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3284)
  (parent 2834)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-2)))
      (send (cat (gen) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3285)
  (parent 2834)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 e eA)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3286)
  (parent 2834)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3287)
  (parent 2834)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3288)
  (parent 2835)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3289)
  (parent 2835)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3290)
  (parent 2835)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 3291)
  (parent 2835)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))))
  (label 3292)
  (parent 2836)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3293)
  (parent 2836)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3294)
  (parent 2836)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3295)
  (parent 2836)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3296)
  (parent 2837)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3297)
  (parent 2837)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3298)
  (parent 2837)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3299)
  (parent 2837)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3300)
  (parent 2837)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))))
  (label 3301)
  (parent 2838)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3302)
  (parent 2838)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3303)
  (parent 2838)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3304)
  (parent 2838)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3305)
  (parent 2838)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3306)
  (parent 2839)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3307)
  (parent 2839)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3308)
  (parent 2839)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3309)
  (parent 2839)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3310)
  (parent 2840)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e))))
      (send (cat (exp (gen) l-2) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3311)
  (parent 2840)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3312)
  (parent 2840)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (eA (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-2)) (mul e w l-2)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-2))) (send (cat (gen) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3313)
  (parent 2840)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-2))
    (mul (rec eA) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-2)))
      (send (cat (gen) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3314)
  (parent 2840)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3315)
  (parent 2840)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 3316)
  (parent 2841)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1)))))
  (label 3317)
  (parent 2841)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-2)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3318)
  (parent 2841)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 3319)
  (parent 2841)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 3320)
  (parent 2841)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 3321)
  (parent 2842)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))))
  (label 3322)
  (parent 2842)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-2)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3323)
  (parent 2842)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3324)
  (parent 2842)
  (unrealized (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 3325)
  (parent 2842)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 3326)
  (parent 2843)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (e (exp (gen) l-1)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))))
  (label 3327)
  (parent 2843)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul e e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul e e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3328)
  (parent 2843)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 3329)
  (parent 2843)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l l-0 (rec l-1)))) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 3330)
  (parent 2843)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3331)
  (parent 2844)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3332)
  (parent 2844)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3333)
  (parent 2844)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3334)
  (parent 2844)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-1)
    (ll l-1) (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3335)
  (parent 2844)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3336)
  (parent 2844)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))))
  (label 3337)
  (parent 2845)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))))
  (label 3338)
  (parent 2845)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3339)
  (parent 2845)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 3340)
  (parent 2845)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0)))))
  (label 3341)
  (parent 2845)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3342)
  (parent 2845)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))))
  (label 3343)
  (parent 2846)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))))
  (label 3344)
  (parent 2846)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3345)
  (parent 2846)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 3346)
  (parent 2846)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0)))))
  (label 3347)
  (parent 2846)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-2)))
      (l-3 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3348)
  (parent 2847)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3349)
  (parent 2847)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3350)
  (parent 2847)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3351)
  (parent 2847)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec e) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-2))
    (mul (rec e) (rec e) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3352)
  (parent 2847)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e-0 (mul e w l-2)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 e-0 e-0) (one)) ((mul e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-2)) (mul e w l-2)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-2)))
      (send (cat (exp (gen) e-0) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3353)
  (parent 2847)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-2 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3354)
  (parent 2847)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 3))
    ((2 1) (3 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization deleted (3 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 3355)
  (parent 2848)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 3356)
  (parent 2849)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2)))))
  (label 3357)
  (parent 2849)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-3)))
      (send (cat (gen) (mul eA (rec e) e-0 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3358)
  (parent 2849)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3359)
  (parent 2849)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 3360)
  (parent 2849)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 3361)
  (parent 2849)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (l-1 l-2 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-2)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-1 l-2 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 3362)
  (parent 2850)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) l-1)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))))
  (label 3363)
  (parent 2850)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 l-2 e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee (mul e-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-2)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-1) (self-1 l-0) (self l)
      (B l-2))
    ("privdh-of" (l-3 self-2) (l-1 self-0) (l-0 self-1) (l self)
      (l-2 B)))
  (neq ((mul e (rec e-0) e-1 l-3) (one)) ((mul e-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-1 l-2 e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) l-3)) (send (cat (exp (gen) e-1) l-3)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-3)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3364)
  (parent 2850)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-2)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 3365)
  (parent 2850)
  (unrealized (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 3366)
  (parent 2850)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (l-0 l-1 l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 3367)
  (parent 2851)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l-0 (rec l-1) l-2)) (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l-0 (rec l-1) l-2))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-2)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (self-1 l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l self-1) (l-0 B)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 (rec l-1) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l-2))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))))
  (label 3368)
  (parent 2851)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name) (l rndx)
    (ll expt) (l-0 l-1 l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul eA e (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-2 l-3) (self l-2) (self-1 l) (self-0 l-1)
      (B l-0))
    ("privdh-of" (l-3 self-2) (l-2 self) (l self-1) (l-1 self-0)
      (l-0 B)))
  (neq ((mul eA e (rec e-0) l-3) (one)) ((mul e l-3) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3369)
  (parent 2851)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (e rndx) (w expt) (l-0 l-1 l-2 e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 3370)
  (parent 2851)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA rndx) (w expt) (l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 3371)
  (parent 2851)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 3372)
  (parent 2851)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e eA e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 3373)
  (parent 2852)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2)))))
  (label 3374)
  (parent 2852)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e eA e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e eA (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul e eA (rec e-0) l-3) (one)) ((mul e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e eA e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3375)
  (parent 2852)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 3376)
  (parent 2852)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3377)
  (parent 2852)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3378)
  (parent 2853)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3379)
  (parent 2853)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data)
    (B self peer self-0 self-1 peer-0 self-2 peer-1 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 0) (7 0)) ((9 1) (5 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3380)
  (parent 2853)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3381)
  (parent 2853)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 3382)
  (parent 2853)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 3383)
  (parent 2853)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3384)
  (parent 2853)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))))
  (label 3385)
  (parent 2854)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self-1 l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self-1) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))))
  (label 3386)
  (parent 2854)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (B l-2) (self-1 l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 B) (l-1 self-1) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3387)
  (parent 2854)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3388)
  (parent 2854)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0))))))
  (label 3389)
  (parent 2854)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))))
  (label 3390)
  (parent 2854)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2))))))
  (label 3391)
  (parent 2854)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))))
  (label 3392)
  (parent 2855)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0)))))
  (label 3393)
  (parent 2855)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-0) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3394)
  (parent 2855)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3395)
  (parent 2855)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0))))))
  (label 3396)
  (parent 2855)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))))
  (label 3397)
  (parent 2855)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-3))) (l-4 l-3)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3398)
  (parent 2856)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3399)
  (parent 2856)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3400)
  (parent 2856)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 0) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3401)
  (parent 2856)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (eA (mul e (rec e-0) w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-3))
    (mul e (rec e-0) w l-3) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3402)
  (parent 2856)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-3)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e-0)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-3))
    (mul (rec eA) (rec e) w l-3) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-3 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3403)
  (parent 2856)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e-0 (mul (rec eA) e w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 e-0) (one)) ((mul (rec eA) e w l-3 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-3))
    (mul (rec eA) e w l-3) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-3 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w e-0)))
      (send (cat (exp (gen) l-3) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3404)
  (parent 2856)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3405)
  (parent 2856)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3406)
  (parent 2857)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1)))))
  (label 3407)
  (parent 2857)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3408)
  (parent 2857)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3409)
  (parent 2857)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3410)
  (parent 2857)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 3411)
  (parent 2857)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-1)) (l-2 l-1)) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3412)
  (parent 2858)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))))
  (label 3413)
  (parent 2858)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3414)
  (parent 2858)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul w (rec l-1))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-1)))
    (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3415)
  (parent 2858)
  (unrealized (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 3416)
  (parent 2858)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-1 (rec e) (rec e))) (l-2 l-1)
      (e-0 e)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 3417)
  (parent 2859)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))))
  (label 3418)
  (parent 2859)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-2 self-1) (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3419)
  (parent 2859)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3420)
  (parent 2859)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA l e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 3421)
  (parent 2859)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3422)
  (parent 2859)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3423)
  (parent 2860)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3424)
  (parent 2860)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3425)
  (parent 2860)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3426)
  (parent 2860)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3427)
  (parent 2860)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3428)
  (parent 2860)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3429)
  (parent 2860)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3430)
  (parent 2861)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-0 eA l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-0 eA l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1)))))
  (label 3431)
  (parent 2861)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-1 eA l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-1 eA l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3432)
  (parent 2861)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3433)
  (parent 2861)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3434)
  (parent 2861)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 3435)
  (parent 2862)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0)))))
  (label 3436)
  (parent 2862)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3437)
  (parent 2862)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3438)
  (parent 2862)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3439)
  (parent 2862)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 3440)
  (parent 2862)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3441)
  (parent 2863)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (B l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3442)
  (parent 2863)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3443)
  (parent 2863)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (B l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3444)
  (parent 2863)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3445)
  (parent 2863)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3446)
  (parent 2863)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3447)
  (parent 2863)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3448)
  (parent 2863)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3449)
  (parent 2865)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1)))))
  (label 3450)
  (parent 2865)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3451)
  (parent 2865)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3452)
  (parent 2865)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3453)
  (parent 2865)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-1)) (l-2 l-1)) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3454)
  (parent 2866)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))))
  (label 3455)
  (parent 2866)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3456)
  (parent 2866)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3457)
  (parent 2866)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (5 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 3458)
  (parent 2866)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (self-0 l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 self-0) (l B)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-1 (rec e) (rec e))) (l-2 l-1)
      (e-0 e)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 3459)
  (parent 2867)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))))
  (label 3460)
  (parent 2867)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-0) (self-0 l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self) (l-1 self-0) (l B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3461)
  (parent 2867)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3462)
  (parent 2867)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3463)
  (parent 2867)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3464)
  (parent 2868)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3465)
  (parent 2868)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3466)
  (parent 2868)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3467)
  (parent 2868)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3468)
  (parent 2868)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3469)
  (parent 2868)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 3470)
  (parent 2869)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))))
  (label 3471)
  (parent 2869)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3472)
  (parent 2869)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3473)
  (parent 2869)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3474)
  (parent 2869)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3475)
  (parent 2869)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3476)
  (parent 2870)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1)))))
  (label 3477)
  (parent 2870)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3478)
  (parent 2870)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3479)
  (parent 2870)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3480)
  (parent 2870)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3481)
  (parent 2871)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3482)
  (parent 2871)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3483)
  (parent 2871)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3484)
  (parent 2871)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3485)
  (parent 2871)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3486)
  (parent 2871)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3487)
  (parent 2871)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-2)))
      (send (cat (gen) (mul eA (rec e) l-2)))))
  (label 3488)
  (parent 2872)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2)))))
  (label 3489)
  (parent 2872)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-2 l-2)))
      (send (cat (gen) (mul eA (rec e) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3490)
  (parent 2872)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-2 l-3)))
      (send (cat (gen) (mul eA (rec e) l-2 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3491)
  (parent 2872)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e l-2 rndx)
    (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w))
    (mul e (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3492)
  (parent 2872)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w))
    (mul (rec eA) (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3493)
  (parent 2872)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2)))))
  (label 3494)
  (parent 2872)
  (unrealized (0 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-3 l-2) (w (mul (rec e-0) l-2))) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2)))))
  (label 3495)
  (parent 2873)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))))
  (label 3496)
  (parent 2873)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3497)
  (parent 2873)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-0) (self-1 l-2) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-0 self-0) (l-2 self-1) (l self)
      (l-1 B)))
  (neq ((mul e (rec e-0) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2 l-3)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3498)
  (parent 2873)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 rndx) (w expt)
    (l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l-2)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l-0) w))
    (mul (rec l-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3499)
  (parent 2873)
  (unrealized (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))))
  (label 3500)
  (parent 2873)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul eA l-2 (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-0 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-3 l-2) (w (mul eA l-2 (rec e) (rec e)))
      (e-0 e)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))))
  (label 3501)
  (parent 2874)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (self-1 l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 self-1) (l B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))))
  (label 3502)
  (parent 2874)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3503)
  (parent 2874)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA l-2 (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self l-1) (self-1 l-2) (self-0 l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self) (l-2 self-1) (l-0 self-0)
      (l B)))
  (neq ((mul eA l-2 (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-0 eA l-2 e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-2 (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3504)
  (parent 2874)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 l-2 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3505)
  (parent 2874)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))))
  (label 3506)
  (parent 2874)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3507)
  (parent 2874)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-4 l-3)
      (w (mul eA (rec e) l-3 (rec e-0))) (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3508)
  (parent 2875)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3509)
  (parent 2875)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3510)
  (parent 2875)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 0) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-3 l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3511)
  (parent 2875)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e l-3 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (eA (mul e (rec l-3) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-3 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-3) w e-0))
    (mul e (rec l-3) w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3512)
  (parent 2875)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA l-3 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-3) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-3 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-3) w e))
    (mul (rec eA) (rec l-3) w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3513)
  (parent 2875)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3514)
  (parent 2875)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-3) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-3) w))
    (mul (rec eA) e (rec l-3) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3515)
  (parent 2875)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))))
  (label 3516)
  (parent 2876)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-1 l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self-1 l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self-1) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))))
  (label 3517)
  (parent 2876)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-2 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-2 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3518)
  (parent 2876)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (B l-2) (self-1 l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 B) (l-1 self-1) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-1 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-1 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3519)
  (parent 2876)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3520)
  (parent 2876)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3521)
  (parent 2876)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-1 l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-2 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))))
  (label 3522)
  (parent 2876)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2))))))
  (label 3523)
  (parent 2876)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))))
  (label 3524)
  (parent 2877)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1)))))
  (label 3525)
  (parent 2877)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3526)
  (parent 2877)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-0) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3527)
  (parent 2877)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3528)
  (parent 2877)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3529)
  (parent 2877)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self-1) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))))
  (label 3530)
  (parent 2877)
  (unrealized (0 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e))))
      (send (cat (exp (gen) l-2) (mul eA (rec e))))))
  (label 3531)
  (parent 2878)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2)))))
  (label 3532)
  (parent 2878)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (6 0)) ((7 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3533)
  (parent 2878)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3534)
  (parent 2878)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3535)
  (parent 2878)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3536)
  (parent 2878)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-4 l-2)
      (w (mul eA (rec e) l-2 (rec l-3))) (l-5 l-3)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3537)
  (parent 2879)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3538)
  (parent 2879)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (6 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3539)
  (parent 2879)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (7 0))
    ((8 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3540)
  (parent 2879)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 eA e l-2 l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 0) (7 0)) ((9 1) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3541)
  (parent 2879)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (eA (mul e (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w l-3))
    (mul e (rec l-2) w l-3) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3542)
  (parent 2879)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (e (mul (rec eA) (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w l-3))
    (mul (rec eA) (rec l-2) w l-3) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3543)
  (parent 2879)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3544)
  (parent 2879)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3545)
  (parent 2879)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3546)
  (parent 2893)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e e-0 rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3547)
  (parent 2907)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3548)
  (parent 2928)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e l rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3549)
  (parent 2941)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3550)
  (parent 2945)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3551)
  (parent 2948)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3552)
  (parent 2951)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3553)
  (parent 2956)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3554)
  (parent 2961)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3555)
  (parent 2967)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 rndx)
    (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3556)
  (parent 2968)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3557)
  (parent 2981)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3558)
  (parent 2984)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3559)
  (parent 2998)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3560)
  (parent 3001)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3561)
  (parent 3022)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3562)
  (parent 3026)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3563)
  (parent 3039)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3564)
  (parent 3042)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3565)
  (parent 3046)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3566)
  (parent 3049)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3567)
  (parent 3052)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3568)
  (parent 3057)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3569)
  (parent 3062)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3570)
  (parent 3068)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3571)
  (parent 3069)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3572)
  (parent 3073)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3573)
  (parent 3076)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3574)
  (parent 3079)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3575)
  (parent 3084)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3576)
  (parent 3089)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3577)
  (parent 3093)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3578)
  (parent 3094)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3579)
  (parent 3100)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3580)
  (parent 3101)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3581)
  (parent 3114)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3582)
  (parent 3117)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e e-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3583)
  (parent 3131)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3584)
  (parent 3134)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e e-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3585)
  (parent 3155)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3586)
  (parent 3159)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e l rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e (rec l) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3587)
  (parent 3172)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3588)
  (parent 3175)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3589)
  (parent 3179)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3590)
  (parent 3182)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0) (one)) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3591)
  (parent 3185)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1) (one)) ((mul (rec eA) e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3592)
  (parent 3190)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0) (one)) ((mul (rec eA) e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (displaced 2 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 3593)
  (parent 3195)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3594)
  (parent 3201)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3595)
  (parent 3202)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3596)
  (parent 3206)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3597)
  (parent 3209)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3598)
  (parent 3212)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3599)
  (parent 3217)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3600)
  (parent 3222)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 3601)
  (parent 3226)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee w))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 3602)
  (parent 3227)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3603)
  (parent 3233)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee w))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 3604)
  (parent 3234)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3605)
  (parent 3237)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-2)))
      (send (cat (gen) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3606)
  (parent 3237)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 e eA)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3607)
  (parent 3237)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3608)
  (parent 3237)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3609)
  (parent 3238)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3610)
  (parent 3238)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3611)
  (parent 3238)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (7 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 3612)
  (parent 3238)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))))
  (label 3613)
  (parent 3239)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3614)
  (parent 3239)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3615)
  (parent 3239)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3616)
  (parent 3239)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3617)
  (parent 3240)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3618)
  (parent 3240)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3619)
  (parent 3240)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3620)
  (parent 3240)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3621)
  (parent 3240)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1)))))
  (label 3622)
  (parent 3241)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3623)
  (parent 3241)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3624)
  (parent 3241)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3625)
  (parent 3241)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3626)
  (parent 3241)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3627)
  (parent 3242)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3628)
  (parent 3242)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3629)
  (parent 3242)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3630)
  (parent 3242)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3631)
  (parent 3243)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e))))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e))))
      (send (cat (exp (gen) l-2) (mul eA (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3632)
  (parent 3243)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3633)
  (parent 3243)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (eA (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-2)) (mul e w l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-2))) (send (cat (gen) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3634)
  (parent 3243)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (mul (rec eA) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-2))
    (mul (rec eA) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-2)))
      (send (cat (gen) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3635)
  (parent 3243)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3636)
  (parent 3243)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (gen) (mul (rec e) e-0 e-0)))
      (send (cat (gen) (mul (rec e) e-0 e-0)))))
  (label 3637)
  (parent 3244)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-1)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-1)))))
  (label 3638)
  (parent 3244)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (gen) (mul (rec e) e-0 e-0 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (gen) (mul (rec e) e-0 e-0 l-2)))
      (send (cat (gen) (mul (rec e) e-0 e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3639)
  (parent 3244)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 3640)
  (parent 3244)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))))
  (label 3641)
  (parent 3244)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul (rec e) e-0)) (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0)))))
  (label 3642)
  (parent 3245)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-1)))))
  (label 3643)
  (parent 3245)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec e) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec e) e-0 l-2)))
      (send (cat (exp (gen) e-0) (mul (rec e) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3644)
  (parent 3245)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec e)))
    (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3645)
  (parent 3245)
  (unrealized (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))))
  (label 3646)
  (parent 3245)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul e e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul e e (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 3647)
  (parent 3246)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (e (exp (gen) l-1)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e e (rec e-0) l-1) (one)) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))))
  (label 3648)
  (parent 3246)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul e e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) (mul l l-0 (rec l-1))))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul e e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3649)
  (parent 3246)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 3650)
  (parent 3246)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l l-0 (rec l-1)))) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 3651)
  (parent 3246)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e) (e-3 e-0) (w (mul (rec e) e-0 e-0 (rec e-1)))
      (e-4 e-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3652)
  (parent 3247)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-1) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (exp (gen) l-1)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3653)
  (parent 3247)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul (rec e) e-0 e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3654)
  (parent 3247)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3655)
  (parent 3247)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-1)
    (ll l-1) (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w e-0)) (mul e w e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3656)
  (parent 3247)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence e-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))))
  (label 3657)
  (parent 3247)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))))
  (label 3658)
  (parent 3248)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec e) e-0 e-0 (rec l-0) l-1)))))
  (label 3659)
  (parent 3248)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3660)
  (parent 3248)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 3661)
  (parent 3248)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0)))))
  (label 3662)
  (parent 3248)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-1 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3663)
  (parent 3248)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-1)))
      (l-2 l-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1))))))
  (label 3664)
  (parent 3249)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0)))))
  (label 3665)
  (parent 3249)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3666)
  (parent 3249)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0))))))
  (label 3667)
  (parent 3249)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0)))))
  (label 3668)
  (parent 3249)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0 (rec l-2)))
      (l-3 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3669)
  (parent 3250)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec e) e-0 e-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3670)
  (parent 3250)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec e) e-0 e-0)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3671)
  (parent 3250)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec e) e-0 e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3672)
  (parent 3250)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec e) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w l-2))
    (mul (rec e) (rec e) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3673)
  (parent 3250)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e-0 (mul e w l-2)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 e-0 e-0) (one)) ((mul e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w l-2)) (mul e w l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-2)))
      (send (cat (exp (gen) e-0) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3674)
  (parent 3250)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (added-absence l-2 (mul e (rec e-0) (rec e-0) w))
    (mul e (rec e-0) (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3675)
  (parent 3250)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0)))
      (send (cat (gen) (mul eA (rec e) e-0)))))
  (label 3676)
  (parent 3252)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-2)))
      (send (cat (gen) (mul eA (rec e) e-0 l-2)))))
  (label 3677)
  (parent 3252)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul eA (rec e) e-0 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul eA (rec e) e-0 l-3)))
      (send (cat (gen) (mul eA (rec e) e-0 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3678)
  (parent 3252)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3679)
  (parent 3252)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))))
  (label 3680)
  (parent 3252)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0)))))
  (label 3681)
  (parent 3252)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (l-1 l-2 e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee e-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-2)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul e (rec e-0) e-1) (one)) (e-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-1 l-2 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-2 e-0) (e-3 e-1) (w (mul (rec e-0) e-1))) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (one)))
      (send (cat (exp (gen) e-1) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1)))))
  (label 3682)
  (parent 3253)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (l-1 e e-0 e-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-1)
    (ee (mul e-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-1) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) l-1)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) e-1 l-2) (one)) ((mul e-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen l-1 e e-0 e-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-1) l-2)) (send (cat (exp (gen) e-1) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-2)))))
  (label 3683)
  (parent 3253)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 l-2 e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee (mul e-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul e (rec e-0) e-1 l-3)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) e-1 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-2)) (e-1 (exp (gen) l-0))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-1) (self-1 l-0) (self l)
      (B l-2))
    ("privdh-of" (l-3 self-2) (l-1 self-0) (l-0 self-1) (l self)
      (l-2 B)))
  (neq ((mul e (rec e-0) e-1 l-3) (one)) ((mul e-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-1 l-2 e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) e-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul e e-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) l-3)) (send (cat (exp (gen) e-1) l-3)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) e-1 l-3)))
      (send (cat (exp (gen) e) (mul (rec e-0) e-1 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3684)
  (parent 3253)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-2)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec e) w))
    (mul (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1))))))
  (label 3685)
  (parent 3253)
  (unrealized (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))))
  (label 3686)
  (parent 3253)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (l-0 l-1 l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul eA e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul eA e (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 3687)
  (parent 3254)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-0)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l-0 (rec l-1) l-2)) (ee (mul eA e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l-0 (rec l-1) l-2))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-2)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (self-1 l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l self-1) (l-0 B)))
  (neq ((mul eA e (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 (rec l-1) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l-1))
      (send (cat (exp (gen) (mul l-0 l-2)) l-1)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))))
  (label 3688)
  (parent 3254)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name) (l rndx)
    (ll expt) (l-0 l-1 l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul eA e (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-2 l-3) (self l-2) (self-1 l) (self-0 l-1)
      (B l-0))
    ("privdh-of" (l-3 self-2) (l-2 self) (l self-1) (l-1 self-0)
      (l-0 B)))
  (neq ((mul eA e (rec e-0) l-3) (one)) ((mul e l-3) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul eA e (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3689)
  (parent 3254)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (e rndx) (w expt) (l-0 l-1 l-2 e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 3690)
  (parent 3254)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA rndx) (w expt) (l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 3691)
  (parent 3254)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 3692)
  (parent 3254)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e eA e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e eA (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0)) (one)) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e eA e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul eA (rec e-0)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0))))
      (send (cat (exp (gen) e) (mul eA (rec e-0))))))
  (label 3693)
  (parent 3255)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e eA e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul e eA (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul e eA (rec e-0) l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l e eA e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-2)))))
  (label 3694)
  (parent 3255)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e eA e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul e eA (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul eA (rec e-0) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul e eA (rec e-0) l-3) (one)) ((mul e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e eA e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e eA (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec e-0) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec e-0) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3695)
  (parent 3255)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 3696)
  (parent 3255)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3697)
  (parent 3255)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-2 e) (e-3 e-0)
      (w (mul eA (rec e) e-0 (rec e-1))) (e-4 e-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1))))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3698)
  (parent 3256)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-2) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 e-1 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3699)
  (parent 3256)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data)
    (B self peer self-0 self-1 peer-0 self-2 peer-1 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 e-1 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 0) (8 0)) ((10 1) (6 0)))
  (absent (e-1 (exp (gen) l-3)) (e-1 (exp (gen) ll-0))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4)))
      (send (cat (exp (gen) e-1) (mul eA (rec e) e-0 (rec e-1) l-4))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1)))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3700)
  (parent 3256)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w e-1))
    (mul e (rec e-0) w e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3701)
  (parent 3256)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 3702)
  (parent 3256)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))))
  (label 3703)
  (parent 3256)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-absence e-1 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))))
  (label 3704)
  (parent 3256)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))))
  (label 3705)
  (parent 3257)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self-1 l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self-1) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen eA e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) e-0 (rec l-1) l-2)))))
  (label 3706)
  (parent 3257)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (B l-2) (self-1 l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 B) (l-1 self-1) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3707)
  (parent 3257)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3708)
  (parent 3257)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0))))))
  (label 3709)
  (parent 3257)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))))
  (label 3710)
  (parent 3257)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2))))))
  (label 3711)
  (parent 3257)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-2))) (l-3 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2))))))
  (label 3712)
  (parent 3258)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0)))))
  (label 3713)
  (parent 3258)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-0) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3714)
  (parent 3258)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-2))
    (mul e (rec e-0) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3715)
  (parent 3258)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0))))))
  (label 3716)
  (parent 3258)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))))
  (label 3717)
  (parent 3258)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0)
      (w (mul eA (rec e) e-0 (rec l-3))) (l-4 l-3)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3718)
  (parent 3259)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) e-0 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3719)
  (parent 3259)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 10 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3720)
  (parent 3259)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 0) (8 0)) ((10 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) e-0 (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3721)
  (parent 3259)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (eA (mul e (rec e-0) w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w l-3))
    (mul e (rec e-0) w l-3) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3722)
  (parent 3259)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-3)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e-0)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w l-3))
    (mul (rec eA) (rec e) w l-3) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-3 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3723)
  (parent 3259)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e-0 (mul (rec eA) e w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 e-0) (one)) ((mul (rec eA) e w l-3 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w l-3))
    (mul (rec eA) e w l-3) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-3 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w e-0)))
      (send (cat (exp (gen) l-3) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3724)
  (parent 3259)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e (rec e-0) w))
    (mul (rec eA) e (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3725)
  (parent 3259)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3726)
  (parent 3260)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-0 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-0 l-1)))
      (send (cat (gen) (mul eA (rec e) l-0 l-1)))))
  (label 3727)
  (parent 3260)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3728)
  (parent 3260)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3729)
  (parent 3260)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3730)
  (parent 3260)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 3731)
  (parent 3260)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-1)) (l-2 l-1)) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3732)
  (parent 3261)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e e-0 l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul e (rec e-0) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e e-0 l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-0 l-1)))))
  (label 3733)
  (parent 3261)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3734)
  (parent 3261)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul w (rec l-1))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-1)))
    (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3735)
  (parent 3261)
  (unrealized (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))))
  (label 3736)
  (parent 3261)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-1 (rec e) (rec e))) (l-2 l-1)
      (e-0 e)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 3737)
  (parent 3262)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA l-0 (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l-0 e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))))
  (label 3738)
  (parent 3262)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-2 self-1) (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3739)
  (parent 3262)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3740)
  (parent 3262)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (self-0 l))
    ("privdh-of" (l-0 self) (l-1 B) (l self-0)))
  (neq ((mul w e l-1) (one)) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA l e l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 3741)
  (parent 3262)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3742)
  (parent 3262)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3743)
  (parent 3263)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (eA e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-1 (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3744)
  (parent 3263)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3745)
  (parent 3263)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3746)
  (parent 3263)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3747)
  (parent 3263)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (l-2 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-2) (one)) ((mul (rec eA) e w e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3748)
  (parent 3263)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3749)
  (parent 3263)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3750)
  (parent 3264)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-0 eA l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-0 eA l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) l-1)))))
  (label 3751)
  (parent 3264)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-1 eA l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul l-1 eA l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3752)
  (parent 3264)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3753)
  (parent 3264)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3754)
  (parent 3264)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 3755)
  (parent 3265)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0)))))
  (label 3756)
  (parent 3265)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3757)
  (parent 3265)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3758)
  (parent 3265)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3759)
  (parent 3265)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-0 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul (rec eA) e w l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w l-0 l-1) (one)) ((mul (rec eA) e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w l-0))
    (mul (rec eA) e w l-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 3760)
  (parent 3265)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3761)
  (parent 3266)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-0)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (B l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3762)
  (parent 3266)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3763)
  (parent 3266)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (B l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3764)
  (parent 3266)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3765)
  (parent 3266)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3766)
  (parent 3266)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3767)
  (parent 3266)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3768)
  (parent 3266)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul eA (rec e) l-1)))
      (send (cat (gen) (mul eA (rec e) l-1)))))
  (label 3769)
  (parent 3268)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-1)))
      (send (cat (gen) (mul eA (rec e) l-1 l-1)))))
  (label 3770)
  (parent 3268)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3771)
  (parent 3268)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3772)
  (parent 3268)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3773)
  (parent 3268)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (w (mul (rec e-0) l-1)) (l-2 l-1)) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1)))))
  (label 3774)
  (parent 3269)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))))
  (label 3775)
  (parent 3269)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3776)
  (parent 3269)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul w (rec l-0)))
    (mul w (rec l-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3777)
  (parent 3269)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e w (rec l-0))))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (7 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (6 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w (rec l-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e w (rec l-0)))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv e) (send e)))
  (label 3778)
  (parent 3269)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (self-0 l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 self-0) (l B)))
  (neq ((mul eA l-1 (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-1 (rec e) (rec e))) (l-2 l-1)
      (e-0 e)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 3779)
  (parent 3270)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))))
  (label 3780)
  (parent 3270)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-0) (self-0 l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self) (l-1 self-0) (l B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3781)
  (parent 3270)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3782)
  (parent 3270)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3783)
  (parent 3270)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-3 l-2)
      (w (mul eA (rec e) l-2 (rec e-0))) (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3784)
  (parent 3271)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3785)
  (parent 3271)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3786)
  (parent 3271)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w e-0))
    (mul e (rec l-2) w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3787)
  (parent 3271)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3788)
  (parent 3271)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3789)
  (parent 3271)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-0)
      (w (mul eA (rec e) l-0 (rec l-1))) (l-3 l-1)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1))))))
  (label 3790)
  (parent 3272)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-1 l-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec e) (rec l-0) l-1 l-1)))))
  (label 3791)
  (parent 3272)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3792)
  (parent 3272)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w l-1))
    (mul e (rec l-0) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3793)
  (parent 3272)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3794)
  (parent 3272)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA e l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (l-1 (mul (rec eA) e (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e (rec l-0) w))
    (mul (rec eA) e (rec l-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 3795)
  (parent 3272)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e))))
      (send (cat (exp (gen) l-1) (mul eA (rec e))))))
  (label 3796)
  (parent 3273)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 8 5 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-1)))))
  (label 3797)
  (parent 3273)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3798)
  (parent 3273)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 3799)
  (parent 3273)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))))
  (label 3800)
  (parent 3273)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3801)
  (parent 3274)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))))
  (label 3802)
  (parent 3274)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3803)
  (parent 3274)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3804)
  (parent 3274)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3805)
  (parent 3274)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3806)
  (parent 3274)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))))
  (label 3807)
  (parent 3274)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-2)))
      (send (cat (gen) (mul eA (rec e) l-2)))))
  (label 3808)
  (parent 3275)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (gen) (mul eA (rec e) l-1 l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-1 l-2)))
      (send (cat (gen) (mul eA (rec e) l-1 l-2)))))
  (label 3809)
  (parent 3275)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (gen) (mul eA (rec e) l-2 l-2)))
      (send (cat (gen) (mul eA (rec e) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3810)
  (parent 3275)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul eA (rec e) l-2 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul eA (rec e) l-2 l-3)))
      (send (cat (gen) (mul eA (rec e) l-2 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3811)
  (parent 3275)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e l-2 rndx)
    (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w))
    (mul e (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 3812)
  (parent 3275)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w))
    (mul (rec eA) (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))))
  (label 3813)
  (parent 3275)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2)))))
  (label 3814)
  (parent 3275)
  (unrealized (0 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e-0) (l-3 l-2) (w (mul (rec e-0) l-2))) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2)))))
  (label 3815)
  (parent 3276)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e e-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e (rec e-0) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-2)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l-0 e e-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-2)))))
  (label 3816)
  (parent 3276)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e (rec e-0) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2 l-2)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3817)
  (parent 3276)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e e-0 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-2 l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-0) (self-1 l-2) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-0 self-0) (l-2 self-1) (l self)
      (l-1 B)))
  (neq ((mul e (rec e-0) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-2 l-3)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-2 l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3818)
  (parent 3276)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 rndx) (w expt)
    (l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e-0 (mul (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l-2)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec l-0) w))
    (mul (rec l-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0))))))
  (label 3819)
  (parent 3276)
  (unrealized (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))))
  (label 3820)
  (parent 3276)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul eA l-2 (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-0 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (l-3 l-2) (w (mul eA l-2 (rec e) (rec e)))
      (e-0 e)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e))))))
  (label 3821)
  (parent 3277)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA l-1 (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (self-1 l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 self-1) (l B)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA l-1 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))))
  (label 3822)
  (parent 3277)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3823)
  (parent 3277)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul eA l-2 (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self l-1) (self-1 l-2) (self-0 l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self) (l-2 self-1) (l-0 self-0)
      (l B)))
  (neq ((mul eA l-2 (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-0 eA l-2 e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-2 (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA l-2 (rec e) (rec e) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3824)
  (parent 3277)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 l-2 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 3825)
  (parent 3277)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))))
  (label 3826)
  (parent 3277)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 3827)
  (parent 3277)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (l-4 l-3)
      (w (mul eA (rec e) l-3 (rec e-0))) (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3828)
  (parent 3278)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-2 (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3829)
  (parent 3278)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) (rec e-0) l-3 l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3830)
  (parent 3278)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 0) (8 0)) ((10 1) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-3 l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4)))
      (send (cat (exp (gen) e-0) (mul eA (rec e) l-3 (rec e-0) l-4))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3831)
  (parent 3278)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e l-3 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (eA (mul e (rec l-3) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-3 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-3) w e-0))
    (mul e (rec l-3) w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3832)
  (parent 3278)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA l-3 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-0 (mul (rec eA) (rec l-3) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-3 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec l-3) w e))
    (mul (rec eA) (rec l-3) w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 3833)
  (parent 3278)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w e-0))
    (mul (rec eA) e w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3834)
  (parent 3278)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)))
  (absent (e-0 (mul (rec eA) e (rec l-3) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e (rec l-3) w))
    (mul (rec eA) e (rec l-3) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 3835)
  (parent 3278)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))))
  (label 3836)
  (parent 3279)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l eA e l-0 l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-1 l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (B l-1) (self-1 l-0) (self l))
    ("privdh-of" (l-2 self-0) (l-1 B) (l-0 self-1) (l self)))
  (neq ((mul eA (rec e) l-0 l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen eA e l-0 l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-0 (rec l-1) l-2)))))
  (label 3837)
  (parent 3279)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-2 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-2 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3838)
  (parent 3279)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (B l-2) (self-1 l-1) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-2 B) (l-1 self-1) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-1 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-1 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3839)
  (parent 3279)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3840)
  (parent 3279)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3841)
  (parent 3279)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-1 l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-2 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))))
  (label 3842)
  (parent 3279)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e (rec l-1) w))
    (mul (rec eA) e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2))))))
  (label 3843)
  (parent 3279)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-1)
      (w (mul eA (rec e) l-1 (rec l-2))) (l-4 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2))))))
  (label 3844)
  (parent 3280)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1)))))
  (label 3845)
  (parent 3280)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3846)
  (parent 3280)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-0 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-0) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3847)
  (parent 3280)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w l-2))
    (mul e (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3848)
  (parent 3280)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3849)
  (parent 3280)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self-1) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w l-1))
    (mul (rec eA) e w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))))
  (label 3850)
  (parent 3280)
  (unrealized (0 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e))))
      (send (cat (exp (gen) l-2) (mul eA (rec e))))))
  (label 3851)
  (parent 3281)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-2)))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec e) l-2)))))
  (label 3852)
  (parent 3281)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (7 0)) ((8 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3853)
  (parent 3281)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-3)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-3))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3854)
  (parent 3281)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))))
  (label 3855)
  (parent 3281)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))))
  (label 3856)
  (parent 3281)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-4 l-2)
      (w (mul eA (rec e) l-2 (rec l-3))) (l-5 l-3)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3857)
  (parent 3282)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-1 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-1 l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-1 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-1 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3858)
  (parent 3282)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (7 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec e) (rec l-2) l-3 l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3859)
  (parent 3282)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (8 0)) ((9 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 10 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3860)
  (parent 3282)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 eA e l-2 l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-2 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 0) (8 0)) ((10 1) (6 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-2 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul eA (rec e) l-2 (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 3861)
  (parent 3282)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (eA (mul e (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w l-3))
    (mul e (rec l-2) w l-3) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3862)
  (parent 3282)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (e (mul (rec eA) (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w l-3))
    (mul (rec eA) (rec l-2) w l-3) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3863)
  (parent 3282)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w l-2))
    (mul (rec eA) e w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))))
  (label 3864)
  (parent 3282)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e (rec l-2) w))
    (mul (rec eA) e (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))))
  (label 3865)
  (parent 3282)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 e eA)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3866)
  (parent 3286)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (6 0) (5 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3867)
  (parent 3287)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((recv e-0) (send e-0)))
  (label 3868)
  (parent 3288)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3869)
  (parent 3290)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3870)
  (parent 3294)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3871)
  (parent 3295)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3872)
  (parent 3298)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3873)
  (parent 3299)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3874)
  (parent 3300)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3875)
  (parent 3303)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3876)
  (parent 3304)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3877)
  (parent 3308)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3878)
  (parent 3309)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-2))) (send (cat (gen) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 3879)
  (parent 3313)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-2)))
      (send (cat (gen) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 3880)
  (parent 3314)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 3881)
  (parent 3315)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 3882)
  (parent 3319)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 3883)
  (parent 3320)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3884)
  (parent 3324)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3885)
  (parent 3325)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3886)
  (parent 3329)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l l-0 (rec l-1)))) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3887)
  (parent 3330)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3888)
  (parent 3334)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-1)
    (ll l-1) (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3889)
  (parent 3335)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 3890)
  (parent 3336)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3891)
  (parent 3340)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3892)
  (parent 3341)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3893)
  (parent 3346)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 3894)
  (parent 3347)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec e) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 3895)
  (parent 3352)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul e w l-2)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 e-0 e-0) (one)) ((mul e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-2)))
      (send (cat (exp (gen) e-0) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 3896)
  (parent 3353)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 3897)
  (parent 3354)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5))
    ((2 1) (3 0)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 3898)
  (parent 3355)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3899)
  (parent 3359)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 3900)
  (parent 3360)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 3901)
  (parent 3361)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-2)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 3902)
  (parent 3365)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 3903)
  (parent 3366)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (e rndx) (w expt) (l-0 l-1 l-2 e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3904)
  (parent 3370)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA rndx) (w expt) (l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3905)
  (parent 3371)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3906)
  (parent 3372)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 3907)
  (parent 3376)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3908)
  (parent 3377)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 3909)
  (parent 3381)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3910)
  (parent 3382)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (9 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 3911)
  (parent 3383)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (9 0)) ((9 1) (7 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 3912)
  (parent 3384)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 3913)
  (parent 3388)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3914)
  (parent 3389)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3915)
  (parent 3390)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 3916)
  (parent 3395)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3917)
  (parent 3396)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 3918)
  (parent 3397)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (eA (mul e (rec e-0) w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv eA) (send eA)))
  (label 3919)
  (parent 3402)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-3)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e-0)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-3 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e-0) (send e-0)))
  (label 3920)
  (parent 3403)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (9 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 e-0) (one)) ((mul (rec eA) e w l-3 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-3 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w e-0)))
      (send (cat (exp (gen) l-3) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e-0) (send e-0)))
  (label 3921)
  (parent 3404)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-3)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 3922)
  (parent 3405)
  (unrealized (0 3) (5 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3923)
  (parent 3409)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3924)
  (parent 3410)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul w (rec l-1))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3925)
  (parent 3415)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3926)
  (parent 3420)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3927)
  (parent 3422)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3928)
  (parent 3426)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3929)
  (parent 3427)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3930)
  (parent 3429)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3931)
  (parent 3433)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3932)
  (parent 3434)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3933)
  (parent 3438)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3934)
  (parent 3439)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 3935)
  (parent 3445)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 3936)
  (parent 3446)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 3937)
  (parent 3448)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3938)
  (parent 3452)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3939)
  (parent 3453)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((recv e-0) (send e-0)))
  (label 3940)
  (parent 3454)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1 l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((recv e-0) (send e-0)))
  (label 3941)
  (parent 3455)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3942)
  (parent 3457)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3943)
  (parent 3462)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3944)
  (parent 3463)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3945)
  (parent 3467)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3946)
  (parent 3468)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3947)
  (parent 3469)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3948)
  (parent 3473)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3949)
  (parent 3474)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 3950)
  (parent 3479)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 3951)
  (parent 3480)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 3952)
  (parent 3485)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 3953)
  (parent 3486)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 3954)
  (parent 3487)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e l-2 rndx)
    (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 3955)
  (parent 3492)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 3956)
  (parent 3493)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e)))
  (deflistener (cat (gen) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 1) (6 0)))
  (absent (l-2 (mul (rec eA) e)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3957)
  (parent 3494)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2))))
    ((recv l-2) (send l-2)))
  (label 3958)
  (parent 3494)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 rndx) (w expt)
    (l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l-2)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 3959)
  (parent 3499)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l e l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) e))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 1) (6 0)))
  (absent (l-2 e) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e-0 l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) e)) (send (cat (exp (gen) l-2) e)))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3960)
  (parent 3500)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 3961)
  (parent 3500)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 l-2 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 3962)
  (parent 3505)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (eA l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul e l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 1) (6 0)))
  (absent (l-2 (mul (rec eA) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul e l-2) (one)) ((mul (rec eA) e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3963)
  (parent 3506)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 3964)
  (parent 3506)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l-1))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 3965)
  (parent 3507)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e l-3 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (eA (mul e (rec l-3) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-3 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 3966)
  (parent 3512)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA l-3 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-3) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-3 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 3967)
  (parent 3513)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 0) (5 0))
    ((8 1) (6 0)))
  (absent (l-3 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul e-0 l-3) (one)) ((mul (rec eA) e e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-4 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3968)
  (parent 3514)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3)))
  (label 3969)
  (parent 3514)
  (unrealized (0 3) (5 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 2) (5 0))
    ((8 2) (9 0)) ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-3) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 3970)
  (parent 3515)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 3971)
  (parent 3520)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 3972)
  (parent 3521)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 1) (6 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3973)
  (parent 3522)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-1 l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-2 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 3974)
  (parent 3522)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 3975)
  (parent 3528)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 3976)
  (parent 3529)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 1) (6 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 3977)
  (parent 3530)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self-1) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 3978)
  (parent 3530)
  (unrealized (0 3) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 3979)
  (parent 3535)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 3980)
  (parent 3536)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (eA (mul e (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv eA) (send eA)))
  (label 3981)
  (parent 3542)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e) (send e)))
  (label 3982)
  (parent 3543)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e l-2)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 0) (5 0))
    ((8 1) (6 0)))
  (absent (l-3 (mul (rec eA) e l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 l-3) (one)) ((mul (rec eA) e l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-4 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e l-2))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 3983)
  (parent 3544)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-3) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 3984)
  (parent 3544)
  (unrealized (0 3) (5 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-3)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 3985)
  (parent 3545)
  (unrealized (0 3) (5 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3986)
  (parent 3546)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3987)
  (parent 3546)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3988)
  (parent 3546)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3989)
  (parent 3547)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3990)
  (parent 3547)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3991)
  (parent 3547)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 3992)
  (parent 3547)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3993)
  (parent 3548)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 3994)
  (parent 3548)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3995)
  (parent 3548)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3996)
  (parent 3548)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3997)
  (parent 3548)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 3998)
  (parent 3549)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 3999)
  (parent 3549)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4000)
  (parent 3549)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4001)
  (parent 3549)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 4002)
  (parent 3549)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec e))))
      (send (cat (gen) (mul l-0 eA (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4003)
  (parent 3550)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4004)
  (parent 3550)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4005)
  (parent 3550)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4006)
  (parent 3550)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4007)
  (parent 3551)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4008)
  (parent 3551)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4009)
  (parent 3551)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4010)
  (parent 3551)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4011)
  (parent 3552)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4012)
  (parent 3552)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4013)
  (parent 3552)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4014)
  (parent 3552)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4015)
  (parent 3553)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 self-2 name)
    (l l-0 rndx) (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4016)
  (parent 3553)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4017)
  (parent 3553)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4018)
  (parent 3553)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4019)
  (parent 3553)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 4020)
  (parent 3554)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l) (self l-0))
    ("privdh-of" (l-1 self-1) (l self-0) (l-0 self)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4021)
  (parent 3554)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4022)
  (parent 3554)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4023)
  (parent 3554)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA e rndx) (w expt)
    (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l-0) (e eA)
    (ll l-0) (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l-0)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l) (self l-0))
    ("privdh-of" (l self-0) (l-0 self)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 4024)
  (parent 3554)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4025)
  (parent 3555)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4026)
  (parent 3555)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4027)
  (parent 3555)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4028)
  (parent 3555)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4029)
  (parent 3555)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4030)
  (parent 3555)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4031)
  (parent 3556)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4032)
  (parent 3556)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-2 self-2) (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4033)
  (parent 3556)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4034)
  (parent 3556)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4035)
  (parent 3556)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4036)
  (parent 3556)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4037)
  (parent 3557)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4038)
  (parent 3557)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4039)
  (parent 3557)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4040)
  (parent 3558)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4041)
  (parent 3558)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4042)
  (parent 3558)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4043)
  (parent 3558)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4044)
  (parent 3559)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4045)
  (parent 3559)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4046)
  (parent 3559)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4047)
  (parent 3559)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4048)
  (parent 3560)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e (exp (gen) l-1)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4049)
  (parent 3560)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4050)
  (parent 3560)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4051)
  (parent 3560)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4052)
  (parent 3560)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4053)
  (parent 3561)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4054)
  (parent 3561)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4055)
  (parent 3561)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4056)
  (parent 3561)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4057)
  (parent 3561)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4058)
  (parent 3562)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-2)) (e (exp (gen) l)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4059)
  (parent 3562)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4060)
  (parent 3562)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4061)
  (parent 3562)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4062)
  (parent 3562)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4063)
  (parent 3562)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4064)
  (parent 3563)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4065)
  (parent 3563)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4066)
  (parent 3563)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4067)
  (parent 3563)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4068)
  (parent 3563)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4069)
  (parent 3564)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e)
    (ll l-0) (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4070)
  (parent 3564)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4071)
  (parent 3564)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4072)
  (parent 3564)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4073)
  (parent 3564)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4074)
  (parent 3564)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec e))))
      (send (cat (gen) (mul l-0 eA (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4075)
  (parent 3565)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4076)
  (parent 3565)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4077)
  (parent 3565)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4078)
  (parent 3565)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4079)
  (parent 3566)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4080)
  (parent 3566)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4081)
  (parent 3566)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4082)
  (parent 3566)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4083)
  (parent 3567)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4084)
  (parent 3567)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4085)
  (parent 3567)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4086)
  (parent 3567)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4087)
  (parent 3568)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4088)
  (parent 3568)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4089)
  (parent 3568)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4090)
  (parent 3568)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4091)
  (parent 3568)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4092)
  (parent 3569)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4093)
  (parent 3569)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4094)
  (parent 3569)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4095)
  (parent 3569)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4096)
  (parent 3569)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4097)
  (parent 3570)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4098)
  (parent 3570)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4099)
  (parent 3570)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4100)
  (parent 3570)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4101)
  (parent 3570)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4102)
  (parent 3570)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4103)
  (parent 3571)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4104)
  (parent 3571)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4105)
  (parent 3571)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4106)
  (parent 3571)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4107)
  (parent 3571)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4108)
  (parent 3571)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec e))))
      (send (cat (gen) (mul l-1 eA (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4109)
  (parent 3572)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4110)
  (parent 3572)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4111)
  (parent 3572)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4112)
  (parent 3572)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4113)
  (parent 3572)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4114)
  (parent 3573)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4115)
  (parent 3573)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4116)
  (parent 3573)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4117)
  (parent 3573)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4118)
  (parent 3573)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4119)
  (parent 3574)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4120)
  (parent 3574)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4121)
  (parent 3574)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4122)
  (parent 3574)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4123)
  (parent 3574)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4124)
  (parent 3575)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-2)) (e (exp (gen) l)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4125)
  (parent 3575)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4126)
  (parent 3575)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4127)
  (parent 3575)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4128)
  (parent 3575)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4129)
  (parent 3575)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4130)
  (parent 3576)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e)
    (ll l-0) (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4131)
  (parent 3576)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4132)
  (parent 3576)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4133)
  (parent 3576)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4134)
  (parent 3576)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4135)
  (parent 3576)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4136)
  (parent 3577)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4137)
  (parent 3577)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4138)
  (parent 3577)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4139)
  (parent 3577)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4140)
  (parent 3577)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4141)
  (parent 3577)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4142)
  (parent 3578)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4143)
  (parent 3578)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4144)
  (parent 3578)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4145)
  (parent 3578)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4146)
  (parent 3578)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4147)
  (parent 3578)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4148)
  (parent 3579)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l-2)) (e (exp (gen) l))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4149)
  (parent 3579)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4150)
  (parent 3579)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4151)
  (parent 3579)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4152)
  (parent 3579)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4153)
  (parent 3579)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4154)
  (parent 3579)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4155)
  (parent 3580)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l-2)) (e (exp (gen) l))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4156)
  (parent 3580)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4157)
  (parent 3580)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4158)
  (parent 3580)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4159)
  (parent 3580)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4160)
  (parent 3580)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4161)
  (parent 3580)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4162)
  (parent 3581)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4163)
  (parent 3581)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4164)
  (parent 3581)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4165)
  (parent 3582)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4166)
  (parent 3582)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4167)
  (parent 3582)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4168)
  (parent 3582)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 e e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4169)
  (parent 3583)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4170)
  (parent 3583)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4171)
  (parent 3583)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4172)
  (parent 3583)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4173)
  (parent 3584)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0) (ll l)
    (ee (mul e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul (rec e) e-0 e-0 l-1)))
  (deflistener (cat (exp (gen) e-0) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul (rec e) e-0 e-0 l-1) (one)) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e-0 e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec e) e-0 e-0 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4174)
  (parent 3584)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4175)
  (parent 3584)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4176)
  (parent 3584)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4177)
  (parent 3584)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4178)
  (parent 3585)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4179)
  (parent 3585)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4180)
  (parent 3585)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4181)
  (parent 3585)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4182)
  (parent 3585)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4183)
  (parent 3586)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) e-0 l-2)))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-2)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4184)
  (parent 3586)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4185)
  (parent 3586)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4186)
  (parent 3586)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4187)
  (parent 3586)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4188)
  (parent 3586)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-1 l-0) (w (mul eA (rec e) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4189)
  (parent 3587)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4190)
  (parent 3587)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l) w))
    (mul e (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4191)
  (parent 3587)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4192)
  (parent 3587)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4193)
  (parent 3587)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4194)
  (parent 3588)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul eA (rec e) l-0 l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul eA (rec e) l-0 l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul eA l-0 l-1))))) (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec e) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4195)
  (parent 3588)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4196)
  (parent 3588)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4197)
  (parent 3588)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4198)
  (parent 3588)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4199)
  (parent 3588)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec e))))
      (send (cat (gen) (mul l-0 eA (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4200)
  (parent 3589)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4201)
  (parent 3589)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4202)
  (parent 3589)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4203)
  (parent 3589)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4204)
  (parent 3590)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4205)
  (parent 3590)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4206)
  (parent 3590)
  (unrealized (2 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l e-0 l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-0)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 l-0) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0 l-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-0)))
      (send (cat (exp (gen) e-0) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4207)
  (parent 3590)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l l-0 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4208)
  (parent 3591)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4209)
  (parent 3591)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4210)
  (parent 3591)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4211)
  (parent 3591)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (l-1 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4212)
  (parent 3592)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4213)
  (parent 3592)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4214)
  (parent 3592)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4215)
  (parent 3592)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4216)
  (parent 3592)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA e l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee l))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l eA (rec e))))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul l eA (rec e)) (one)) (l (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4217)
  (parent 3593)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4218)
  (parent 3593)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l)) (mul e w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4219)
  (parent 3593)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4220)
  (parent 3593)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA e rndx) (w expt) (l l-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) e w)) (l (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l) (B l-0))
    ("privdh-of" (l self) (l-0 B)))
  (neq ((mul w l) (one)) ((mul (rec eA) e w l) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (uniq-orig n)
  (operation nonce-test (added-absence l-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) e w)))
      (send (cat (exp (gen) l) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 4221)
  (parent 3593)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4222)
  (parent 3594)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4223)
  (parent 3594)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4224)
  (parent 3594)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4225)
  (parent 3594)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4226)
  (parent 3594)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4227)
  (parent 3594)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4228)
  (parent 3595)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4229)
  (parent 3595)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4230)
  (parent 3595)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4231)
  (parent 3595)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4232)
  (parent 3595)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4233)
  (parent 3595)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec e))))
      (send (cat (gen) (mul l-1 eA (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4234)
  (parent 3596)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec e) l-1)))
      (send (cat (gen) (mul l-0 eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4235)
  (parent 3596)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4236)
  (parent 3596)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4237)
  (parent 3596)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4238)
  (parent 3596)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4239)
  (parent 3597)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul e l-0 (rec e-0) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l-1)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-0 (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4240)
  (parent 3597)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4241)
  (parent 3597)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4242)
  (parent 3597)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (e rndx) (w expt)
    (l l-0 e-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 l-1)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul e w)) (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 l-1) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1)))
      (send (cat (exp (gen) e-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4243)
  (parent 3597)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4244)
  (parent 3598)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4245)
  (parent 3598)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l) (A l-0))
    ("privdh-of" (l-2 self-0) (l-1 self) (l B) (l-0 A)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4246)
  (parent 3598)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4247)
  (parent 3598)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4248)
  (parent 3598)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (l-2 eA e e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4249)
  (parent 3599)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 self-1 name) (l l-0 rndx)
    (ll expt) (l-1 eA e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-2)) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul l-1 eA (rec e) (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4250)
  (parent 3599)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 self-1 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4251)
  (parent 3599)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4252)
  (parent 3599)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4253)
  (parent 3599)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4254)
  (parent 3599)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4255)
  (parent 3600)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l-1)
    (ee (mul l eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l l-1) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul l eA (rec e) l-1) (one)) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec e) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4256)
  (parent 3600)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (B l-1) (A l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 B) (l A)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4257)
  (parent 3600)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4258)
  (parent 3600)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4259)
  (parent 3600)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self l-0) (B l-1) (A l))
    ("privdh-of" (l-0 self) (l-1 B) (l A)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4260)
  (parent 3600)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4261)
  (parent 3601)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4262)
  (parent 3601)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-0) (A l-1) (B l))
    ("privdh-of" (l-2 self-0) (l-0 self) (l-1 A) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4263)
  (parent 3601)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-0)) (mul e w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4264)
  (parent 3601)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 4265)
  (parent 3601)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 4266)
  (parent 3601)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-2 l-1) (w (mul eA (rec e) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4267)
  (parent 3602)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-1)))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4268)
  (parent 3602)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4269)
  (parent 3602)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-0) w))
    (mul e (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4270)
  (parent 3602)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4271)
  (parent 3602)
  (unrealized (0 3) (2 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-absence l-1 (mul (rec eA) e w))
    (mul (rec eA) e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 4272)
  (parent 3602)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4273)
  (parent 3603)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-2))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4274)
  (parent 3603)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4275)
  (parent 3603)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4276)
  (parent 3603)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4277)
  (parent 3603)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4278)
  (parent 3603)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4279)
  (parent 3603)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4280)
  (parent 3604)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 0) (0 0))
    ((8 0) (2 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-2))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4281)
  (parent 3604)
  (unrealized (2 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 0) (5 0))
    ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4282)
  (parent 3604)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4283)
  (parent 3604)
  (unrealized (3 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4284)
  (parent 3604)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4285)
  (parent 3604)
  (unrealized (0 3) (3 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4286)
  (parent 3604)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 e eA)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 4287)
  (parent 3607)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 eA rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (uniq-gen l l-1 eA e)
  (uniq-orig n)
  (precur (7 0) (6 0))
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 4288)
  (parent 3608)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((recv e-0) (send e-0)))
  (label 4289)
  (parent 3609)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4290)
  (parent 3611)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 4291)
  (parent 3615)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 4292)
  (parent 3616)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w e-0))) (send (cat (gen) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 4293)
  (parent 3619)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 4294)
  (parent 3620)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (9 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 4295)
  (parent 3621)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4296)
  (parent 3624)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4297)
  (parent 3625)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-1))) (send (cat (gen) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4298)
  (parent 3629)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4299)
  (parent 3630)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w l-2))) (send (cat (gen) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 4300)
  (parent 3634)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (e (mul (rec eA) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w l-2)))
      (send (cat (gen) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 4301)
  (parent 3635)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (9 0))
    ((9 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 4302)
  (parent 3636)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 4303)
  (parent 3640)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 l-1 e rndx) (w expt)
    (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (gen) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (gen) (mul w e-0 e-0)))
      (send (cat (gen) (mul w e-0 e-0)))) ((recv e-0) (send e-0)))
  (label 4304)
  (parent 3641)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4305)
  (parent 3645)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w e-0)))
      (send (cat (exp (gen) e-0) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 4306)
  (parent 3646)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w e e-0 e-0) (one)) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 4307)
  (parent 3650)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (e rndx) (w expt)
    (l l-0 l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e-0)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) (mul l l-0 (rec l-1)))) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (self-0 l) (B l-0))
    ("privdh-of" (l-1 self) (l self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e l l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4308)
  (parent 3651)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (e-1 (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 4309)
  (parent 3655)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-1)
    (ll l-1) (ee (mul e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-1 e-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w e-1 e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (e-1 (mul e w e-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-1 e-1) (one)) ((mul e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1 e-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-1)) (recv (exp (gen) (mul e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-1 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-1) (mul e w e-0)))
      (send (cat (exp (gen) e-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w e-1 e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1 e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 4310)
  (parent 3656)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (9 0))
    ((9 1) (7 0)))
  (absent (e-1 (mul e (rec e-0) (rec e-0) w)) (e-1 (exp (gen) l-2))
    (e-1 (exp (gen) ll)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 4311)
  (parent 3657)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4312)
  (parent 3661)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 4313)
  (parent 3662)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul (rec e) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 (rec e-0)) (one)) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4314)
  (parent 3667)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-0) (ee (mul e w l-1 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul e w l-1)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul w l-1 e-0 e-0) (one)) ((mul e w l-1 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-1 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w l-1)))
      (send (cat (exp (gen) e-0) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 4315)
  (parent 3668)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec e) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 4316)
  (parent 3673)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul e w l-2)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w l-2 e-0 e-0) (one)) ((mul e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w l-2)))
      (send (cat (exp (gen) e-0) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0 e-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e-0) (send e-0)))
  (label 4317)
  (parent 3674)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (9 0))
    ((9 1) (7 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 4318)
  (parent 3675)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 4319)
  (parent 3679)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (gen) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec e-0))))
      (send (cat (gen) (mul w (rec e-0))))) ((recv e-0) (send e-0)))
  (label 4320)
  (parent 3680)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (gen) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (gen) (mul w e-0))) (send (cat (gen) (mul w e-0))))
    ((recv e-0) (send e-0)))
  (label 4321)
  (parent 3681)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-1)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-1 (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-2)) (e (exp (gen) l-0)) (e (exp (gen) ll))
    (e-1 (exp (gen) l)) (e-1 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 (rec e-1)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv e-1) (send e-1)))
  (label 4322)
  (parent 3685)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (l-1 l-2 e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e-0)
    (ll l-2) (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-0) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-1 (mul e w)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-2))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e-0 e-1) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen e l-1 l-2 e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul e w e-0 e-1))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul e w)))
      (send (cat (exp (gen) e-1) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1)))) ((recv e-1) (send e-1)))
  (label 4323)
  (parent 3686)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (e rndx) (w expt) (l-0 l-1 l-2 e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e-0 eA) (one)) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul w e-0 e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 4324)
  (parent 3690)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA rndx) (w expt) (l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w e e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (e-0 (exp (gen) l))
    (e-0 (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e e-0) (one)) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 4325)
  (parent 3691)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name) (l rndx)
    (ll expt) (eA e rndx) (w expt) (l-0 l-1 l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e-0)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) (mul l-0 l-1 (rec l-2)))) (e (exp (gen) l))
    (e (exp (gen) ll)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4326)
  (parent 3692)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 rndx) (ll expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e w)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 4327)
  (parent 3696)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 rndx) (ll expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) w)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4328)
  (parent 3697)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (e e-0 rndx)
    (w expt) (e-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (eA (mul e (rec e-0) w e-1)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-1 eA) (one)) ((mul e w e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 e-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w e-1)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w e-1))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w eA)))
      (send (cat (exp (gen) e-1) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv eA) (send eA)))
  (label 4329)
  (parent 3701)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-1)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 (rec e-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (e-1 (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e (exp (gen) l-2)) (e (exp (gen) ll))
    (e-1 (exp (gen) l-0)) (e-1 (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 (rec e-1)) (one)) ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-1))
      (recv (exp (gen) (mul w e-0 (rec e-1))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-1))))
      (send (cat (exp (gen) e-0) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 4330)
  (parent 3702)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt) (eA e rndx)
    (w expt) (e-0 e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 e-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 e-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-1) (ll ll))
  (deflistener (cat (exp (gen) e-0) (mul w e-1)))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-0)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (10 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (e-1 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-3))
    (e-0 (exp (gen) ll-0)) (e-1 (exp (gen) l-2)) (e-1 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 e-1) (one)) ((mul (rec eA) e w e-0 e-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w e-1) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 e-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 e-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-1) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) e-1) (mul (rec eA) e w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-1)))
    ((recv (cat (exp (gen) e-0) (mul w e-1)))
      (send (cat (exp (gen) e-0) (mul w e-1))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-0))) ((recv e-1) (send e-1)))
  (label 4331)
  (parent 3703)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 peer-1 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 rndx) (ll-0 expt)
    (eA e e-0 rndx) (w expt) (e-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) e-1) (mul w (rec e-1))))
  (defstrand resp 3 (self self-2) (peer peer-1) (l l-3) (e e-1)
    (ll ll-0))
  (deflistener e-1)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (10 0)) ((10 1) (8 0)))
  (absent (e-1 (mul (rec eA) e (rec e-0) w)) (e-1 (exp (gen) l-3))
    (e-1 (exp (gen) ll-0)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 e-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-1) (mul w (rec e-1)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e-1) (mul w (rec e-1))))
      (send (cat (exp (gen) e-1) (mul w (rec e-1)))))
    ((recv (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll-0) peer-1 (privk peer-1)))
      (send (exp (gen) e-1))) ((recv e-1) (send e-1)))
  (label 4332)
  (parent 3704)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 4333)
  (parent 3708)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4334)
  (parent 3709)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 4335)
  (parent 3710)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec e-0) w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-2)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 4336)
  (parent 3715)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e-0)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-2 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4337)
  (parent 3716)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 e-0)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w l-2)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 e-0) (one)) ((mul (rec eA) e w l-2 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w e-0)))
      (send (cat (exp (gen) l-2) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 4338)
  (parent 3717)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (eA (mul e (rec e-0) w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w l-3)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv eA) (send eA)))
  (label 4339)
  (parent 3722)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w l-3)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e-0)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w l-3 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec e-0)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e-0) (send e-0)))
  (label 4340)
  (parent 3723)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w e-0)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (10 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w l-3)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 e-0) (one)) ((mul (rec eA) e w l-3 e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w e-0) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-3 e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w e-0)))
      (send (cat (exp (gen) l-3) (mul w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e-0) (send e-0)))
  (label 4341)
  (parent 3724)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (10 0)) ((10 1) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-3)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 4342)
  (parent 3725)
  (unrealized (0 3) (6 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 4343)
  (parent 3729)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 4344)
  (parent 3730)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul w (rec l-1))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (B l-1) (self l))
    ("privdh-of" (l-0 self-0) (l-1 B) (l self)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4345)
  (parent 3735)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 4346)
  (parent 3740)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (self-0 l))
    ("privdh-of" (l-1 self) (l-0 B) (l self-0)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 4347)
  (parent 3742)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w e-0 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 4348)
  (parent 3746)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 4349)
  (parent 3747)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (9 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 4350)
  (parent 3749)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4351)
  (parent 3753)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4352)
  (parent 3754)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4353)
  (parent 3758)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (B l-0) (self l))
    ("privdh-of" (l-1 self-0) (l-0 B) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4354)
  (parent 3759)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 4355)
  (parent 3765)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 4356)
  (parent 3766)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (9 0))
    ((9 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 4357)
  (parent 3768)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e l-1 rndx) (w expt)
    (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 4358)
  (parent 3772)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 4359)
  (parent 3773)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1))))
    ((recv e-0) (send e-0)))
  (label 4360)
  (parent 3774)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e e-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-0)
    (ee (mul l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e (rec e-0) l-1 l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-1) (self l) (B l-0))
    ("privdh-of" (l-1 self-0) (l self) (l-0 B)))
  (neq ((mul e (rec e-0) l-1 l-1) (one)) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e e-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul (rec e-0) l-1 l-1)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e (rec e-0) l-1 l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec e-0) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec e-0) l-1 l-1))))
    ((recv e-0) (send e-0)))
  (label 4361)
  (parent 3775)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l rndx) (w expt)
    (l-0 l-1 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e-0 (mul w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4362)
  (parent 3777)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (w expt)
    (l l-0 l-1 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 4363)
  (parent 3782)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 4364)
  (parent 3783)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (e l-2 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e (rec l-2) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-2 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w e-0)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 4365)
  (parent 3787)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA l-2 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 4366)
  (parent 3788)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 name)
    (l l-0 l-1 rndx) (ll expt) (eA e l-2 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (9 0))
    ((9 1) (7 0)))
  (absent (e-0 (mul (rec eA) e (rec l-2) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 4367)
  (parent 3789)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4368)
  (parent 3793)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-0) w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4369)
  (parent 3794)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 4370)
  (parent 3799)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec e))))) ((recv e) (send e)))
  (label 4371)
  (parent 3800)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv eA) (send eA)))
  (label 4372)
  (parent 3805)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv e) (send e)))
  (label 4373)
  (parent 3806)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (9 0))
    ((9 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-2) (send l-2)))
  (label 4374)
  (parent 3807)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e l-2 rndx)
    (w expt) (eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 4375)
  (parent 3812)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w (rec e))))
      (send (cat (gen) (mul w (rec e))))) ((recv e) (send e)))
  (label 4376)
  (parent 3813)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee l-2))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e)))
  (deflistener (cat (gen) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (l-2 (one)) ((mul (rec eA) e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) l-2))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4377)
  (parent 3814)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (gen) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2))))
    ((recv l-2) (send l-2)))
  (label 4378)
  (parent 3814)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 rndx) (w expt)
    (l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e e) (ll l-2)
    (ee w))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-1 l-2)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l-2)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-1 l-2))))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-0) (self l) (B l-2))
    ("privdh-of" (l-1 self-0) (l-0 self-1) (l self) (l-2 B)))
  (neq ((mul w e (rec e-0)) (one)) (w (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w))
      (send
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 4379)
  (parent 3819)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l e l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) e))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 1) (7 0)))
  (absent (l-2 e) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e-0 l-2) (one)) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) e)) (send (cat (exp (gen) l-2) e)))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4380)
  (parent 3820)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-0 l-0) (self-1 l-2) (self l) (B l-1))
    ("privdh-of" (l-0 self-0) (l-2 self-1) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 4381)
  (parent 3820)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 l-2 e eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w e eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w e eA) (one)) ((mul w e e) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen l l-0 l-1 e eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 4382)
  (parent 3825)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (eA l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul e l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul e l-2) (one)) ((mul (rec eA) e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4383)
  (parent 3826)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (self-1 l-2) (self-0 l-0) (B l))
    ("privdh-of" (l-1 self) (l-2 self-1) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 4384)
  (parent 3826)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA l rndx) (w expt)
    (l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-2) (e e)
    (ll (mul l-0 l-1 (rec l-2))) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l-2))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self-1) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-2))
    (e (exp (gen) (mul l-0 l-1 (rec l-2)))) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-2) (self-1 l) (self-0 l-1) (B l-0))
    ("privdh-of" (l-2 self) (l self-1) (l-1 self-0) (l-0 B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (8 0) (6 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l-0 l-1 (rec l-2))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l-2))
      (send (cat (exp (gen) (mul l-0 l-1)) l-2)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 4385)
  (parent 3827)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e l-3 rndx) (w expt) (e-0 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (eA (mul e (rec l-3) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 eA) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-3 e-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e (rec l-3) w e-0)))
      (send (cat (exp (gen) l-3) (mul e (rec l-3) w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 4386)
  (parent 3832)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA l-3 rndx) (w expt) (e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec l-3) w e)) (e (exp (gen) l-2))
    (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e (rec e-0)) (one)) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-3 e e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) (rec l-3) w e))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e) (mul w (rec e-0))))
      (send (cat (exp (gen) e) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 4387)
  (parent 3833)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e e-0)))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 0) (6 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul e-0 l-3) (one)) ((mul (rec eA) e e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-4 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e e-0))))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4388)
  (parent 3834)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (10 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3)))
  (label 4389)
  (parent 3834)
  (unrealized (0 3) (6 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 peer-0 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e l-3 rndx) (w expt) (e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand resp 3 (self self-2) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 2) (6 0)) ((9 2) (10 0)) ((10 1) (8 0)))
  (absent (e-0 (mul (rec eA) e (rec l-3) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e (rec l-3) w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 4390)
  (parent 3835)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e w l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 4391)
  (parent 3840)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (B l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 B) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 4392)
  (parent 3841)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4393)
  (parent 3842)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (B l-1) (self-1 l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 B) (l-2 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (8 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 4394)
  (parent 3842)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 4395)
  (parent 3848)
  (unrealized (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-1) w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 4396)
  (parent 3849)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 l-2) (one)) ((mul (rec eA) e l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-3 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4397)
  (parent 3850)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (9 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self-1 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self-1) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-2) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2)))
  (label 4398)
  (parent 3850)
  (unrealized (0 3) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (eA (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA)))) ((recv eA) (send eA)))
  (label 4399)
  (parent 3855)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul w (rec e))))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec e))))) ((recv e) (send e)))
  (label 4400)
  (parent 3856)
  (unrealized (0 3) (6 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w eA)))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (eA (mul e (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 eA) (one)) ((mul e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w eA)))
      (send (cat (exp (gen) l-3) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv eA) (send eA)))
  (label 4401)
  (parent 3862)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec e))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-2) w l-3)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3 (rec e)) (one)) ((mul (rec eA) w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec e)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-3)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec e)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv e) (send e)))
  (label 4402)
  (parent 3863)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e l-2)))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 0) (6 0)) ((9 1) (7 0)))
  (absent (l-3 (mul (rec eA) e l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 l-3) (one)) ((mul (rec eA) e l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-4 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e l-2))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4403)
  (parent 3864)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (10 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (8 0)) ((10 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-3) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 4404)
  (parent 3864)
  (unrealized (0 3) (6 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (defstrand ltx-gen 1 (self self-2) (l l-3))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 0) (6 0)) ((9 0) (10 0)) ((10 1) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-3)) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2))))
    ((recv l-3) (send l-3)))
  (label 4405)
  (parent 3865)
  (unrealized (0 3) (6 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4406)
  (parent 3881)
  (unrealized (0 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4407)
  (parent 3897)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((2 1) (3 0)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 4408)
  (parent 3898)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-3 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4409)
  (parent 3922)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4410)
  (parent 3937)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4411)
  (parent 3954)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4412)
  (parent 3958)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4413)
  (parent 3961)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4414)
  (parent 3964)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-3 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4415)
  (parent 3969)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4416)
  (parent 3974)
  (unrealized (0 3) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4417)
  (parent 3978)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-3 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4418)
  (parent 3984)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-3 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4419)
  (parent 3985)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 4420)
  (parent 3987)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4421)
  (parent 3988)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 4422)
  (parent 3991)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) e-0 e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 4423)
  (parent 3992)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4424)
  (parent 3995)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4425)
  (parent 3996)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4426)
  (parent 3997)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e l rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 4427)
  (parent 4000)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA l rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4428)
  (parent 4001)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e rndx) (w expt)
    (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 4429)
  (parent 4005)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l eA rndx) (w expt)
    (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4430)
  (parent 4006)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-0 (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 4431)
  (parent 4007)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-0 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e-0) (send e-0)))
  (label 4432)
  (parent 4009)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 4433)
  (parent 4013)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4434)
  (parent 4014)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4435)
  (parent 4017)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4436)
  (parent 4018)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (ll expt) (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self-0) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (8 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4437)
  (parent 4019)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (e rndx) (w expt)
    (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv eA) (send eA)))
  (label 4438)
  (parent 4022)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 name) (eA rndx) (w expt)
    (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l))
    ("privdh-of" (l-0 self-0) (l self)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv e) (send e)))
  (label 4439)
  (parent 4023)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4440)
  (parent 4028)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 4441)
  (parent 4029)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (8 0))
    ((6 1) (4 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4442)
  (parent 4030)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4443)
  (parent 4034)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 4444)
  (parent 4035)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (3 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-0) (self-0 l-1) (self l))
    ("privdh-of" (l-0 self-1) (l-1 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk self) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4445)
  (parent 4036)
  (seen 4648)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4446)
  (parent 4038)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4447)
  (parent 4039)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4448)
  (parent 4042)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4449)
  (parent 4043)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4450)
  (parent 4046)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) e-0 e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4451)
  (parent 4047)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4452)
  (parent 4051)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0 e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4453)
  (parent 4052)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4454)
  (parent 4055)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4455)
  (parent 4056)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4456)
  (parent 4057)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4457)
  (parent 4061)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4458)
  (parent 4062)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4459)
  (parent 4063)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4460)
  (parent 4066)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4461)
  (parent 4067)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4462)
  (parent 4072)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4463)
  (parent 4073)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4464)
  (parent 4077)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4465)
  (parent 4078)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-0 (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4466)
  (parent 4079)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-0 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4467)
  (parent 4081)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4468)
  (parent 4085)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4469)
  (parent 4086)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4470)
  (parent 4089)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e-0)
    (ll l) (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4471)
  (parent 4090)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (8 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4472)
  (parent 4091)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4473)
  (parent 4094)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4474)
  (parent 4095)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4475)
  (parent 4100)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4476)
  (parent 4101)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (8 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4477)
  (parent 4102)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4478)
  (parent 4106)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4479)
  (parent 4107)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (8 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 4480)
  (parent 4108)
  (seen 4649)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4481)
  (parent 4112)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4482)
  (parent 4113)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4483)
  (parent 4114)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4484)
  (parent 4117)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4485)
  (parent 4122)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4486)
  (parent 4123)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4487)
  (parent 4127)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4488)
  (parent 4128)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4489)
  (parent 4129)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4490)
  (parent 4133)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4491)
  (parent 4134)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4492)
  (parent 4139)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4493)
  (parent 4140)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((2 0) (8 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 4494)
  (parent 4141)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4495)
  (parent 4145)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4496)
  (parent 4146)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4497)
  (parent 4147)
  (seen 4650)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4498)
  (parent 4152)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4499)
  (parent 4153)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 4500)
  (parent 4154)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4501)
  (parent 4159)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4502)
  (parent 4160)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (B l-0) (A l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 B) (l A)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 4503)
  (parent 4161)
  (seen 4651)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4504)
  (parent 4163)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4505)
  (parent 4164)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4506)
  (parent 4167)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4507)
  (parent 4168)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4508)
  (parent 4171)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e-0) (ll l)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (mul e w)) (l-0 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-0) e-0 e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e-0 e-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4509)
  (parent 4172)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4510)
  (parent 4176)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e-0) (ll l-0)
    (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 (mul e w)) (l-1 (mul e w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0 e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0 e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4511)
  (parent 4177)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e e-0 rndx) (w expt) (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (eA (mul e (rec e-0) w)) (l-1 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4512)
  (parent 4180)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4513)
  (parent 4181)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-1) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e-0)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e-0))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4514)
  (parent 4182)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e e-0 rndx) (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec e-0) w)) (l-2 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4515)
  (parent 4186)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4516)
  (parent 4187)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 2) (4 0))
    ((5 2) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4517)
  (parent 4188)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e l rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l) (mul e (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e (rec l) w)) (l-0 (mul e (rec l) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l) (mul e (rec l) w)))
      (send (cat (exp (gen) l) (mul e (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4518)
  (parent 4191)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-0) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec e))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4519)
  (parent 4192)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4520)
  (parent 4197)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4521)
  (parent 4198)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w)) (l-0 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4522)
  (parent 4202)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec e))))
      (send (cat (gen) (mul w l-0 (rec e))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4523)
  (parent 4203)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul e l-0 (rec e-0)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-0 (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-0 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4524)
  (parent 4204)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 e-0 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-0 (rec e-0))))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e-0 w) (l-0 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 (rec e-0)) (one)) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-0 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-0 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e-0) (send e-0)))
  (label 4525)
  (parent 4206)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (w expt) (l e l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e l-0 eA) (one)) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4526)
  (parent 4210)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4527)
  (parent 4211)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (e rndx) (w expt) (e-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w e-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (eA (mul e w e-0)) (l-1 (mul e w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e-0 l-1 eA) (one)) ((mul e w e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w e-0)))
      (send (cat (exp (gen) l-1) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 eA)))
      (send (cat (exp (gen) e-0) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4528)
  (parent 4214)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA rndx) (w expt) (e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-1)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4529)
  (parent 4215)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self peer-0 self-0 name) (l l-0 rndx) (ll expt)
    (eA e rndx) (w expt) (l-1 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-0) (e e-0) (ll ll))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 2) (3 0)) ((5 2) (8 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-1 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4530)
  (parent 4216)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (e rndx) (w expt) (l l-0 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul e w l)) (l-0 (mul e w l)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 eA) (one)) ((mul e w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e l l-0 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l l-0 eA))))) (send n))
    ((recv (cat (exp (gen) l-0) (mul e w l)))
      (send (cat (exp (gen) l-0) (mul e w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 4531)
  (parent 4219)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w l l-0 (rec e)) (one)) ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l l-0 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec e))))
      (send (cat (exp (gen) l) (mul w l-0 (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 4532)
  (parent 4220)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4533)
  (parent 4225)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (4 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4534)
  (parent 4226)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 0) (3 0)) ((5 0) (8 0))
    ((6 1) (4 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4535)
  (parent 4227)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4536)
  (parent 4231)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4537)
  (parent 4232)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (8 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 4538)
  (parent 4233)
  (seen 4652)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4539)
  (parent 4237)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 eA rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec e))))
      (send (cat (gen) (mul w l-1 (rec e))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4540)
  (parent 4238)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul e l-1 (rec e-0)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e l-1 (rec e-0))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4541)
  (parent 4239)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e-0) (ll l)
    (ee (mul w e l-1 (rec e-0))))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e-0))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e-0 w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l) (B l-0))
    ("privdh-of" (l-1 self) (l A) (l-0 B)))
  (neq ((mul w e l-1 (rec e-0)) (one)) ((mul w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-1 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-1 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e-0)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e-0) (send e-0)))
  (label 4542)
  (parent 4242)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w e l-1 eA) (one)) ((mul w e e l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4543)
  (parent 4247)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (B l) (A l-0))
    ("privdh-of" (l-1 self) (l B) (l-0 A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4544)
  (parent 4248)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (e rndx) (w expt) (e-0 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w e-0 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 eA)))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w e-0)) (l-2 (mul e w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) ll)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e-0 l-2 eA) (one)) ((mul e w e-0 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e-0 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w e-0 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w e-0)))
      (send (cat (exp (gen) l-2) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 eA)))
      (send (cat (exp (gen) e-0) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4545)
  (parent 4252)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA rndx) (w expt) (e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e-0) (ll l-0)
    (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) ll)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4546)
  (parent 4253)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self peer-0 self-0 name) (l l-0 l-1 rndx)
    (ll expt) (eA e rndx) (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
  (defstrand resp 3 (self self) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 2) (4 0)) ((6 2) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-2 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4547)
  (parent 4254)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4548)
  (parent 4258)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self l-1) (B l-0) (A l))
    ("privdh-of" (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4549)
  (parent 4259)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul e w l-0)) (l-1 (mul e w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 eA) (one)) ((mul e w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-0 l-1 eA))))) (send n))
    ((recv (cat (exp (gen) l-1) (mul e w l-0)))
      (send (cat (exp (gen) l-1) (mul e w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 4550)
  (parent 4264)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (8 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-1) (A l-0) (B l))
    ("privdh-of" (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-0 l-1 (rec e)) (one)) ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 l-1 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec e))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 4551)
  (parent 4265)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((2 0) (8 0))
    ((3 2) (4 0)) ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3))
    ((5 1) (3 3)) ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (A l-1) (B l))
    ("privdh-of" (l-0 self) (l-1 A) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 4552)
  (parent 4266)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-0) (mul e (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (eA (mul e (rec l-0) w)) (l-1 (mul e (rec l-0) w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul e (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4553)
  (parent 4270)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (5 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (e (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-1) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4554)
  (parent 4271)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 0) (3 0)) ((4 0) (8 0)) ((5 1) (2 3))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (2 1)) ((7 1) (6 0))
    ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-0) (self l-1) (B l))
    ("privdh-of" (l-0 self-0) (l-1 self) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 4555)
  (parent 4272)
  (seen 4653)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4556)
  (parent 4277)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4557)
  (parent 4278)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 0) (4 0)) ((6 0) (9 0)) ((7 1) (5 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 4558)
  (parent 4279)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w eA)))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4559)
  (parent 4284)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (9 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4560)
  (parent 4285)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (6 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 0) (4 0))
    ((5 0) (9 0)) ((6 1) (3 3)) ((7 1) (6 0)) ((8 0) (4 0))
    ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-2) (A l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-2 self) (l-0 A) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 4561)
  (parent 4286)
  (seen 4654)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) e w)))
      (send (cat (gen) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4562)
  (parent 4302)
  (unrealized (0 3) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e e-0 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e (rec e-0) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul e (rec e-0) (rec e-0) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq (w (one)) ((mul e (rec e-0) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e (rec e-0) w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) (rec e-0) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4563)
  (parent 4318)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e e-0 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e (rec e-0) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 11 ltx-gen 2) l-3 (10 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4564)
  (parent 4342)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4565)
  (parent 4357)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 8 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4566)
  (parent 4374)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA e rndx)
    (w expt) (l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (gen) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (gen) (mul w l-2))) (send (cat (gen) (mul w l-2))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4567)
  (parent 4378)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4568)
  (parent 4381)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2) (one)) ((mul (rec eA) w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4569)
  (parent 4384)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3) (one)) ((mul (rec eA) e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 11 ltx-gen 2) l-3 (10 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w e-0)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3)))
      (send (cat (exp (gen) e-0) (mul w l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4570)
  (parent 4389)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4571)
  (parent 4394)
  (unrealized (0 3) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2) (one)) ((mul (rec eA) e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4572)
  (parent 4398)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3) (one)) ((mul (rec eA) e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 11 ltx-gen 2) l-3 (10 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3)))
      (send (cat (exp (gen) l-2) (mul w l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4573)
  (parent 4404)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx) (w expt) (l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee w))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq (w (one)) ((mul (rec eA) e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 9 11 ltx-gen 2) l-3 (10 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e)) (recv (exp (gen) w))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4574)
  (parent 4405)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4575)
  (parent 4406)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4576)
  (parent 4406)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4577)
  (parent 4406)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4578)
  (parent 4406)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4579)
  (parent 4407)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4580)
  (parent 4407)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 l-2 e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4581)
  (parent 4407)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-2 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4582)
  (parent 4407)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e-0 (mul e w)) (l-2 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4583)
  (parent 4407)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((1 0) (0 1)) ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((0 2) (3 0)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 4584)
  (parent 4408)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (one)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-3 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0)))
      (send (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4585)
  (parent 4409)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4586)
  (parent 4409)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (5 0)))
  (absent (l-3 l-4) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-3 eA e e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4)))
      (send (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4587)
  (parent 4409)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (eA (mul e (rec e-0) w)) (l-3 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4588)
  (parent 4409)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-3 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4589)
  (parent 4409)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4590)
  (parent 4409)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4591)
  (parent 4410)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-0 l-2) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4592)
  (parent 4410)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-1 l-3) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4593)
  (parent 4410)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4594)
  (parent 4410)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4595)
  (parent 4410)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4596)
  (parent 4410)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4597)
  (parent 4411)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4598)
  (parent 4411)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4599)
  (parent 4411)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4600)
  (parent 4411)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4601)
  (parent 4411)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (gen) (mul l-2 eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (gen) (mul l-2 eA (rec e))))
      (send (cat (gen) (mul l-2 eA (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4602)
  (parent 4412)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4603)
  (parent 4412)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (gen) (mul l-2 eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (gen) (mul l-2 eA (rec e) l-3)))
      (send (cat (gen) (mul l-2 eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4604)
  (parent 4412)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (gen) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (gen) (mul w l-2 eA)))
      (send (cat (gen) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4605)
  (parent 4412)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-2 (rec e))))
      (send (cat (gen) (mul w l-2 (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4606)
  (parent 4412)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e l-2 (rec e-0)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4607)
  (parent 4413)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-2)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4608)
  (parent 4413)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul e l-2 (rec e-0) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4609)
  (parent 4413)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e-0 w) (l-2 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4610)
  (parent 4413)
  (unrealized (2 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (6 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (5 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4611)
  (parent 4413)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4612)
  (parent 4414)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4613)
  (parent 4414)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self l-1) (self-0 l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self) (l-0 self-0)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4614)
  (parent 4414)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (w expt)
    (l l-0 l-1 e l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul w e e)) (l-2 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2 eA) (one)) ((mul w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul w e e)))
      (send (cat (exp (gen) l-2) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4615)
  (parent 4414)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4616)
  (parent 4414)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-3 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (one)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-3 eA (rec e)) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-3 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-3 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4617)
  (parent 4415)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4618)
  (parent 4415)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-3 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (deflistener (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (5 0)))
  (absent (l-3 l-4) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-3 eA (rec e) l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-3 eA e e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-3 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((recv (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4)))
      (send (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4619)
  (parent 4415)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (eA (mul e w e-0)) (l-3 (mul e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3 eA) (one)) ((mul e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w e-0)))
      (send (cat (exp (gen) l-3) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 eA)))
      (send (cat (exp (gen) e-0) (mul w l-3 eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4620)
  (parent 4415)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-3 (mul (rec eA) w e))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e l-3 (rec e-0)) (one)) ((mul (rec eA) w e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-3 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4621)
  (parent 4415)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4622)
  (parent 4415)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4623)
  (parent 4416)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-0 l-2) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul l-0 eA l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul l-0 eA l-2))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4624)
  (parent 4416)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-1 l-3) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul l-1 eA l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul l-1 eA l-3))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4625)
  (parent 4416)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4626)
  (parent 4416)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4627)
  (parent 4416)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul (rec l) l-2)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-2)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-2)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4628)
  (parent 4416)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4629)
  (parent 4417)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4630)
  (parent 4417)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-2))
      (send (cat (exp (gen) (mul l (rec l-0))) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4631)
  (parent 4417)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4632)
  (parent 4417)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4633)
  (parent 4417)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-3)))
      (l-4 l-3)) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4634)
  (parent 4418)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-2) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (send l-1)))
  (label 4635)
  (parent 4418)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 10 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e))))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4636)
  (parent 4418)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 eA e l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (5 0)))
  (absent (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e) l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4637)
  (parent 4418)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (eA (mul e w l-2)) (l-3 (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 eA) (one)) ((mul e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-2)) (mul e w l-2)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w l-2)))
      (send (cat (exp (gen) l-3) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 eA)))
      (send (cat (exp (gen) l-2) (mul w l-3 eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4638)
  (parent 4418)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) w l-2)) (l-3 (mul (rec eA) w l-2))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 (rec e)) (one)) ((mul (rec eA) w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-2))
    (mul (rec eA) w l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-3 (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4639)
  (parent 4418)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4640)
  (parent 4418)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-4 l-3) (w (mul eA (rec e) l-3)))
    (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4641)
  (parent 4419)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-2) (l l-1))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 10 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (send l-1)))
  (label 4642)
  (parent 4419)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 0) (6 0)) ((9 1) (5 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4643)
  (parent 4419)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 eA e l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (5 0)))
  (absent (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-3 l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4644)
  (parent 4419)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (eA (mul e (rec l-2) w)) (l-3 (mul e (rec l-2) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w))
    (mul e (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4645)
  (parent 4419)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-2) w))
    (l-3 (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w))
    (mul (rec eA) (rec l-2) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4646)
  (parent 4419)
  (unrealized (0 3) (2 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w))
    (mul (rec eA) e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4647)
  (parent 4419)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4648)
  (parent 4442)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4649)
  (parent 4477)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4650)
  (parent 4494)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4651)
  (parent 4500)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4652)
  (parent 4535)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA e rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) e w)) (l-0 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0) (one)) ((mul (rec eA) e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (uniq-orig n)
  (operation nonce-test (displaced 2 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4653)
  (parent 4552)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA e rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4654)
  (parent 4558)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4655)
  (parent 4562)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv (cat (gen) l-3)) (send (cat (gen) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4656)
  (parent 4562)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4657)
  (parent 4562)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4658)
  (parent 4562)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (one)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul (rec e) e-0 e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 l-2 e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (e-1 e) (e-2 e-0) (w (mul (rec e) e-0 e-0))) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4659)
  (parent 4563)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e-0)
    (ll l-0) (ee (mul e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul (rec e) e-0 e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) e-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul (rec e) e-0 e-0 l-2) (one)) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul e-0 e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec e) e-0 e-0 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4660)
  (parent 4563)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul (rec e) e-0 e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul (rec e) e-0 e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 l-2 e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e-0 e-0 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec e) e-0 e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4661)
  (parent 4563)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-2 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4662)
  (parent 4563)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e-0 (mul e w)) (l-2 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4663)
  (parent 4563)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (one)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0) (one)) (e-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-3 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (e-2 e-0) (w (mul eA (rec e) e-0)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0)))
      (send (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4664)
  (parent 4564)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) e-0 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) e-0) l-3))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) e-0 l-3) (one)) ((mul e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 11 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA e-0 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) e-0) l-3)) (send (cat (exp (gen) e-0) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) e-0 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4665)
  (parent 4564)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e-0 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) e-0 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-4))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 0) (8 0))
    ((11 1) (6 0)))
  (absent (l-3 l-4) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) e-0 l-4) (one)) ((mul e-0 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-3 eA e e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e-0 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e-0 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) l-4)) (send (cat (exp (gen) e-0) l-4)))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4)))
      (send (cat (exp (gen) l-3) (mul (rec l-3) eA (rec e) e-0 l-4))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4666)
  (parent 4564)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (eA (mul e (rec e-0) w)) (l-3 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec e-0) w))
    (mul e (rec e-0) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4667)
  (parent 4564)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-3 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4668)
  (parent 4564)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4669)
  (parent 4564)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 (one)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4670)
  (parent 4565)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul eA (rec e) l-1 l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-0 l-2) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul eA (rec e) l-1 l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul eA l-1 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec e) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4671)
  (parent 4565)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-1 l-3) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul eA l-2 l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4672)
  (parent 4565)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4673)
  (parent 4565)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4674)
  (parent 4565)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4675)
  (parent 4565)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-3 l-2) (w (mul eA (rec e) l-2)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2)))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4676)
  (parent 4566)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-2 l-2) (one)) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4677)
  (parent 4566)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4678)
  (parent 4566)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-1) w))
    (mul e (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4679)
  (parent 4566)
  (unrealized (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4680)
  (parent 4566)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (gen) (mul l-2 eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (gen) (mul l-2 eA (rec e))))
      (send (cat (gen) (mul l-2 eA (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4681)
  (parent 4567)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec e) l-2)))
      (send (cat (gen) (mul l-1 eA (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4682)
  (parent 4567)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (gen) (mul l-2 eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (gen) (mul l-2 eA (rec e) l-3)))
      (send (cat (gen) (mul l-2 eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4683)
  (parent 4567)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (gen) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (gen) (mul w l-2 eA)))
      (send (cat (gen) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4684)
  (parent 4567)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-2 (rec e))))
      (send (cat (gen) (mul w l-2 (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4685)
  (parent 4567)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e l-2 (rec e-0)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (contracted (e-1 e-0) (w (rec e-0))) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4686)
  (parent 4568)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-2)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4687)
  (parent 4568)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0) l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-0) (self l)
      (B l-1))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-0 self-0) (l self)
      (l-1 B)))
  (neq ((mul e l-2 (rec e-0) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0) l-3)))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4688)
  (parent 4568)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e-0 w) (l-2 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 w) w (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4689)
  (parent 4568)
  (unrealized (2 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-2)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (7 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (l-2 (mul e w)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 l-2) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen e l-0 l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-2)))
      (send (cat (exp (gen) e-0) (mul w l-2)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4690)
  (parent 4568)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4691)
  (parent 4569)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4692)
  (parent 4569)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self l-1) (self-0 l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self) (l-0 self-0)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec e) (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4693)
  (parent 4569)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (w expt)
    (l l-0 l-1 e l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul w e e)) (l-2 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2 eA) (one)) ((mul w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul w e e)))
      (send (cat (exp (gen) l-2) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4694)
  (parent 4569)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4695)
  (parent 4569)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-3 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (deflistener (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (one)) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-3 eA (rec e)) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-3 eA e e-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA (rec e) (rec e-0)))
      (e-2 e-0)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-3 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((recv (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4696)
  (parent 4570)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 rndx) (ll expt) (l-2 eA e e-0 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-1) (e e-0) (ll ll))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-2 l-3) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-3)))
    (eA (exp (gen) l-3)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-2 eA e e-0 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 11 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul l-2 eA (rec e) (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4697)
  (parent 4570)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 self-3 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (l-3 eA e e-0 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-3 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (deflistener (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 0) (7 0))
    ((11 1) (6 0)))
  (absent (l-3 l-4) (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-3 eA (rec e) l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-3 eA e e-0 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-3 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-3 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((recv (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4)))
      (send (cat (exp (gen) e-0) (mul l-3 eA (rec e) (rec e-0) l-4))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4698)
  (parent 4570)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (eA (mul e w e-0)) (l-3 (mul e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3 eA) (one)) ((mul e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w e-0)) (mul e w e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w e-0)))
      (send (cat (exp (gen) l-3) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 eA)))
      (send (cat (exp (gen) e-0) (mul w l-3 eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4699)
  (parent 4570)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-3 (mul (rec eA) w e))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e l-3 (rec e-0)) (one)) ((mul (rec eA) w e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e))
    (mul (rec eA) w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-3 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4700)
  (parent 4570)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-absence e-0 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4701)
  (parent 4570)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 (one)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4702)
  (parent 4571)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 eA e l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l-1)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-1 l-2)) (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-0 l-2) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-1 l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-0) (B l-1) (self l))
    ("privdh-of" (l-2 self-0) (l-0 self-1) (l-1 B) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul l-0 eA l-2))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-1 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-1 l-2))
            (exp (gen) (mul l-0 eA l-2))))) (send n))
    ((recv (cat (exp (gen) (mul l-1 l-2)) l))
      (send (cat (exp (gen) (mul l-1 l-2)) l)))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec e) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 4703)
  (parent 4571)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-1 l-3) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-2))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (B l-2) (self-0 l-0)
      (self l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 B) (l-0 self-0)
      (l self)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul l-1 eA l-3))))) (recv n))
    ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2))
            (exp (gen) (mul l-1 eA l-3))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4704)
  (parent 4571)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4705)
  (parent 4571)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4706)
  (parent 4571)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA e rndx)
    (w expt) (l-1 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-2)
    (ee (mul (rec eA) e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-2))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-2)) (ee (mul w l-1)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) e w)) (l-1 (mul (rec eA) e w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-2)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (self-1 l-1) (B l-2) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-2 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1) (one)) ((mul (rec eA) e w l-1) (one)))
  (non-orig (privk B) (privk self-0) l-2)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA e l-1 l-2)
  (uniq-orig n)
  (operation nonce-test (added-absence l-2 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-2) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-2) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e w l-1)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4707)
  (parent 4571)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-2) (self l-0) (B l))
    ("privdh-of" (l-1 self-1) (l-2 self-0) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-2)))
      (l-3 l-2)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4708)
  (parent 4572)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l-0 self) (l B)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 10 5 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4709)
  (parent 4572)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-2)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-2)) l-0))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-2))) (eA (exp (gen) l-2))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-1) (self-0 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-1 self-1) (l-2 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-2)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-2)) l-0))
      (send (cat (exp (gen) (mul l l-2)) l-0)))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4710)
  (parent 4572)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-1)) (mul e w l-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4711)
  (parent 4572)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4712)
  (parent 4572)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e) (rec l-3)))
      (l-4 l-3)) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4713)
  (parent 4573)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-2) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 11 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec e) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (send l-1)))
  (label 4714)
  (parent 4573)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e) l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 11 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e))))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4715)
  (parent 4573)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 eA e l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (deflistener (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 0) (7 0))
    ((11 1) (6 0)))
  (absent (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e) l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((recv (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4)))
      (send (cat (exp (gen) l-3) (mul l-2 eA (rec e) (rec l-3) l-4))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4716)
  (parent 4573)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (eA (mul e w l-2)) (l-3 (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 eA) (one)) ((mul e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w l-2)) (mul e w l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w l-2)))
      (send (cat (exp (gen) l-3) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 eA)))
      (send (cat (exp (gen) l-2) (mul w l-3 eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4717)
  (parent 4573)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e (mul (rec eA) w l-2)) (l-3 (mul (rec eA) w l-2))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 (rec e)) (one)) ((mul (rec eA) w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w l-2))
    (mul (rec eA) w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-3 (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4718)
  (parent 4573)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4719)
  (parent 4573)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-3))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (one)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3) (one)) (l-3 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (l-4 l-3) (w (mul eA (rec e) l-3)))
    (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-3))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (one)))
      (send (cat (exp (gen) l-3) (one))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4720)
  (parent 4574)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul eA (rec e) l-2 l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-2) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-1) (self-1 l-2) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-1 self-2) (l-2 self-1) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-2 l-3) (one)) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 11 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-2 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul eA l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec e) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-2 (privk self-2)))
      (send l-1)))
  (label 4721)
  (parent 4574)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 0) (7 0)) ((10 1) (6 0)))
  (absent (l-2 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul eA (rec e) l-3 l-3) (one)) ((mul l-3 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 11 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-3)) (send (cat (exp (gen) l-3) l-3)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4722)
  (parent 4574)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 eA e l-3 l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-3 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul eA (rec e) l-3 l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) l-4))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 0) (8 0))
    ((11 1) (6 0)))
  (absent (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-2) (self-1 l-3) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-2 self-2) (l-3 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul eA (rec e) l-3 l-4) (one)) ((mul l-3 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3 l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-3 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-3 l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-3 l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) l-4)) (send (cat (exp (gen) l-3) l-4)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec e) l-3 l-4))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4723)
  (parent 4574)
  (unrealized (2 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (eA (mul e (rec l-2) w)) (l-3 (mul e (rec l-2) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e (rec l-2) w))
    (mul e (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4724)
  (parent 4574)
  (unrealized (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (e (mul (rec eA) (rec l-2) w))
    (l-3 (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l-2) w))
    (mul (rec eA) (rec l-2) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4725)
  (parent 4574)
  (unrealized (0 3) (2 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-absence l-3 (mul (rec eA) e w))
    (mul (rec eA) e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4726)
  (parent 4574)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4727)
  (parent 4577)
  (unrealized (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4728)
  (parent 4578)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-2 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4729)
  (parent 4582)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e-0 (mul e w)) (l-2 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0 e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4730)
  (parent 4583)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((1 0) (0 1)) ((1 0) (3 0)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((2 0) (3 0)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 4731)
  (parent 4584)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e (rec e-0) w)) (l-3 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-3) eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4732)
  (parent 4588)
  (unrealized (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-3 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-3) (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4733)
  (parent 4589)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((6 2) (10 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-3) e-0)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4734)
  (parent 4590)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4735)
  (parent 4594)
  (unrealized (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4736)
  (parent 4595)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4737)
  (parent 4600)
  (unrealized (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4738)
  (parent 4601)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (gen) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (gen) (mul w l-2 eA)))
      (send (cat (gen) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4739)
  (parent 4605)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-2 (rec e))))
      (send (cat (gen) (mul w l-2 (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4740)
  (parent 4606)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e l-2 (rec e-0)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-2 (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4741)
  (parent 4607)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (l-1 l-2) (e (exp (gen) l-2)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-2))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4742)
  (parent 4608)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e-0 w) (l-2 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4743)
  (parent 4610)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (w expt)
    (l l-0 l-1 e l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul w e e)) (l-2 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2 eA) (one)) ((mul w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-0 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul w e e)))
      (send (cat (exp (gen) l-2) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4744)
  (parent 4615)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen eA l l-0 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4745)
  (parent 4616)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (eA (mul e w e-0)) (l-3 (mul e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3 eA) (one)) ((mul e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-3 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w e-0)))
      (send (cat (exp (gen) l-3) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 eA)))
      (send (cat (exp (gen) e-0) (mul w l-3 eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4746)
  (parent 4620)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-3 (mul (rec eA) w e))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e l-3 (rec e-0)) (one)) ((mul (rec eA) w e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-3 (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-3 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4747)
  (parent 4621)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 2) (5 0)) ((7 2) (10 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-3 (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4748)
  (parent 4622)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4749)
  (parent 4626)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (6 0) (5 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4750)
  (parent 4627)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4751)
  (parent 4632)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4752)
  (parent 4633)
  (unrealized (0 3) (5 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (eA (mul e w l-2)) (l-3 (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 eA) (one)) ((mul e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-3 eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w l-2)))
      (send (cat (exp (gen) l-3) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 eA)))
      (send (cat (exp (gen) l-2) (mul w l-3 eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4753)
  (parent 4638)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (e (mul (rec eA) w l-2)) (l-3 (mul (rec eA) w l-2))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 (rec e)) (one)) ((mul (rec eA) w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-3 (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-3 (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4754)
  (parent 4639)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 0) (5 0)) ((7 0) (10 0)) ((8 1) (6 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (6 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-2 (rec l-3))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)) ((recv l-3) (send l-3)))
  (label 4755)
  (parent 4640)
  (unrealized (0 3) (5 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e (rec l-2) w)) (l-3 (mul e (rec l-2) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-3) eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4756)
  (parent 4645)
  (unrealized (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-2) w))
    (l-3 (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-3) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4757)
  (parent 4646)
  (unrealized (0 3) (5 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 0) (5 0)) ((6 0) (10 0)) ((7 1) (2 3)) ((8 1) (7 0))
    ((9 0) (5 0)) ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-2) l-3)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)) ((recv l-3) (send l-3)))
  (label 4758)
  (parent 4647)
  (seen 4821)
  (unrealized (0 3) (5 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4759)
  (parent 4648)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-2) (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4760)
  (parent 4648)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4761)
  (parent 4648)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 4762)
  (parent 4648)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4763)
  (parent 4649)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4764)
  (parent 4649)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4765)
  (parent 4649)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4766)
  (parent 4649)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4767)
  (parent 4650)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4768)
  (parent 4650)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4769)
  (parent 4650)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4770)
  (parent 4650)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4771)
  (parent 4651)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-1) (peer peer) (l l-2) (e e) (ll l)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 0) (0 0)) ((9 0) (2 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l-2)) (e (exp (gen) l))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 10 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4772)
  (parent 4651)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4773)
  (parent 4651)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4774)
  (parent 4651)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4775)
  (parent 4651)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4776)
  (parent 4652)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4777)
  (parent 4652)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4778)
  (parent 4652)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4779)
  (parent 4652)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l l-0 l-1 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e)) (one)) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4780)
  (parent 4653)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4781)
  (parent 4653)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4782)
  (parent 4653)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 4783)
  (parent 4653)
  (unrealized (0 3) (2 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 l-1 l-2 eA e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e)) (one)) (l-1 (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4784)
  (parent 4654)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 self-1 name)
    (l l-0 l-1 eA e l-2 rndx))
  (defstrand init 6 (n n) (self self-1) (peer B) (l l-2) (e eA) (ll l)
    (ee (mul l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-2)
    (ee (mul l-0 eA (rec e) l-2)))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 0) (0 0)) ((9 0) (2 1)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (e (exp (gen) l)) (e (exp (gen) l-2))
    (eA (exp (gen) l-2)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul l-0 eA (rec e) l-2) (one)) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (uniq-orig n)
  (operation nonce-test (displaced 2 10 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))))
      (send n))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec e) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 4785)
  (parent 4654)
  (unrealized (2 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 self-1 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-1 l-3) (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 4786)
  (parent 4654)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4787)
  (parent 4654)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 4788)
  (parent 4654)
  (unrealized (0 3) (3 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul e w))) (send (cat (gen) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4789)
  (parent 4657)
  (unrealized (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4790)
  (parent 4658)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul (rec e) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (l-2 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4791)
  (parent 4662)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e-0)
    (ll l-1) (ee (mul e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e-0 e-0)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e-0 (mul e w)) (l-2 (mul e w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e-0 e-0) (one)) ((mul e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-2) e-0 e-0)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w e-0 e-0))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) e-0) (mul e w)))
      (send (cat (exp (gen) e-0) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e-0 e-0))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4792)
  (parent 4663)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e e-0 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul e (rec e-0) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (11 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (eA (mul e (rec e-0) w)) (l-3 (mul e (rec e-0) w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-3) eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul e (rec e-0) w)))
      (send (cat (exp (gen) e-0) (mul e (rec e-0) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4793)
  (parent 4667)
  (unrealized (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (11 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (l-3 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e-0)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-3) (rec e-0))
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w (rec e-0))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e-0)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4794)
  (parent 4668)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e w)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((7 2) (11 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0) (one)) ((mul (rec eA) e w e-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w (rec l-3) e-0)
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w e-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w e-0)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e w)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e w))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) e-0)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) e-0))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4795)
  (parent 4669)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4796)
  (parent 4673)
  (unrealized (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l-0 l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4797)
  (parent 4674)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e l-1 rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul e (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e (rec l-1) w)) (l-2 (mul e (rec l-1) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul e (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul e (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4798)
  (parent 4679)
  (unrealized (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA l-1 rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-2) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4799)
  (parent 4680)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 e rndx)
    (w expt) (l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (gen) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (gen) (mul w l-2 eA)))
      (send (cat (gen) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4800)
  (parent 4684)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 l-1 eA rndx)
    (w expt) (l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-2 (rec e))))
      (send (cat (gen) (mul w l-2 (rec e))))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4801)
  (parent 4685)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul e l-2 (rec e-0)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-2 (rec e-0)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-2))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4802)
  (parent 4686)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name)
    (l l-0 e l-1 e-0 l-2 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-2) (e e) (ll l-0)
    (ee (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-2)) (ee (mul e l-1 (rec e-0) l-2)))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (l-1 l-2) (e (exp (gen) l-2)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-2))))
  (fn-of ("principal-of" (self-0 l-2) (self-1 l-1) (self l) (B l-0))
    ("privdh-of" (l-2 self-0) (l-1 self-1) (l self) (l-0 B)))
  (neq ((mul e l-1 (rec e-0) l-2) (one)) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 e-0 l-2)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul l-1 (rec e-0) l-2)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-2)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul e l-1 (rec e-0) l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul e l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec e-0) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec e-0) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e-0) (send e-0)))
  (label 4803)
  (parent 4687)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l rndx) (w expt)
    (l-0 l-1 e l-2 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e e) (ll l-1)
    (ee (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e-0)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w e l-2 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e-0))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e-0 w) (l-2 w) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) (mul (rec l) l-0 l-1))))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-0) (self l) (B l-1))
    ("privdh-of" (l-2 self-1) (l-0 self-0) (l self) (l-1 B)))
  (neq ((mul w e l-2 (rec e-0)) (one)) ((mul w l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 l-1 e l-2 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-2 (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-2 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e-0)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e-0) (send e-0)))
  (label 4804)
  (parent 4689)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (w expt)
    (l l-0 l-1 e l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w e l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul w e e)) (l-2 (mul w e e)) (e (exp (gen) l-1))
    (e (exp (gen) (mul l l-0 (rec l-1)))) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w e l-2 eA) (one)) ((mul w e e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-0 e l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0))
            (exp (gen) (mul w e e l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul w e e)))
      (send (cat (exp (gen) l-2) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA)))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4805)
  (parent 4694)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (eA rndx) (w expt)
    (l l-0 l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-1) (e e)
    (ll (mul l l-0 (rec l-1))) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self l-1) (self-0 l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self) (l-0 self-0) (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen eA l l-0 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4806)
  (parent 4695)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (e rndx) (w expt) (e-0 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w e-0 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e-0 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 eA)))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (11 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (eA (mul e w e-0)) (l-3 (mul e w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e-0 l-3 eA) (one)) ((mul e w e-0 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e e-0 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-3 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w e-0 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e-0 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w e-0 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w e-0)))
      (send (cat (exp (gen) l-3) (mul e w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 eA)))
      (send (cat (exp (gen) e-0) (mul w l-3 eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4807)
  (parent 4699)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA rndx) (w expt) (e l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e-0)
    (ll (mul l (rec l-0) l-1)) (ee (mul w e l-3 (rec e-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (11 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (e-0 (mul (rec eA) w e)) (l-3 (mul (rec eA) w e))
    (e (exp (gen) l-2)) (e (exp (gen) ll)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w e l-3 (rec e-0)) (one)) ((mul (rec eA) w e l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-3 (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e-0))
      (recv (exp (gen) (mul w e l-3 (rec e-0))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4808)
  (parent 4700)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 peer-0 self-2 name)
    (l l-0 l-1 l-2 rndx) (ll expt) (eA e rndx) (w expt) (l-3 e-0 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
  (defstrand resp 3 (self self-1) (peer peer-0) (l l-2) (e e-0) (ll ll))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e-0)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 2) (6 0)) ((8 2) (11 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (e-0 (mul (rec eA) e w)) (l-3 (mul (rec eA) e w))
    (e-0 (exp (gen) l-2)) (e-0 (exp (gen) ll)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-3 e-0)
  (uniq-orig n)
  (operation nonce-test (added-listener e-0) (mul w l-3 (rec e-0))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) e-0) (mul w l-3 (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w l-3 (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) ll) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e-0) (send e-0)))
  (label 4809)
  (parent 4701)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l))
    (e (exp (gen) (mul (rec l) l-0 l-1))) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4810)
  (parent 4705)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-0) (e eA) (ll l-1)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e)
    (ll (mul (rec l) l-0 l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l)) (e (exp (gen) (mul (rec l) l-0 l-1)))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (self-1 l-2) (B l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-2 self-1) (l-1 B) (l-0 self-0) (l self)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l-1)
  (precur (7 0) (6 0))
  (uniq-gen l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul (rec l) l-0 l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4811)
  (parent 4706)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 e rndx)
    (w expt) (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (10 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (eA (mul e w l-1)) (l-2 (mul e w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 eA) (one)) ((mul e w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-1 l-2 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w l-1)))
      (send (cat (exp (gen) l-2) (mul e w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4812)
  (parent 4711)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 name) (l l-0 eA rndx)
    (w expt) (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-1 l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (10 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)))
  (absent (e (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-2) (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-1 l-2 (rec e)) (one)) ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 l-2 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1 l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv e) (send e)))
  (label 4813)
  (parent 4712)
  (unrealized (0 3) (6 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul e w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 eA)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (11 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (eA (mul e w l-2)) (l-3 (mul e w l-2)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 eA) (one)) ((mul e w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-3 eA) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (recv n))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1))
            (exp (gen) (mul e w l-2 l-3 eA))))) (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul e w l-2)))
      (send (cat (exp (gen) l-3) (mul e w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 eA)))
      (send (cat (exp (gen) l-2) (mul w l-3 eA))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4814)
  (parent 4717)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 l-3 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
  (deflistener (cat (exp (gen) l-2) (mul w l-3 (rec e))))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (11 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (7 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (e (mul (rec eA) w l-2)) (l-3 (mul (rec eA) w l-2))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 l-3 (rec e)) (one)) ((mul (rec eA) w l-2 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-3 (rec e)) (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 l-3 (rec e))))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2 l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) w l-2)))
      (send (cat (exp (gen) l-3) (mul (rec eA) w l-2))))
    ((recv (cat (exp (gen) l-2) (mul w l-3 (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-3 (rec e)))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4815)
  (parent 4718)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 0) (6 0)) ((8 0) (11 0))
    ((9 1) (7 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w l-2 (rec l-3))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)) ((recv l-3) (send l-3)))
  (label 4816)
  (parent 4719)
  (unrealized (0 3) (6 0) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e l-2 rndx) (w expt) (l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (11 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (eA (mul e (rec l-2) w)) (l-3 (mul e (rec l-2) w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w eA) (one)) ((mul e w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w (rec l-3) eA) (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w eA)))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul e (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) eA))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4817)
  (parent 4724)
  (unrealized (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA l-2 rndx) (w expt) (l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self-1) (l l-2))
  (deflistener (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (11 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (8 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (e (mul (rec eA) (rec l-2) w))
    (l-3 (mul (rec eA) (rec l-2) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w (rec e)) (one)) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w (rec l-3) (rec e))
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e))))
      (recv (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) w))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-3) (mul w (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w (rec l-3) (rec e)))))
    ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4818)
  (parent 4725)
  (unrealized (0 3) (6 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-3)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul (rec eA) e w)))
  (defstrand ltx-gen 1 (self self-1) (l l-3))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (deflistener l-3)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (8 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 0) (6 0)) ((7 0) (11 0)) ((8 1) (2 3))
    ((9 1) (8 0)) ((10 0) (6 0)) ((10 1) (9 0)) ((11 1) (8 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-2) (self-1 l-3) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-2 self-2) (l-3 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-3) (one)) ((mul (rec eA) e w l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (8 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (added-listener l-3) (mul w (rec l-2) l-3)
    (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-3) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-3) (mul (rec eA) e w))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)) ((recv l-3) (send l-3)))
  (label 4819)
  (parent 4726)
  (seen 4836)
  (unrealized (0 3) (6 0) (11 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul eA w) (one)) ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization weakened ((1 0) (3 0)))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 4820)
  (parent 4731)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 7 11 ltx-gen 2) l-3 (10 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4821)
  (parent 4755)
  (unrealized (0 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4822)
  (parent 4761)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (self peer self-0 self-1 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer self) (l l) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 0))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0))
    ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-1 l-1) (self-0 l-0) (self l))
    ("privdh-of" (l-1 self-1) (l-0 self-0) (l self)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv e) (send e)))
  (label 4823)
  (parent 4762)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4824)
  (parent 4765)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 0)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4825)
  (parent 4766)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4826)
  (parent 4769)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self-0) (peer peer) (l l-1) (e e) (ll l)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 1)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4827)
  (parent 4770)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4828)
  (parent 4774)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l) (e eA) (ll l-0)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l))
  (defstrand resp 6 (n n) (self A) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 1)) ((2 0) (0 0)) ((2 0) (3 0)) ((3 2) (4 0))
    ((3 2) (10 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (B l-0) (A l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 B) (l A)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec l-2) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4829)
  (parent 4775)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4830)
  (parent 4778)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (2 1)) ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (3 0))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4831)
  (parent 4779)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l e rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 eA)))
  (deflistener (cat (exp (gen) l-0) (mul e w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul e w)) (l-1 (mul e w)) (l-0 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 eA) (one)) ((mul e w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 l-1 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-0)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-0 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul e w)))
      (send (cat (exp (gen) l-0) (mul e w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 4832)
  (parent 4782)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B peer self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-1)
    (ee (mul w l-0 (rec e))))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (2 0)) ((2 2) (3 0)) ((2 2) (9 0)) ((2 5) (0 5))
    ((3 1) (0 3)) ((4 1) (2 3)) ((5 1) (4 0)) ((6 0) (3 0))
    ((6 1) (5 0)) ((7 1) (4 0)) ((8 0) (0 0)) ((8 0) (2 1))
    ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-1) (self l-0) (B l))
    ("privdh-of" (l-1 self-0) (l-0 self) (l B)))
  (neq ((mul w l-0 (rec e)) (one)) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-0 (rec l-1) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
      (send n))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 4833)
  (parent 4783)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 e rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul e w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 eA)))
  (deflistener (cat (exp (gen) l-1) (mul e w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (5 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (eA (mul e w)) (l-2 (mul e w)) (l-1 (mul e w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 eA) (one)) ((mul e w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 l-2 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-1)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-0)) (exp (gen) (mul e w l-1 eA)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul e w)))
      (send (cat (exp (gen) l-1) (mul e w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 4834)
  (parent 4787)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (A B peer self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 e rndx))
  (defstrand init 6 (n n) (self A) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul (rec eA) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (defstrand resp 6 (n n) (self B) (peer peer) (l l) (e e) (ll l-0)
    (ee (mul w l-1 (rec e))))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((0 4) (3 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 0) (0 0)) ((2 0) (3 1)) ((3 2) (4 0))
    ((3 2) (10 0)) ((3 5) (0 5)) ((4 1) (0 3)) ((5 1) (3 3))
    ((6 1) (5 0)) ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0))
    ((9 0) (4 0)) ((9 1) (8 0)) ((10 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self-0 l-2) (self l-1) (A l-0) (B l))
    ("privdh-of" (l-2 self-0) (l-1 self) (l-0 A) (l B)))
  (neq ((mul w l-1 (rec e)) (one)) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk A) (privk B) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-1 (rec l-2) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A))))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
      (send n))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 4835)
  (parent 4788)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0)))
  (absent (l-3 (mul (rec eA) e w)) (l-2 (mul (rec eA) e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2) (one)) ((mul (rec eA) e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA e l-2 l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 8 12 ltx-gen 2) l-3 (11 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) e w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4836)
  (parent 4816)
  (unrealized (0 3) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand init 6 (n n) (self self) (peer B) (l l-0) (e eA) (ll l)
    (ee (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((1 0) (0 1)) ((2 0) (0 0)) ((2 1) (0 5)))
  (absent (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (self l-0) (B l))
    ("privdh-of" (l-0 self) (l B)))
  (neq ((mul w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (uniq-orig n)
  (operation generalization forgot ((neq ((mul eA w) (one)))))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e)))
      (send
        (enc n (hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) e) w))))
  (label 4837)
  (parent 4820)
  (seen 2880)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 l-3 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)))
  (absent (l-3 (one)) (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 l-3 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4838)
  (parent 4821)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 11 4 ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-3))
      (send (cat (exp (gen) (mul l (rec l-0))) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4839)
  (parent 4821)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 l-3 eA e l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)) ((11 0) (6 0))
    ((11 1) (5 0)))
  (absent (l-3 l-4) (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e) l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 l-2 l-3 eA e l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4)))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4840)
  (parent 4821)
  (unrealized (2 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)))
  (absent (eA (mul e w)) (l-3 (mul e w)) (l-2 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4841)
  (parent 4821)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)))
  (absent (e (mul (rec eA) w)) (l-3 (mul (rec eA) w))
    (l-2 (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4842)
  (parent 4821)
  (unrealized (0 3) (2 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 l-3 eA e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0)))
  (absent (l-3 (one)) (l-2 (one)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul l-2 eA (rec e)) (one)) (l-2 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 l-3 eA e)
  (uniq-orig n)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul eA (rec e)))) (one) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4843)
  (parent 4836)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 l-2 eA e l-3 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-3) (e eA) (ll l)
    (ee (mul l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-3)) (ee (mul l-1 eA (rec e) l-3)))
  (deflistener (cat (exp (gen) (mul l l-3)) l-0))
  (deflistener (cat (exp (gen) l) l-3))
  (defstrand ltx-gen 2 (self self-0) (l l-3))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0)))
  (absent (l-2 l-3) (l-1 l-3) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-3))) (eA (exp (gen) l-3))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-0 l-3) (self-2 l-2) (self-1 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-0) (l-2 self-2) (l-1 self-1) (l-0 self)
      (l B)))
  (neq ((mul l-1 eA (rec e) l-3) (one)) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 eA e l-3)
  (uniq-orig n)
  (operation nonce-test (displaced 12 5 ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-3)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eA (rec e) l-3)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-3)) (exp (gen) (mul l-1 eA l-3)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-3)) l-0))
      (send (cat (exp (gen) (mul l l-3)) l-0)))
    ((recv (cat (exp (gen) l) l-3)) (send (cat (exp (gen) l) l-3)))
    ((send (enc "cert" (exp (gen) l-3) self-0 (privk self-0)))
      (send l-3))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec e) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-2 (privk self-2)))
      (send l-2)))
  (label 4844)
  (parent 4836)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 self-3 name)
    (l l-0 l-1 l-2 l-3 eA e l-4 rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul l-2 l-4)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul l-2 eA (rec e) l-4)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-4))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (defstrand ltx-gen 2 (self self-3) (l l-4))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0))
    ((12 0) (7 0)) ((12 1) (6 0)))
  (absent (l-3 l-4) (l-2 l-4) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-3 l-4) (self-2 l-3) (self-1 l-2) (self-0 l-1)
      (self l-0) (B l))
    ("privdh-of" (l-4 self-3) (l-3 self-2) (l-2 self-1) (l-1 self-0)
      (l-0 self) (l B)))
  (neq ((mul l-2 eA (rec e) l-4) (one)) ((mul l-2 l-4) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 l-2 l-3 eA e l-4)
  (uniq-orig n)
  (operation nonce-test (added-strand ltx-gen 2) l-4 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-4)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-2 eA (rec e) l-4)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-2 eA l-4)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) l-4)) (send (cat (exp (gen) l-2) l-4)))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4)))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3) eA (rec e) l-4))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3))
    ((send (enc "cert" (exp (gen) l-4) self-3 (privk self-3)))
      (send l-4)))
  (label 4845)
  (parent 4836)
  (unrealized (2 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0)))
  (absent (eA (mul e w)) (l-3 (mul e w)) (l-2 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-absence eA (mul e w)) (mul e w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4846)
  (parent 4836)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0)))
  (absent (e (mul (rec eA) w)) (l-3 (mul (rec eA) w))
    (l-2 (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)))
  (label 4847)
  (parent 4836)
  (unrealized (0 3) (2 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (11 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)) ((11 1) (6 0)))
  (absent (eA (mul e w)) (l-3 (mul e w)) (l-2 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec l-3) eA)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4848)
  (parent 4841)
  (unrealized (0 3) (5 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (3 0)) ((2 2) (5 0)) ((2 2) (11 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 0) (0 0)) ((4 1) (3 0)) ((5 1) (0 3))
    ((6 1) (2 3)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (7 0))
    ((9 1) (6 0)) ((10 0) (5 0)) ((10 1) (9 0)) ((11 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-3 (mul (rec eA) w))
    (l-2 (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (6 0) (5 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec l-3) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4849)
  (parent 4842)
  (unrealized (0 3) (5 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 e rndx) (w expt) (l-2 l-3 eA rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul e w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 eA)))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul e w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (12 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0))
    ((12 1) (7 0)))
  (absent (eA (mul e w)) (l-3 (mul e w)) (l-2 (mul e w))
    (e (exp (gen) l-0)) (e (exp (gen) (mul l (rec l-0) l-1)))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 eA) (one)) ((mul e w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 e l-2 l-3 eA)
  (uniq-orig n)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec l-3) eA)
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e w l-2)))
      (send
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 eA)))
      (recv
        (enc n
          (hash (exp (gen) (mul l l-1)) (exp (gen) (mul e w l-2 eA)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul e w)))
      (send (cat (exp (gen) l-2) (mul e w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA)))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv eA) (send eA)))
  (label 4850)
  (parent 4846)
  (unrealized (0 3) (6 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton eadh-um
  (vars (n data) (B self peer self-0 self-1 self-2 name)
    (l l-0 l-1 eA rndx) (w expt) (l-2 l-3 e rndx))
  (defstrand init 6 (n n) (self self-0) (peer B) (l l-1) (e eA) (ll l)
    (ee (mul (rec eA) w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand resp 6 (n n) (self self) (peer peer) (l l-0) (e e)
    (ll (mul l (rec l-0) l-1)) (ee (mul w l-2 (rec e))))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (deflistener l-3)
  (defstrand ltx-gen 2 (self self-2) (l l-3))
  (deflistener e)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((0 4) (2 4)) ((1 0) (0 1))
    ((1 0) (4 0)) ((2 2) (6 0)) ((2 2) (12 0)) ((2 5) (0 5))
    ((3 1) (2 1)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 1) (4 0))
    ((6 1) (0 3)) ((7 1) (2 3)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (8 0)) ((10 1) (7 0)) ((11 0) (6 0)) ((11 1) (10 0))
    ((12 1) (7 0)))
  (absent (e (mul (rec eA) w)) (l-3 (mul (rec eA) w))
    (l-2 (mul (rec eA) w)) (e (exp (gen) l-0))
    (e (exp (gen) (mul l (rec l-0) l-1))) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of
    ("principal-of" (self-2 l-3) (self-1 l-2) (self-0 l-1) (self l-0)
      (B l))
    ("privdh-of" (l-3 self-2) (l-2 self-1) (l-1 self-0) (l-0 self)
      (l B)))
  (neq ((mul w l-2 (rec e)) (one)) ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (7 0) (6 0))
  (uniq-gen l l-1 eA l-2 l-3 e)
  (uniq-orig n)
  (operation nonce-test (added-listener e) (mul w l-2 (rec l-3) (rec e))
    (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (recv n)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv
        (enc "cert" (exp (gen) (mul l (rec l-0) l-1)) peer
          (privk peer))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-2 (rec e))))
      (recv
        (enc n (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-2)))))
      (send n))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e))))
      (send (cat (exp (gen) l-3) (mul w l-2 (rec l-3) (rec e)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)) ((recv l-3) (send l-3))
    ((send (enc "cert" (exp (gen) l-3) self-2 (privk self-2)))
      (send l-3)) ((recv e) (send e)))
  (label 4851)
  (parent 4847)
  (unrealized (0 3) (6 0) (12 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")
